@@ -1,3 +1,5 @@
+import sys
+
 from .account import Account, Identity
 from .attachments import FileAttachment, ItemAttachment
 from .autodiscover import discover
@@ -8,15 +10,15 @@ from .ewsdatetime import EWSDate, EWSDateTime, EWSTimeZone, UTC, UTC_NOW
 from .extended_properties import ExtendedProperty
 from .folders import Folder, RootOfHierarchy, FolderCollection, SHALLOW, DEEP
 from .items import AcceptItem, TentativelyAcceptItem, DeclineItem, CalendarItem, CancelCalendarItem, Contact, \
-    DistributionList, Message, PostItem, Task
+    DistributionList, Message, PostItem, Task, ForwardItem, ReplyToItem, ReplyAllToItem
 from .properties import Body, HTMLBody, ItemId, Mailbox, Attendee, Room, RoomList, UID, DLMailbox
 from .protocol import FaultTolerance, FailFast, BaseProtocol, NoVerifyHTTPAdapter, TLSClientAuth
-from .settings import OofSettings
 from .restriction import Q
+from .settings import OofSettings
 from .transport import BASIC, DIGEST, NTLM, GSSAPI, SSPI, OAUTH2, CBA
 from .version import Build, Version
 
-__version__ = '3.2.0'
+__version__ = '4.6.1'
 
 __all__ = [
     '__version__',
@@ -29,7 +31,7 @@ __all__ = [
     'ExtendedProperty',
     'Folder', 'RootOfHierarchy', 'FolderCollection', 'SHALLOW', 'DEEP',
     'AcceptItem', 'TentativelyAcceptItem', 'DeclineItem', 'CalendarItem', 'CancelCalendarItem', 'Contact',
-    'DistributionList', 'Message', 'PostItem', 'Task',
+    'DistributionList', 'Message', 'PostItem', 'Task', 'ForwardItem', 'ReplyToItem', 'ReplyAllToItem',
     'ItemId', 'Mailbox', 'DLMailbox', 'Attendee', 'Room', 'RoomList', 'Body', 'HTMLBody', 'UID',
     'FailFast', 'FaultTolerance', 'BaseProtocol', 'NoVerifyHTTPAdapter', 'TLSClientAuth',
     'OofSettings',
@@ -42,6 +44,11 @@ __all__ = [
 import requests.utils
 BaseProtocol.USERAGENT = "%s/%s (%s)" % (__name__, __version__, requests.utils.default_user_agent())
 
+# Support fromisoformat() in Python < 3.7
+if sys.version_info[:2] < (3, 7):
+    from backports.datetime_fromisoformat import MonkeyPatch
+    MonkeyPatch.patch_fromisoformat()
+
 
 def close_connections():
     from .autodiscover import close_connections as close_autodiscover_connections
