@@ -16,27 +16,28 @@ from .folders import Folder, AdminAuditLogs, ArchiveDeletedItems, ArchiveInbox,
     Notes, Outbox, PeopleConnect, PublicFoldersRoot, QuickContacts, RecipientCache, RecoverableItemsDeletions, \
     RecoverableItemsPurges, RecoverableItemsRoot, RecoverableItemsVersions, Root, SearchFolders, SentItems, \
     ServerFailures, SyncIssues, Tasks, ToDoSearch, VoiceMail
-from .items import Item, BulkCreateResult, HARD_DELETE, AUTO_RESOLVE, SEND_TO_NONE, SAVE_ONLY, ALL_OCCURRENCIES, ID_ONLY
+from .items import HARD_DELETE, AUTO_RESOLVE, SEND_TO_NONE, SAVE_ONLY, ALL_OCCURRENCIES, ID_ONLY
 from .properties import Mailbox, SendingAs
 from .protocol import Protocol
 from .queryset import QuerySet
 from .services import ExportItems, UploadItems, GetItem, CreateItem, UpdateItem, DeleteItem, MoveItem, SendItem, \
-    CopyItem, GetUserOofSettings, SetUserOofSettings, GetMailTips, ArchiveItem, GetDelegate
+    CopyItem, GetUserOofSettings, SetUserOofSettings, GetMailTips, ArchiveItem, GetDelegate, MarkAsJunk, GetPersona
 from .util import get_domain, peek
 
 log = getLogger(__name__)
 
 
 class Identity:
-    """Contains information that uniquely identifies an account. Currently only used for SOAP impersonation headers.
-    """
+    """Contains information that uniquely identifies an account. Currently only used for SOAP impersonation headers."""
+
     def __init__(self, primary_smtp_address=None, smtp_address=None, upn=None, sid=None):
         """
-        :param primary_smtp_address: The primary email address associated with the account
-        :param smtp_address: The (non-)primary email address associated with the account
-        :param: upn: The User Principal Name (UPN) of this account
-        :param: sid: The security identifier (SID) of this account, in security descriptor definition language (SDDL)
-        form.
+
+        :param primary_smtp_address: The primary email address associated with the account (Default value = None)
+        :param smtp_address: The (non-)primary email address associated with the account (Default value = None)
+        :param upn: (Default value = None)
+        :param sid: (Default value = None)
+        :return:
         """
         self.primary_smtp_address = primary_smtp_address
         self.smtp_address = smtp_address
@@ -44,7 +45,7 @@ class Identity:
         self.sid = sid
 
     def __eq__(self, other):
-        for k in self.__dict__.keys():
+        for k in self.__dict__:
             if getattr(self, k) != getattr(other, k):
                 return False
         return True
@@ -57,21 +58,25 @@ class Identity:
 
 
 class Account:
-    """Models an Exchange server user account.
-    """
+    """Models an Exchange server user account."""
+
     def __init__(self, primary_smtp_address, fullname=None, access_type=None, autodiscover=False, credentials=None,
                  config=None, locale=None, default_timezone=None):
         """
+
         :param primary_smtp_address: The primary email address associated with the account on the Exchange server
-        :param fullname: The full name of the account. Optional.
+        :param fullname: The full name of the account. Optional. (Default value = None)
         :param access_type: The access type granted to 'credentials' for this account. Valid options are 'delegate'
-        (default) and 'impersonation'.
+            and 'impersonation'. 'delegate' is default if 'credentials' is set. Otherwise, 'impersonation' is default.
         :param autodiscover: Whether to look up the EWS endpoint automatically using the autodiscover protocol.
-        :param credentials: A Credentials object containing valid credentials for this account.
+            (Default value = False)
+        :param credentials: A Credentials object containing valid credentials for this account. (Default value = None)
         :param config: A Configuration object containing EWS endpoint information. Required if autodiscover is disabled
+            (Default value = None)
         :param locale: The locale of the user, e.g. 'en_US'. Defaults to the locale of the host, if available.
         :param default_timezone: EWS may return some datetime values without timezone information. In this case, we will
-        assume values to be in the provided timezone. Defaults to the timezone of the host.
+            assume values to be in the provided timezone. Defaults to the timezone of the host.
+        :return:
         """
         if '@' not in primary_smtp_address:
             raise ValueError("primary_smtp_address %r is not an email address" % primary_smtp_address)
@@ -84,19 +89,23 @@ class Account:
             self.locale = locale or getlocale()[0] or None  # get_locale() might not be able to determine the locale
         except ValueError as e:
             # getlocale() may throw ValueError if it fails to parse the system locale
-            log.warning('Failed to get locale (%s)' % e)
+            log.warning('Failed to get locale (%s)', e)
             self.locale = None
         if not isinstance(self.locale, (type(None), str)):
             raise ValueError("Expected 'locale' to be a string, got %r" % self.locale)
-        try:
-            self.default_timezone = default_timezone or EWSTimeZone.localzone()
-        except (ValueError, UnknownTimeZone) as e:
-            # There is no translation from local timezone name to Windows timezone name, or e failed to find the
-            # local timezone.
-            log.warning('%s. Fallback to UTC', e.args[0])
-            self.default_timezone = UTC
-        if not isinstance(self.default_timezone, EWSTimeZone):
-            raise ValueError("Expected 'default_timezone' to be an EWSTimeZone, got %r" % self.default_timezone)
+        if default_timezone:
+            try:
+                self.default_timezone = EWSTimeZone.from_timezone(default_timezone)
+            except TypeError:
+                raise ValueError("Expected 'default_timezone' to be an EWSTimeZone, got %r" % default_timezone)
+        else:
+            try:
+                self.default_timezone = EWSTimeZone.localzone()
+            except (ValueError, UnknownTimeZone) as e:
+                # There is no translation from local timezone name to Windows timezone name, or e failed to find the
+                # local timezone.
+                log.warning('%s. Fallback to UTC', e.args[0])
+                self.default_timezone = UTC
         if not isinstance(config, (Configuration, type(None))):
             raise ValueError("Expected 'config' to be a Configuration, got %r" % config)
         if autodiscover:
@@ -113,7 +122,6 @@ class Account:
         else:
             if not config:
                 raise AttributeError('non-autodiscover requires a config')
-            primary_smtp_address = primary_smtp_address
             self.ad_response = None
             self.protocol = Protocol(config=config)
 
@@ -311,7 +319,7 @@ class Account:
         # supports the 'del self.oof_settings' syntax to invalidate the cache, but does not support custom setter
         # methods. Having a non-cached service call here goes against the assumption that properties are cheap, but the
         # alternative is to create get_oof_settings() and set_oof_settings(), and that's just too Java-ish for my taste.
-        return GetUserOofSettings(account=self).call(
+        return GetUserOofSettings(account=self).get(
             mailbox=Mailbox(email_address=self.primary_smtp_address),
         )
 
@@ -323,65 +331,68 @@ class Account:
         )
 
     def _consume_item_service(self, service_cls, items, chunk_size, kwargs):
-        # 'items' could be an unevaluated QuerySet, e.g. if we ended up here via `some_folder.filter(...).delete()`. In
-        # that case, we want to use its iterator. Otherwise, peek() will start a count() which is wasteful because we
-        # need the item IDs immediately afterwards. iterator() will only do the bare minimum.
         if isinstance(items, QuerySet):
-            items = items.iterator()
+            # We just want an iterator over the results
+            items = iter(items)
         is_empty, items = peek(items)
         if is_empty:
             # We accept generators, so it's not always convenient for caller to know up-front if 'ids' is empty. Allow
             # empty 'ids' and return early.
             return
         kwargs['items'] = items
-        for i in service_cls(account=self, chunk_size=chunk_size).call(**kwargs):
-            yield i
+        yield from service_cls(account=self, chunk_size=chunk_size).call(**kwargs)
 
     def export(self, items, chunk_size=None):
-        """Return export strings of the given items
+        """Return export strings of the given items.
 
         :param items: An iterable containing the Items we want to export
-        :param chunk_size: The number of items to send to the server in a single request
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
 
-        :return A list of strings, the exported representation of the object
+        :return: A list of strings, the exported representation of the object
         """
         return list(
-            self._consume_item_service(service_cls=ExportItems, items=items, chunk_size=chunk_size, kwargs=dict())
+            self._consume_item_service(service_cls=ExportItems, items=items, chunk_size=chunk_size, kwargs={})
         )
 
     def upload(self, data, chunk_size=None):
-        """Adds objects retrieved from export into the given folders
+        """Upload objects retrieved from an export to the given folders.
 
-        :param data: An iterable of tuples containing the folder we want to upload the data to and the
-            string outputs of exports.
-        :param chunk_size: The number of items to send to the server in a single request
+        :param data: An iterable of tuples containing the folder we want to upload the data to and the string outputs of
+            exports. If you want to update items instead of create, the data must be a tuple of
+            (ItemId, is_associated, data) values.
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
 
-        :return A list of tuples with the new ids and changekeys
+        :return: A list of tuples with the new ids and changekeys
 
-        Example:
-        account.upload([(account.inbox, "AABBCC..."),
-                        (account.inbox, "XXYYZZ..."),
-                        (account.calendar, "ABCXYZ...")])
-        -> [("idA", "changekey"), ("idB", "changekey"), ("idC", "changekey")]
+          Example:
+          account.upload([
+              (account.inbox, "AABBCC..."),
+              (account.inbox, (ItemId('AA', 'BB'), False, "XXYYZZ...")),
+              (account.inbox, (('CC', 'DD'), None, "XXYYZZ...")),
+              (account.calendar, "ABCXYZ..."),
+          ])
+          -> [("idA", "changekey"), ("idB", "changekey"), ("idC", "changekey")]
         """
+        items = ((f, (None, False, d) if isinstance(d, str) else d) for f, d in data)
         return list(
-            self._consume_item_service(service_cls=UploadItems, items=data, chunk_size=chunk_size, kwargs=dict())
+            self._consume_item_service(service_cls=UploadItems, items=items, chunk_size=chunk_size, kwargs={})
         )
 
     def bulk_create(self, folder, items, message_disposition=SAVE_ONLY, send_meeting_invitations=SEND_TO_NONE,
                     chunk_size=None):
-        """Creates new items in 'folder'
+        """Create new items in 'folder'.
 
         :param folder: the folder to create the items in
         :param items: an iterable of Item objects
         :param message_disposition: only applicable to Message items. Possible values are specified in
-               MESSAGE_DISPOSITION_CHOICES
+            MESSAGE_DISPOSITION_CHOICES (Default value = SAVE_ONLY)
         :param send_meeting_invitations: only applicable to CalendarItem items. Possible values are specified in
-               SEND_MEETING_INVITATIONS_CHOICES
-        :param chunk_size: The number of items to send to the server in a single request
+            SEND_MEETING_INVITATIONS_CHOICES (Default value = SEND_TO_NONE)
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
+
         :return: a list of either BulkCreateResult or exception instances in the same order as the input. The returned
-                 BulkCreateResult objects are normal Item objects except they only contain the 'id' and 'changekey'
-                 of the created item, and the 'id' of any attachments that were also created.
+          BulkCreateResult objects are normal Item objects except they only contain the 'id' and 'changekey'
+          of the created item, and the 'id' of any attachments that were also created.
         """
         if isinstance(items, QuerySet):
             # bulk_create() on a queryset does not make sense because it returns items that have already been created
@@ -393,31 +404,27 @@ class Account:
             message_disposition,
             send_meeting_invitations,
         )
-        return list(
-            i if isinstance(i, Exception)
-            else BulkCreateResult.from_xml(elem=i, account=self)
-            for i in self._consume_item_service(service_cls=CreateItem, items=items, chunk_size=chunk_size, kwargs=dict(
-                folder=folder,
-                message_disposition=message_disposition,
-                send_meeting_invitations=send_meeting_invitations,
-            ))
-        )
+        return list(self._consume_item_service(service_cls=CreateItem, items=items, chunk_size=chunk_size, kwargs=dict(
+            folder=folder,
+            message_disposition=message_disposition,
+            send_meeting_invitations=send_meeting_invitations,
+        )))
 
     def bulk_update(self, items, conflict_resolution=AUTO_RESOLVE, message_disposition=SAVE_ONLY,
                     send_meeting_invitations_or_cancellations=SEND_TO_NONE, suppress_read_receipts=True,
                     chunk_size=None):
-        """
-        Bulk updates existing items
+        """Bulk update existing items.
 
         :param items: a list of (Item, fieldnames) tuples, where 'Item' is an Item object, and 'fieldnames' is a list
-                      containing the attributes on this Item object that we want to be updated.
+            containing the attributes on this Item object that we want to be updated.
         :param conflict_resolution: Possible values are specified in CONFLICT_RESOLUTION_CHOICES
+            (Default value = AUTO_RESOLVE)
         :param message_disposition: only applicable to Message items. Possible values are specified in
-               MESSAGE_DISPOSITION_CHOICES
+            MESSAGE_DISPOSITION_CHOICES (Default value = SAVE_ONLY)
         :param send_meeting_invitations_or_cancellations: only applicable to CalendarItem items. Possible values are
-               specified in SEND_MEETING_INVITATIONS_AND_CANCELLATIONS_CHOICES
-        :param suppress_read_receipts: nly supported from Exchange 2013. True or False
-        :param chunk_size: The number of items to send to the server in a single request
+            specified in SEND_MEETING_INVITATIONS_AND_CANCELLATIONS_CHOICES (Default value = SEND_TO_NONE)
+        :param suppress_read_receipts: nly supported from Exchange 2013. True or False (Default value = True)
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
 
         :return: a list of either (id, changekey) tuples or exception instances, in the same order as the input
         """
@@ -433,29 +440,26 @@ class Account:
             message_disposition,
             send_meeting_invitations_or_cancellations,
         )
-        return list(
-            i if isinstance(i, Exception) else Item.id_from_xml(i)
-            for i in self._consume_item_service(service_cls=UpdateItem, items=items, chunk_size=chunk_size, kwargs=dict(
-                conflict_resolution=conflict_resolution,
-                message_disposition=message_disposition,
-                send_meeting_invitations_or_cancellations=send_meeting_invitations_or_cancellations,
-                suppress_read_receipts=suppress_read_receipts,
-            ))
-        )
+        return list(self._consume_item_service(service_cls=UpdateItem, items=items, chunk_size=chunk_size, kwargs=dict(
+            conflict_resolution=conflict_resolution,
+            message_disposition=message_disposition,
+            send_meeting_invitations_or_cancellations=send_meeting_invitations_or_cancellations,
+            suppress_read_receipts=suppress_read_receipts,
+        )))
 
     def bulk_delete(self, ids, delete_type=HARD_DELETE, send_meeting_cancellations=SEND_TO_NONE,
                     affected_task_occurrences=ALL_OCCURRENCIES, suppress_read_receipts=True, chunk_size=None):
-        """
-        Bulk deletes items.
+        """Bulk delete items.
 
         :param ids: an iterable of either (id, changekey) tuples or Item objects.
         :param delete_type: the type of delete to perform. Possible values are specified in DELETE_TYPE_CHOICES
+            (Default value = HARD_DELETE)
         :param send_meeting_cancellations: only applicable to CalendarItem. Possible values are specified in
-               SEND_MEETING_CANCELLATIONS_CHOICES.
+            SEND_MEETING_CANCELLATIONS_CHOICES. (Default value = SEND_TO_NONE)
         :param affected_task_occurrences: only applicable for recurring Task items. Possible values are specified in
-               AFFECTED_TASK_OCCURRENCES_CHOICES.
-        :param suppress_read_receipts: only supported from Exchange 2013. True or False.
-        :param chunk_size: The number of items to send to the server in a single request
+            AFFECTED_TASK_OCCURRENCES_CHOICES. (Default value = ALL_OCCURRENCIES)
+        :param suppress_read_receipts: only supported from Exchange 2013. True or False. (Default value = True)
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
 
         :return: a list of either True or exception instances, in the same order as the input
         """
@@ -476,12 +480,13 @@ class Account:
         )
 
     def bulk_send(self, ids, save_copy=True, copy_to_folder=None, chunk_size=None):
-        """ Send existing draft messages. If requested, save a copy in 'copy_to_folder'
+        """Send existing draft messages. If requested, save a copy in 'copy_to_folder'.
 
         :param ids: an iterable of either (id, changekey) tuples or Item objects.
-        :param save_copy: If true, saves a copy of the message
+        :param save_copy: If true, saves a copy of the message (Default value = True)
         :param copy_to_folder: If requested, save a copy of the message in this folder. Default is the Sent folder
-        :param chunk_size: The number of items to send to the server in a single request
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
+
         :return: Status for each send operation, in the same order as the input
         """
         if copy_to_folder and not save_copy:
@@ -495,35 +500,31 @@ class Account:
         )
 
     def bulk_copy(self, ids, to_folder, chunk_size=None):
-        """ Copy items to another folder
+        """Copy items to another folder.
 
         :param ids: an iterable of either (id, changekey) tuples or Item objects.
         :param to_folder: The destination folder of the copy operation
-        :param chunk_size: The number of items to send to the server in a single request
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
+
         :return: Status for each send operation, in the same order as the input
         """
-        return list(
-            i if isinstance(i, Exception) else Item.id_from_xml(i)
-            for i in self._consume_item_service(service_cls=CopyItem, items=ids, chunk_size=chunk_size, kwargs=dict(
-                to_folder=to_folder,
-            ))
-        )
+        return list(self._consume_item_service(service_cls=CopyItem, items=ids, chunk_size=chunk_size, kwargs=dict(
+            to_folder=to_folder,
+        )))
 
     def bulk_move(self, ids, to_folder, chunk_size=None):
-        """Move items to another folder
+        """Move items to another folder.
 
         :param ids: an iterable of either (id, changekey) tuples or Item objects.
         :param to_folder: The destination folder of the copy operation
-        :param chunk_size: The number of items to send to the server in a single request
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
+
         :return: The new IDs of the moved items, in the same order as the input. If 'to_folder' is a public folder or a
-        folder in a different mailbox, an empty list is returned.
+          folder in a different mailbox, an empty list is returned.
         """
-        return list(
-            i if isinstance(i, Exception) else Item.id_from_xml(i)
-            for i in self._consume_item_service(service_cls=MoveItem, items=ids, chunk_size=chunk_size, kwargs=dict(
-                to_folder=to_folder,
-            ))
-        )
+        return list(self._consume_item_service(service_cls=MoveItem, items=ids, chunk_size=chunk_size, kwargs=dict(
+            to_folder=to_folder,
+        )))
 
     def bulk_archive(self, ids, to_folder, chunk_size=None):
         """Archive items to a folder in the archive mailbox. An archive mailbox must be enabled in order for this
@@ -531,7 +532,8 @@ class Account:
 
         :param ids: an iterable of either (id, changekey) tuples or Item objects.
         :param to_folder: The destination folder of the archive operation
-        :param chunk_size: The number of items to send to the server in a single request
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
+
         :return: A list containing True or an exception instance in stable order of the requested items
         """
         return list(self._consume_item_service(service_cls=ArchiveItem, items=ids, chunk_size=chunk_size, kwargs=dict(
@@ -539,13 +541,30 @@ class Account:
             ))
         )
 
-    def fetch(self, ids, folder=None, only_fields=None, chunk_size=None):
-        """ Fetch items by ID
+    def bulk_mark_as_junk(self, ids, is_junk, move_item, chunk_size=None):
+        """Mark or un-mark message items as junk email and add or remove the sender from the blocked sender list.
 
         :param ids: an iterable of either (id, changekey) tuples or Item objects.
-        :param folder: used for validating 'only_fields'
+        :param is_junk: Whether the messages are junk or not
+        :param move_item: Whether to move the messages to the junk folder or not
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
+
+        :return: A list containing the new IDs of the moved items, if items were moved, or True, or an exception
+          instance, in stable order of the requested items.
+        """
+        return list(self._consume_item_service(service_cls=MarkAsJunk, items=ids, chunk_size=chunk_size, kwargs=dict(
+            is_junk=is_junk,
+            move_item=move_item,
+        )))
+
+    def fetch(self, ids, folder=None, only_fields=None, chunk_size=None):
+        """Fetch items by ID.
+
+        :param ids: an iterable of either (id, changekey) tuples or Item objects.
+        :param folder: used for validating 'only_fields' (Default value = None)
         :param only_fields: A list of string or FieldPath items specifying the fields to fetch. Default to all fields
-        :param chunk_size: The number of items to send to the server in a single request
+        :param chunk_size: The number of items to send to the server in a single request (Default value = None)
+
         :return: A generator of Item objects, in the same order as the input
         """
         validation_folder = folder or Folder(root=self.root)  # Default to a folder type that supports all item types
@@ -564,20 +583,33 @@ class Account:
             additional_fields = {f for f in validation_folder.normalize_fields(fields=only_fields)
                                  if not f.field.is_attribute}
         # Always use IdOnly here, because AllProperties doesn't actually get *all* properties
-        for i in self._consume_item_service(service_cls=GetItem, items=ids, chunk_size=chunk_size, kwargs=dict(
+        yield from self._consume_item_service(service_cls=GetItem, items=ids, chunk_size=chunk_size, kwargs=dict(
                 additional_fields=additional_fields,
                 shape=ID_ONLY,
-        )):
-            if isinstance(i, Exception):
-                yield i
-            else:
-                item = validation_folder.item_model_from_tag(i.tag).from_xml(elem=i, account=self)
-                yield item
+        ))
+
+    def fetch_personas(self, ids):
+        """Fetch personas by ID.
+
+        :param ids: an iterable of either (id, changekey) tuples or Persona objects.
+        :return: A generator of Persona objects, in the same order as the input
+        """
+        if isinstance(ids, QuerySet):
+            # We just want an iterator over the results
+            ids = iter(ids)
+        is_empty, ids = peek(ids)
+        if is_empty:
+            # We accept generators, so it's not always convenient for caller to know up-front if 'ids' is empty. Allow
+            # empty 'ids' and return early.
+            return
+        # GetPersona only accepts one persona ID per request. Crazy.
+        svc = GetPersona(account=self)
+        for i in ids:
+            yield svc.call(persona=i)
 
     @property
     def mail_tips(self):
-        """See self.oof_settings about caching considerations
-        """
+        """See self.oof_settings about caching considerations."""
         # mail_tips_requested must be one of properties.MAIL_TIPS_TYPES
         return GetMailTips(protocol=self.protocol).get(
             sending_as=SendingAs(email_address=self.primary_smtp_address),
@@ -587,9 +619,13 @@ class Account:
 
     @property
     def delegates(self):
-        """Returns a list of DelegateUser objects representing the delegates that are set on this account
-        """
-        return list(GetDelegate(account=self).call(user_ids=None, include_permissions=True))
+        """Return a list of DelegateUser objects representing the delegates that are set on this account."""
+        delegates = []
+        for d in GetDelegate(account=self).call(user_ids=None, include_permissions=True):
+            if isinstance(d, Exception):
+                raise d
+            delegates.append(d)
+        return delegates
 
     def __str__(self):
         txt = '%s' % self.primary_smtp_address
