@@ -1,11 +1,7 @@
 # flake8: noqa
-"""
-Stores errors specific to this package, and mirrors all the possible errors that EWS can return.
-"""
+"""Stores errors specific to this package, and mirrors all the possible errors that EWS can return."""
 from urllib.parse import urlparse
 
-import pytz.exceptions
-
 
 class MultipleObjectsReturned(Exception):
     pass
@@ -16,8 +12,8 @@ class DoesNotExist(Exception):
 
 
 class EWSError(Exception):
-    """Global error type within this module.
-    """
+    """Global error type within this module."""
+
     def __init__(self, value):
         super().__init__(value)
         self.value = value
@@ -25,6 +21,9 @@ class EWSError(Exception):
     def __str__(self):
         return str(self.value)
 
+    def __eq__(self, other):
+        return repr(self) == repr(other)
+
 
 # Warnings
 class EWSWarning(EWSError):
@@ -100,18 +99,19 @@ class AutoDiscoverRedirect(AutoDiscoverError):
 
 
 class NaiveDateTimeNotAllowed(ValueError):
-    pass
+    def __init__(self, local_dt):
+        super().__init__()
+        from .ewsdatetime import EWSDateTime
+        if not isinstance(local_dt, EWSDateTime):
+            raise ValueError("'local_dt' value %r must be an EWSDateTime" % local_dt)
+        self.local_dt = local_dt
 
 
 class UnknownTimeZone(EWSError):
     pass
 
 
-class AmbiguousTimeError(EWSError, pytz.exceptions.AmbiguousTimeError):
-    pass
-
-
-class NonExistentTimeError(EWSError, pytz.exceptions.NonExistentTimeError):
+class TimezoneDefinitionInvalidForYear(EWSError):
     pass
 
 
@@ -119,6 +119,10 @@ class SessionPoolMinSizeReached(EWSError):
     pass
 
 
+class SessionPoolMaxSizeReached(EWSError):
+    pass
+
+
 class ResponseMessageError(TransportError):
     pass
 
@@ -127,6 +131,7 @@ class CASError(EWSError):
     """EWS will sometimes return an error message in an 'X-CasErrorCode' custom HTTP header in an HTTP 500 error code.
     This exception is for those cases. The caller may want to do something with the original response, so store that.
     """
+
     def __init__(self, cas_error, response):
         self.cas_error = cas_error
         self.response = response
@@ -203,6 +208,7 @@ class ErrorChangeKeyRequired(ResponseMessageError): pass
 class ErrorChangeKeyRequiredForWriteOperations(ResponseMessageError): pass
 class ErrorClientDisconnected(ResponseMessageError): pass
 class ErrorConnectionFailed(ResponseMessageError): pass
+class ErrorConnectionFailedTransientError(ResponseMessageError): pass
 class ErrorContainsFilterWrongType(ResponseMessageError): pass
 class ErrorContentConversionFailed(ResponseMessageError): pass
 class ErrorCorruptData(ResponseMessageError): pass
