@@ -1,27 +1,34 @@
-from fnmatch import fnmatch
+import abc
 import logging
+from fnmatch import fnmatch
 from operator import attrgetter
 
+from .collections import FolderCollection, SyncCompleted
+from .queryset import SingleFolderQuerySet, SHALLOW as SHALLOW_FOLDERS, DEEP as DEEP_FOLDERS
 from ..errors import ErrorAccessDenied, ErrorFolderNotFound, ErrorCannotEmptyFolder, ErrorCannotDeleteObject, \
-    ErrorDeleteDistinguishedFolder
+    ErrorDeleteDistinguishedFolder, ErrorInvalidSubscription, ErrorNoPublicFolderReplicaAvailable, ErrorItemNotFound
 from ..fields import IntegerField, CharField, FieldPath, EffectiveRightsField, PermissionSetField, EWSElementField, \
-    Field, IdElementField
-from ..items import CalendarItem, RegisterMixIn, Persona, ITEM_CLASSES, ITEM_TRAVERSAL_CHOICES, SHAPE_CHOICES, \
-    ID_ONLY, DELETE_TYPE_CHOICES, HARD_DELETE, SHALLOW as SHALLOW_ITEMS
-from ..properties import Mailbox, FolderId, ParentFolderId, InvalidField, DistinguishedFolderId, Fields
-from ..queryset import QuerySet, SearchableMixIn, DoesNotExist
-from ..restriction import Restriction
-from ..services import CreateFolder, UpdateFolder, DeleteFolder, EmptyFolder, FindPeople
+    Field, IdElementField, InvalidField
+from ..items import CalendarItem, RegisterMixIn, ITEM_CLASSES, DELETE_TYPE_CHOICES, HARD_DELETE, \
+    SHALLOW as SHALLOW_ITEMS
+from ..properties import Mailbox, FolderId, ParentFolderId, DistinguishedFolderId, UserConfiguration, \
+    UserConfigurationName, UserConfigurationNameMNS, EWSMeta
+from ..queryset import SearchableMixIn, DoesNotExist
+from ..services import CreateFolder, UpdateFolder, DeleteFolder, EmptyFolder, GetUserConfiguration, \
+    CreateUserConfiguration, UpdateUserConfiguration, DeleteUserConfiguration, SubscribeToPush, SubscribeToPull, \
+    Unsubscribe, GetEvents, GetStreamingEvents, MoveFolder
+from ..services.get_user_configuration import ALL
 from ..util import TNS, require_id
 from ..version import Version, EXCHANGE_2007_SP1, EXCHANGE_2010
-from .collections import FolderCollection
-from .queryset import SingleFolderQuerySet, SHALLOW as SHALLOW_FOLDERS, DEEP as DEEP_FOLDERS
 
 log = logging.getLogger(__name__)
 
+MISSING_FOLDER_ERRORS = (ErrorFolderNotFound, ErrorItemNotFound, ErrorNoPublicFolderReplicaAvailable)
+
+
+class BaseFolder(RegisterMixIn, SearchableMixIn, metaclass=EWSMeta):
+    """Base class for all classes that implement a folder."""
 
-class BaseFolder(RegisterMixIn, SearchableMixIn):
-    """Base class for all classes that implement a folder"""
     ELEMENT_NAME = 'Folder'
     NAMESPACE = TNS
     # See https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/distinguishedfolderid
@@ -37,47 +44,50 @@ class BaseFolder(RegisterMixIn, SearchableMixIn):
     get_folder_allowed = True
     DEFAULT_FOLDER_TRAVERSAL_DEPTH = DEEP_FOLDERS
     DEFAULT_ITEM_TRAVERSAL_DEPTH = SHALLOW_ITEMS
-    LOCALIZED_NAMES = dict()  # A map of (str)locale: (tuple)localized_folder_names
+    LOCALIZED_NAMES = {}  # A map of (str)locale: (tuple)localized_folder_names
     ITEM_MODEL_MAP = {cls.response_tag(): cls for cls in ITEM_CLASSES}
     ID_ELEMENT_CLS = FolderId
-    FIELDS = Fields(
-        IdElementField('_id', field_uri='folder:FolderId', value_cls=ID_ELEMENT_CLS),
-        EWSElementField('parent_folder_id', field_uri='folder:ParentFolderId', value_cls=ParentFolderId,
-                        is_read_only=True),
-        CharField('folder_class', field_uri='folder:FolderClass', is_required_after_save=True),
-        CharField('name', field_uri='folder:DisplayName'),
-        IntegerField('total_count', field_uri='folder:TotalCount', is_read_only=True),
-        IntegerField('child_folder_count', field_uri='folder:ChildFolderCount', is_read_only=True),
-        IntegerField('unread_count', field_uri='folder:UnreadCount', is_read_only=True),
-    )
 
-    __slots__ = tuple(f.name for f in FIELDS) + ('is_distinguished',)
+    _id = IdElementField(field_uri='folder:FolderId', value_cls=ID_ELEMENT_CLS)
+    parent_folder_id = EWSElementField(field_uri='folder:ParentFolderId', value_cls=ParentFolderId,
+                                       is_read_only=True)
+    folder_class = CharField(field_uri='folder:FolderClass', is_required_after_save=True)
+    name = CharField(field_uri='folder:DisplayName')
+    total_count = IntegerField(field_uri='folder:TotalCount', is_read_only=True)
+    child_folder_count = IntegerField(field_uri='folder:ChildFolderCount', is_read_only=True)
+    unread_count = IntegerField(field_uri='folder:UnreadCount', is_read_only=True)
+
+    __slots__ = 'is_distinguished', 'item_sync_state', 'folder_sync_state'
 
     # Used to register extended properties
     INSERT_AFTER_FIELD = 'child_folder_count'
 
     def __init__(self, **kwargs):
         self.is_distinguished = kwargs.pop('is_distinguished', False)
+        self.item_sync_state = kwargs.pop('item_sync_state', None)
+        self.folder_sync_state = kwargs.pop('folder_sync_state', None)
         super().__init__(**kwargs)
 
     @property
+    @abc.abstractmethod
     def account(self):
-        raise NotImplementedError()
+        pass
 
     @property
+    @abc.abstractmethod
     def root(self):
-        raise NotImplementedError()
+        pass
 
     @property
+    @abc.abstractmethod
     def parent(self):
-        raise NotImplementedError()
+        pass
 
     @property
-    def is_deleteable(self):
+    def is_deletable(self):
         return not self.is_distinguished
 
     def clean(self, version=None):
-        # pylint: disable=access-member-before-definition
         super().clean(version=version)
         # Set a default folder class for new folders. A folder class cannot be changed after saving.
         if self.id is None and self.folder_class is None:
@@ -105,8 +115,7 @@ class BaseFolder(RegisterMixIn, SearchableMixIn):
     def _walk(self):
         for c in self.children:
             yield c
-            for f in c.walk():
-                yield f
+            yield from c.walk()
 
     def walk(self):
         return FolderCollection(account=self.account, folders=self._walk())
@@ -116,14 +125,12 @@ class BaseFolder(RegisterMixIn, SearchableMixIn):
         head, tail = (split_pattern[0], None) if len(split_pattern) == 1 else split_pattern
         if head == '':
             # We got an absolute path. Restart globbing at root
-            for f in self.root.glob(tail or '*'):
-                yield f
+            yield from self.root.glob(tail or '*')
         elif head == '..':
             # Relative path with reference to parent. Restart globbing at parent
             if not self.parent:
                 raise ValueError('Already at top')
-            for f in self.parent.glob(tail or '*'):
-                yield f
+            yield from self.parent.glob(tail or '*')
         elif head == '**':
             # Match anything here or in any subfolder at arbitrary depth
             for c in self.walk():
@@ -141,15 +148,13 @@ class BaseFolder(RegisterMixIn, SearchableMixIn):
                 if tail is None:
                     yield c
                     continue
-                for f in c.glob(tail):
-                    yield f
+                yield from c.glob(tail)
 
     def glob(self, pattern):
         return FolderCollection(account=self.account, folders=self._glob(pattern))
 
     def tree(self):
-        """
-        Returns a string representation of the folder structure of this folder. Example:
+        """Return a string representation of the folder structure of this folder. Example:
 
         root
         ├── inbox
@@ -198,8 +203,11 @@ class BaseFolder(RegisterMixIn, SearchableMixIn):
 
     @staticmethod
     def folder_cls_from_container_class(container_class):
-        """Returns a reasonable folder class given a container class, e.g. 'IPF.Note'. Don't iterate WELLKNOWN_FOLDERS
+        """Return a reasonable folder class given a container class, e.g. 'IPF.Note'. Don't iterate WELLKNOWN_FOLDERS
         because many folder classes have the same CONTAINER_CLASS.
+
+        :param container_class:
+        :return:
         """
         from .known_folders import Messages, Tasks, Calendar, ConversationSettings, Contacts, GALContacts, Reminders, \
             RecipientCache, RSSFeeds
@@ -299,64 +307,8 @@ class BaseFolder(RegisterMixIn, SearchableMixIn):
         return FolderCollection(account=self.account, folders=[self]).exclude(*args, **kwargs)
 
     def people(self):
-        return QuerySet(
-            folder_collection=FolderCollection(account=self.account, folders=[self]),
-            request_type=QuerySet.PERSONA,
-        )
-
-    def find_people(self, q, shape=ID_ONLY, depth=None, additional_fields=None, order_fields=None,
-                    page_size=None, max_items=None, offset=0):
-        """
-        Private method to call the FindPeople service
-
-        :param q: a Q instance containing any restrictions
-        :param shape: controls whether to return (id, chanegkey) tuples or Persona objects. If additional_fields is
-               non-null, we always return Persona objects.
-        :param depth: controls the whether to return soft-deleted items or not.
-        :param additional_fields: the extra properties we want on the return objects. Default is no properties.
-        :param order_fields: the SortOrder fields, if any
-        :param page_size: the requested number of items per page
-        :param max_items: the max number of items to return
-        :param offset: the offset relative to the first item in the item collection
-        :return: a generator for the returned personas
-        """
-        if shape not in SHAPE_CHOICES:
-            raise ValueError("'shape' %s must be one of %s" % (shape, SHAPE_CHOICES))
-        if depth is None:
-            depth = self.DEFAULT_ITEM_TRAVERSAL_DEPTH
-        if depth not in ITEM_TRAVERSAL_CHOICES:
-            raise ValueError("'depth' %s must be one of %s" % (depth, ITEM_TRAVERSAL_CHOICES))
-        if additional_fields:
-            for f in additional_fields:
-                Persona.validate_field(field=f, version=self.account.version)
-                if f.field.is_complex:
-                    raise ValueError("find_people() does not support field '%s'" % f.field.name)
-
-        # Build up any restrictions
-        if q.is_empty():
-            restriction = None
-            query_string = None
-        elif q.query_string:
-            restriction = None
-            query_string = Restriction(q, folders=[self], applies_to=Restriction.ITEMS)
-        else:
-            restriction = Restriction(q, folders=[self], applies_to=Restriction.ITEMS)
-            query_string = None
-        personas = FindPeople(account=self.account, chunk_size=page_size).call(
-                folder=self,
-                additional_fields=additional_fields,
-                restriction=restriction,
-                order_fields=order_fields,
-                shape=shape,
-                query_string=query_string,
-                depth=depth,
-                max_items=max_items,
-                offset=offset,
-        )
-        for p in personas:
-            if isinstance(p, Exception):
-                raise p
-            yield p
+        # No point in using a FolderCollection because FindPeople only supports one folder
+        return FolderCollection(account=self.account, folders=[self]).people()
 
     def bulk_create(self, items, *args, **kwargs):
         return self.account.bulk_create(folder=self, items=items, *args, **kwargs)
@@ -379,10 +331,11 @@ class BaseFolder(RegisterMixIn, SearchableMixIn):
                 if f.is_read_only:
                     # These cannot be changed
                     continue
-                if f.is_required or f.is_required_after_save:
-                    if getattr(self, f.name) is None or (f.is_list and not getattr(self, f.name)):
-                        # These are required and cannot be deleted
-                        continue
+                if (f.is_required or f.is_required_after_save) and (
+                        getattr(self, f.name) is None or (f.is_list and not getattr(self, f.name))
+                ):
+                    # These are required and cannot be deleted
+                    continue
                 update_fields.append(f.name)
         res = UpdateFolder(account=self.account).get(folders=[(self, update_fields)])
         folder_id, changekey = res.id, res.changekey
@@ -391,7 +344,17 @@ class BaseFolder(RegisterMixIn, SearchableMixIn):
         # Don't check changekey value. It may not change on no-op updates
         self.changekey = changekey
         self.root.update_folder(self)  # Update the folder in the cache
-        return None
+        return self
+
+    def move(self, to_folder):
+        res = MoveFolder(account=self.account).get(folders=[self], to_folder=to_folder)
+        folder_id, changekey = res.id, res.changekey
+        if self.id != folder_id:
+            raise ValueError('ID mismatch')
+        # Don't check changekey value. It may not change on no-op moves
+        self.changekey = changekey
+        self.parent_folder_id = ParentFolderId(id=to_folder.id, changekey=to_folder.changekey)
+        self.root.update_folder(self)  # Update the folder in the cache
 
     def delete(self, delete_type=HARD_DELETE):
         if delete_type not in DELETE_TYPE_CHOICES:
@@ -410,34 +373,38 @@ class BaseFolder(RegisterMixIn, SearchableMixIn):
             # We don't know exactly what was deleted, so invalidate the entire folder cache to be safe
             self.root.clear_cache()
 
-    def wipe(self, page_size=None):
+    def wipe(self, page_size=None, _seen=None, _level=0):
         # Recursively deletes all items in this folder, and all subfolders and their content. Attempts to protect
         # distinguished folders from being deleted. Use with caution!
+        _seen = _seen or set()
+        if self.id in _seen:
+            raise RecursionError('We already tried to wipe %s' % self)
+        if _level > 16:
+            raise RecursionError('Max recursion level reached: %s' % _level)
+        _seen.add(self.id)
         log.warning('Wiping %s', self)
-        delete_kwargs = {}
-        if page_size:
-            delete_kwargs['page_size'] = page_size
         has_distinguished_subfolders = any(f.is_distinguished for f in self.children)
         try:
             if has_distinguished_subfolders:
                 self.empty(delete_sub_folders=False)
             else:
                 self.empty(delete_sub_folders=True)
-        except (ErrorAccessDenied, ErrorCannotEmptyFolder):
+        except (ErrorAccessDenied, ErrorCannotEmptyFolder, ErrorItemNotFound):
             try:
                 if has_distinguished_subfolders:
                     raise  # We already tried this
                 self.empty(delete_sub_folders=False)
-            except (ErrorAccessDenied, ErrorCannotEmptyFolder):
+            except (ErrorAccessDenied, ErrorCannotEmptyFolder, ErrorItemNotFound):
                 log.warning('Not allowed to empty %s. Trying to delete items instead', self)
                 try:
-                    self.all().delete(**delete_kwargs)
-                except (ErrorAccessDenied, ErrorCannotDeleteObject):
+                    self.all().delete(**dict(page_size=page_size) if page_size else {})
+                except (ErrorAccessDenied, ErrorCannotDeleteObject, ErrorItemNotFound):
                     log.warning('Not allowed to delete items in %s', self)
+        _level += 1
         for f in self.children:
-            f.wipe(page_size=page_size)
+            f.wipe(page_size=page_size, _seen=_seen, _level=_level)
             # Remove non-distinguished children that are empty and have no subfolders
-            if f.is_deleteable and not f.children:
+            if f.is_deletable and not f.children:
                 log.warning('Deleting folder %s', f)
                 try:
                     f.delete()
@@ -445,8 +412,7 @@ class BaseFolder(RegisterMixIn, SearchableMixIn):
                     log.warning('Tried to delete a distinguished folder (%s)', f)
 
     def test_access(self):
-        """
-        Does a simple FindItem to test (read) access to the folder. Maybe the account doesn't exist, maybe the
+        """Does a simple FindItem to test (read) access to the folder. Maybe the account doesn't exist, maybe the
         service user doesn't have access to the calendar. This will throw the most common errors.
         """
         self.all().exists()
@@ -463,7 +429,7 @@ class BaseFolder(RegisterMixIn, SearchableMixIn):
             kwargs['name'] = cls.DISTINGUISHED_FOLDER_ID
         return kwargs
 
-    def to_xml(self, version):
+    def to_folder_id(self):
         if self.is_distinguished:
             # Don't add the changekey here. When modifying folder content, we usually don't care if others have changed
             # the folder content since we fetched the changekey.
@@ -471,11 +437,17 @@ class BaseFolder(RegisterMixIn, SearchableMixIn):
                 return DistinguishedFolderId(
                     id=self.DISTINGUISHED_FOLDER_ID,
                     mailbox=Mailbox(email_address=self.account.primary_smtp_address)
-                ).to_xml(version=version)
-            return DistinguishedFolderId(id=self.DISTINGUISHED_FOLDER_ID).to_xml(version=version)
+                )
+            return DistinguishedFolderId(id=self.DISTINGUISHED_FOLDER_ID)
         if self.id:
-            return FolderId(id=self.id, changekey=self.changekey).to_xml(version=version)
-        return super().to_xml(version=version)
+            return FolderId(id=self.id, changekey=self.changekey)
+        raise ValueError('Must be a distinguished folder or have an ID')
+
+    def to_xml(self, version):
+        try:
+            return self.to_folder_id().to_xml(version=version)
+        except ValueError:
+            return super().to_xml(version=version)
 
     def to_id_xml(self, version):
         # Folder(name='Foo') is a perfectly valid ID to e.g. create a folder
@@ -504,11 +476,222 @@ class BaseFolder(RegisterMixIn, SearchableMixIn):
         # Apparently, the changekey may get updated
         for f in self.FIELDS:
             setattr(self, f.name, getattr(fresh_folder, f.name))
+        return self
+
+    @require_id
+    def get_user_configuration(self, name, properties=ALL):
+        return GetUserConfiguration(account=self.account).get(
+            user_configuration_name=UserConfigurationNameMNS(name=name, folder=self),
+            properties=properties,
+        )
+
+    @require_id
+    def create_user_configuration(self, name, dictionary=None, xml_data=None, binary_data=None):
+        user_configuration = UserConfiguration(
+            user_configuration_name=UserConfigurationName(name=name, folder=self),
+            dictionary=dictionary,
+            xml_data=xml_data,
+            binary_data=binary_data,
+        )
+        return CreateUserConfiguration(account=self.account).get(user_configuration=user_configuration)
+
+    @require_id
+    def update_user_configuration(self, name, dictionary=None, xml_data=None, binary_data=None):
+        user_configuration = UserConfiguration(
+            user_configuration_name=UserConfigurationName(name=name, folder=self),
+            dictionary=dictionary,
+            xml_data=xml_data,
+            binary_data=binary_data,
+        )
+        return UpdateUserConfiguration(account=self.account).get(user_configuration=user_configuration)
+
+    @require_id
+    def delete_user_configuration(self, name):
+        return DeleteUserConfiguration(account=self.account).get(
+            user_configuration_name=UserConfigurationNameMNS(name=name, folder=self)
+        )
+
+    @require_id
+    def subscribe_to_pull(self, event_types=SubscribeToPull.EVENT_TYPES, watermark=None, timeout=60):
+        """Create a pull subscription.
+
+        :param event_types: List of event types to subscribe to. Possible values defined in SubscribeToPull.EVENT_TYPES
+        :param watermark: An event bookmark as returned by some sync services
+        :param timeout: Timeout of the subscription, in minutes. Timeout is reset when the server receives a
+        GetEvents request for this subscription.
+        :return: The subscription ID and a watermark
+        """
+        s_ids = list(FolderCollection(account=self.account, folders=[self]).subscribe_to_pull(
+            event_types=event_types, watermark=watermark, timeout=timeout,
+        ))
+        if len(s_ids) != 1:
+            raise ValueError('Expected result length 1, but got %s' % s_ids)
+        s_id = s_ids[0]
+        if isinstance(s_id, Exception):
+            raise s_id
+        return s_id
+
+    @require_id
+    def subscribe_to_push(self, callback_url, event_types=SubscribeToPush.EVENT_TYPES, watermark=None,
+                          status_frequency=1):
+        """Create a push subscription.
+
+        :param callback_url: A client-defined URL that the server will call
+        :param event_types: List of event types to subscribe to. Possible values defined in SubscribeToPush.EVENT_TYPES
+        :param watermark: An event bookmark as returned by some sync services
+        :param status_frequency: The frequency, in minutes, that the callback URL will be called with.
+        :return: The subscription ID and a watermark
+        """
+        s_ids = list(FolderCollection(account=self.account, folders=[self]).subscribe_to_push(
+            event_types=event_types, watermark=watermark, status_frequency=status_frequency, callback_url=callback_url,
+        ))
+        if len(s_ids) != 1:
+            raise ValueError('Expected result length 1, but got %s' % s_ids)
+        s_id = s_ids[0]
+        if isinstance(s_id, Exception):
+            raise s_id
+        return s_id
+
+    @require_id
+    def subscribe_to_streaming(self, event_types=SubscribeToPush.EVENT_TYPES):
+        """Create a streaming subscription.
+
+        :param event_types: List of event types to subscribe to. Possible values defined in SubscribeToPush.EVENT_TYPES
+        :return: The subscription ID
+        """
+        s_ids = list(FolderCollection(account=self.account, folders=[self]).subscribe_to_streaming(
+            event_types=event_types,
+        ))
+        if len(s_ids) != 1:
+            raise ValueError('Expected result length 1, but got %s' % s_ids)
+        s_id = s_ids[0]
+        if isinstance(s_id, Exception):
+            raise s_id
+        return s_id
+
+    @require_id
+    def pull_subscription(self, **kwargs):
+        return PullSubscription(folder=self, **kwargs)
+
+    @require_id
+    def push_subscription(self, **kwargs):
+        return PushSubscription(folder=self, **kwargs)
+
+    @require_id
+    def streaming_subscription(self, **kwargs):
+        return StreamingSubscription(folder=self, **kwargs)
+
+    def unsubscribe(self, subscription_id):
+        """Unsubscribe. Only applies to pull and streaming notifications.
+
+        :param subscription_id: A subscription ID as acquired by .subscribe_to_[pull|streaming]()
+        :return: True
+
+        This method doesn't need the current folder instance, but it makes sense to keep the method along the other
+        sync methods.
+        """
+        return Unsubscribe(account=self.account).get(subscription_id=subscription_id)
+
+    def sync_items(self, sync_state=None, only_fields=None, ignore=None, max_changes_returned=None, sync_scope=None):
+        """Return all item changes to a folder, as a generator. If sync_state is specified, get all item changes after
+        this sync state. After fully consuming the generator, self.item_sync_state will hold the new sync state.
+
+        :param sync_state: The state of the sync. Returned by a successful call to the SyncFolderitems service.
+        :param only_fields: A list of string or FieldPath items specifying the fields to fetch. Default to all fields
+        :param ignore: A list of Item IDs to ignore in the sync
+        :param max_changes_returned: The max number of change
+        :param sync_scope: Specify whether to return just items, or items and folder associated information. Possible
+           values are specified in SyncFolderitems.SYNC_SCOPES
+        :return: A generator of (change_type, item) tuples
+        """
+        if not sync_state:
+            sync_state = self.item_sync_state
+        try:
+            yield from FolderCollection(account=self.account, folders=[self]).sync_items(
+                sync_state=sync_state,
+                only_fields=only_fields,
+                ignore=ignore,
+                max_changes_returned=max_changes_returned,
+                sync_scope=sync_scope,
+            )
+        except SyncCompleted as e:
+            # Set the new sync state on the folder instance
+            self.item_sync_state = e.sync_state
+
+    def sync_hierarchy(self, sync_state=None, only_fields=None):
+        """Return all folder changes to a folder hierarchy, as a generator. If sync_state is specified, get all folder
+        changes after this sync state. After fully consuming the generator, self.folder_sync_state will hold the new
+        sync state.
+
+        :param sync_state: The state of the sync. Returned by a successful call to the SyncFolderitems service.
+        :param only_fields: A list of string or FieldPath items specifying the fields to fetch. Default to all fields
+        :return:
+        """
+        if not sync_state:
+            sync_state = self.folder_sync_state
+        try:
+            yield from FolderCollection(account=self.account, folders=[self]).sync_hierarchy(
+                sync_state=sync_state,
+                only_fields=only_fields,
+            )
+        except SyncCompleted as e:
+            # Set the new sync state on the folder instance
+            self.folder_sync_state = e.sync_state
+
+    def get_events(self, subscription_id, watermark):
+        """Get events since the given watermark. Non-blocking.
+
+        :param subscription_id: A subscription ID as acquired by .subscribe_to_[pull|push]()
+        :param watermark: Either the watermark from the subscription, or as returned in the last .get_events() call.
+        :return: A Notification object containing a list of events
+
+        This method doesn't need the current folder instance, but it makes sense to keep the method along the other
+        sync methods.
+        """
+        svc = GetEvents(account=self.account)
+        while True:
+            notification = svc.get(subscription_id=subscription_id, watermark=watermark)
+            yield notification
+            if not notification.more_events:
+                break
+
+    def get_streaming_events(self, subscription_id, connection_timeout=1, max_notifications_returned=None):
+        """Get events since the subscription was created, in streaming mode. This method will block as many minutes
+        as specified by 'connection_timeout'.
+
+        :param subscription_id: A subscription ID as acquired by .subscribe_to_streaming()
+        :param connection_timeout: Timeout of the connection, in minutes. The connection is closed after this timeout
+        is reached.
+        :param max_notifications_returned: If specified, will exit after receiving this number of notifications
+        :return: A generator of Notification objects, each containing a list of events
+
+        This method doesn't need the current folder instance, but it makes sense to keep the method along the other
+        sync methods.
+        """
+        # Add 60 seconds to the timeout, to allow us to always get the final message containing ConnectionStatus=Closed
+        request_timeout = connection_timeout*60 + 60
+        svc = GetStreamingEvents(account=self.account, timeout=request_timeout)
+        for i, notification in enumerate(
+                svc.call(subscription_ids=[subscription_id], connection_timeout=connection_timeout),
+                start=1
+        ):
+            yield notification
+            if max_notifications_returned and i >= max_notifications_returned:
+                svc.stop_streaming()
+                break
+        if svc.error_subscription_ids:
+            raise ErrorInvalidSubscription('Invalid subscription IDs: %s' % svc.error_subscription_ids)
 
     def __floordiv__(self, other):
-        """Same as __truediv__ but does not touch the folder cache.
+        """Support the some_folder // 'child_folder' // 'child_of_child_folder' navigation syntax.
 
-        This is useful if the folder hierarchy contains a huge number of folders and you don't want to fetch them all"""
+        Works like as __truediv__ but does not touch the folder cache.
+
+        This is useful if the folder hierarchy contains a huge number of folders and you don't want to fetch them all
+
+        :param other:
+        :return:
+        """
         if other == '..':
             raise ValueError('Cannot get parent without a folder cache')
 
@@ -522,7 +705,7 @@ class BaseFolder(RegisterMixIn, SearchableMixIn):
             raise ErrorFolderNotFound("No subfolder with name '%s'" % other)
 
     def __truediv__(self, other):
-        # Support the some_folder / 'child_folder' / 'child_of_child_folder' navigation syntax
+        """Support the some_folder / 'child_folder' / 'child_of_child_folder' navigation syntax."""
         if other == '..':
             if not self.parent:
                 raise ValueError('Already at top')
@@ -545,14 +728,12 @@ class BaseFolder(RegisterMixIn, SearchableMixIn):
 
 class Folder(BaseFolder):
     """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/folder"""
-    LOCAL_FIELDS = Fields(
-        PermissionSetField('permission_set', field_uri='folder:PermissionSet', supported_from=EXCHANGE_2007_SP1),
-        EffectiveRightsField('effective_rights', field_uri='folder:EffectiveRights', is_read_only=True,
-                             supported_from=EXCHANGE_2007_SP1),
-    )
-    FIELDS = BaseFolder.FIELDS + LOCAL_FIELDS
 
-    __slots__ = tuple(f.name for f in LOCAL_FIELDS) + ('_root',)
+    permission_set = PermissionSetField(field_uri='folder:PermissionSet', supported_from=EXCHANGE_2007_SP1)
+    effective_rights = EffectiveRightsField(field_uri='folder:EffectiveRights', is_read_only=True,
+                                            supported_from=EXCHANGE_2007_SP1)
+
+    __slots__ = '_root',
 
     def __init__(self, **kwargs):
         self._root = kwargs.pop('root', None)  # This is a pointer to the root of the folder hierarchy
@@ -563,9 +744,8 @@ class Folder(BaseFolder):
                     raise ValueError("'parent.root' must match 'root'")
             else:
                 self.root = parent.root
-            if 'parent_folder_id' in kwargs:
-                if parent.id != kwargs['parent_folder_id']:
-                    raise ValueError("'parent_folder_id' must match 'parent' ID")
+            if 'parent_folder_id' in kwargs and parent.id != kwargs['parent_folder_id']:
+                raise ValueError("'parent_folder_id' must match 'parent' ID")
             kwargs['parent_folder_id'] = ParentFolderId(id=parent.id, changekey=parent.changekey)
         super().__init__(**kwargs)
 
@@ -597,13 +777,17 @@ class Folder(BaseFolder):
 
     @classmethod
     def get_distinguished(cls, root):
-        """Gets the distinguished folder for this folder class"""
+        """Get the distinguished folder for this folder class.
+
+        :param root:
+        :return:
+        """
         try:
             return cls.resolve(
                 account=root.account,
                 folder=cls(root=root, name=cls.DISTINGUISHED_FOLDER_ID, is_distinguished=True)
             )
-        except ErrorFolderNotFound:
+        except MISSING_FOLDER_ERRORS:
             raise ErrorFolderNotFound('Could not find distinguished folder %r' % cls.DISTINGUISHED_FOLDER_ID)
 
     @property
@@ -613,7 +797,7 @@ class Folder(BaseFolder):
         if self.parent_folder_id.id == self.id:
             # Some folders have a parent that references itself. Avoid circular references here
             return None
-        return self.root.get_folder(self.parent_folder_id.id)
+        return self.root.get_folder(self.parent_folder_id)
 
     @parent.setter
     def parent(self, value):
@@ -626,20 +810,14 @@ class Folder(BaseFolder):
             self.parent_folder_id = ParentFolderId(id=value.id, changekey=value.changekey)
 
     def clean(self, version=None):
-        # pylint: disable=access-member-before-definition
         from .roots import RootOfHierarchy
         super().clean(version=version)
         if self.root and not isinstance(self.root, RootOfHierarchy):
             raise ValueError("'root' %r must be a RootOfHierarchy instance" % self.root)
 
-    @classmethod
-    def from_xml(cls, elem, account):
-        raise NotImplementedError('Use from_xml_with_root() instead')
-
     @classmethod
     def from_xml_with_root(cls, elem, root):
-        kwargs = cls._kwargs_from_elem(elem=elem, account=root.account)
-        cls._clear(elem)
+        folder = cls.from_xml(elem=elem, account=root.account)
         folder_cls = cls
         if cls == Folder:
             # We were called on the generic Folder class. Try to find a more specific class to return objects as.
@@ -658,21 +836,57 @@ class Folder(BaseFolder):
             #
             # The returned XML may contain neither folder class nor name. In that case, we default to the generic
             # Folder class.
-            if kwargs['name']:
+            if folder.name:
                 try:
                     # TODO: fld_class.LOCALIZED_NAMES is most definitely neither complete nor authoritative
-                    folder_cls = root.folder_cls_from_folder_name(folder_name=kwargs['name'],
+                    folder_cls = root.folder_cls_from_folder_name(folder_name=folder.name,
                                                                   locale=root.account.locale)
-                    log.debug('Folder class %s matches localized folder name %s', folder_cls, kwargs['name'])
+                    log.debug('Folder class %s matches localized folder name %s', folder_cls, folder.name)
                 except KeyError:
                     pass
-            if kwargs['folder_class'] and folder_cls == Folder:
+            if folder.folder_class and folder_cls == Folder:
                 try:
-                    folder_cls = cls.folder_cls_from_container_class(container_class=kwargs['folder_class'])
-                    log.debug('Folder class %s matches container class %s (%s)', folder_cls, kwargs['folder_class'],
-                              kwargs['name'])
+                    folder_cls = cls.folder_cls_from_container_class(container_class=folder.folder_class)
+                    log.debug('Folder class %s matches container class %s (%s)', folder_cls, folder.folder_class,
+                              folder.name)
                 except KeyError:
                     pass
             if folder_cls == Folder:
-                log.debug('Fallback to class Folder (folder_class %s, name %s)', kwargs['folder_class'], kwargs['name'])
-        return folder_cls(root=root, **kwargs)
+                log.debug('Fallback to class Folder (folder_class %s, name %s)', folder.folder_class, folder.name)
+        return folder_cls(root=root, **{f.name: getattr(folder, f.name) for f in folder.FIELDS})
+
+
+class BaseSubscription(metaclass=abc.ABCMeta):
+    def __init__(self, folder, **subscription_kwargs):
+        self.folder = folder
+        self.subscription_kwargs = subscription_kwargs
+        self.subscription_id = None
+
+    def __enter__(self):
+        pass
+
+    def __exit__(self, *args, **kwargs):
+        self.folder.unsubscribe(subscription_id=self.subscription_id)
+        self.subscription_id = None
+
+
+class PullSubscription(BaseSubscription):
+    def __enter__(self):
+        self.subscription_id, watermark = self.folder.subscribe_to_pull(**self.subscription_kwargs)
+        return self.subscription_id, watermark
+
+
+class PushSubscription(BaseSubscription):
+    def __enter__(self):
+        self.subscription_id, watermark = self.folder.subscribe_to_push(**self.subscription_kwargs)
+        return self.subscription_id, watermark
+
+    def __exit__(self, *args, **kwargs):
+        # Cannot unsubscribe to push subscriptions
+        pass
+
+
+class StreamingSubscription(BaseSubscription):
+    def __enter__(self):
+        self.subscription_id = self.folder.subscribe_to_streaming(**self.subscription_kwargs)
+        return self.subscription_id
