@@ -1,12 +1,13 @@
+from .base import Folder
+from .collections import FolderCollection
 from ..items import CalendarItem, Contact, Message, Task, DistributionList, MeetingRequest, MeetingResponse, \
     MeetingCancellation, ITEM_CLASSES, ASSOCIATED
 from ..version import EXCHANGE_2010_SP1, EXCHANGE_2013, EXCHANGE_2013_SP1
-from .base import Folder
-from .collections import FolderCollection
 
 
 class Calendar(Folder):
-    """An interface for the Exchange calendar"""
+    """An interface for the Exchange calendar."""
+
     DISTINGUISHED_FOLDER_ID = 'calendar'
     CONTAINER_CLASS = 'IPF.Appointment'
     supported_item_models = (CalendarItem,)
@@ -22,7 +23,6 @@ class Calendar(Folder):
         'sv_SE': ('Kalender',),
         'zh_CN': ('日历',),
     }
-    __slots__ = tuple()
 
     def view(self, *args, **kwargs):
         return FolderCollection(account=self.account, folders=[self]).view(*args, **kwargs)
@@ -44,13 +44,11 @@ class DeletedItems(Folder):
         'sv_SE': ('Borttaget',),
         'zh_CN': ('已删除邮件',),
     }
-    __slots__ = tuple()
 
 
 class Messages(Folder):
     CONTAINER_CLASS = 'IPF.Note'
     supported_item_models = (Message, MeetingRequest, MeetingResponse, MeetingCancellation)
-    __slots__ = tuple()
 
 
 class Drafts(Messages):
@@ -67,7 +65,6 @@ class Drafts(Messages):
         'sv_SE': ('Utkast',),
         'zh_CN': ('草稿',),
     }
-    __slots__ = tuple()
 
 
 class Inbox(Messages):
@@ -84,7 +81,6 @@ class Inbox(Messages):
         'sv_SE': ('Inkorgen',),
         'zh_CN': ('收件箱',),
     }
-    __slots__ = tuple()
 
 
 class Outbox(Messages):
@@ -101,7 +97,6 @@ class Outbox(Messages):
         'sv_SE': ('Utkorgen',),
         'zh_CN': ('发件箱',),
     }
-    __slots__ = tuple()
 
 
 class SentItems(Messages):
@@ -118,7 +113,6 @@ class SentItems(Messages):
         'sv_SE': ('Skickat',),
         'zh_CN': ('已发送邮件',),
     }
-    __slots__ = tuple()
 
 
 class JunkEmail(Messages):
@@ -135,7 +129,6 @@ class JunkEmail(Messages):
         'sv_SE': ('Skräppost',),
         'zh_CN': ('垃圾邮件',),
     }
-    __slots__ = tuple()
 
 
 class Tasks(Folder):
@@ -154,7 +147,6 @@ class Tasks(Folder):
         'sv_SE': ('Uppgifter',),
         'zh_CN': ('任务',),
     }
-    __slots__ = tuple()
 
 
 class Contacts(Folder):
@@ -173,32 +165,28 @@ class Contacts(Folder):
         'sv_SE': ('Kontakter',),
         'zh_CN': ('联系人',),
     }
-    __slots__ = tuple()
 
 
 class WellknownFolder(Folder):
-    """A base class to use until we have a more specific folder implementation for this folder"""
+    """A base class to use until we have a more specific folder implementation for this folder."""
+
     supported_item_models = ITEM_CLASSES
-    __slots__ = tuple()
 
 
 class AdminAuditLogs(WellknownFolder):
     DISTINGUISHED_FOLDER_ID = 'adminauditlogs'
     supported_from = EXCHANGE_2013
     get_folder_allowed = False
-    __slots__ = tuple()
 
 
 class ArchiveDeletedItems(WellknownFolder):
     DISTINGUISHED_FOLDER_ID = 'archivedeleteditems'
     supported_from = EXCHANGE_2010_SP1
-    __slots__ = tuple()
 
 
 class ArchiveInbox(WellknownFolder):
     DISTINGUISHED_FOLDER_ID = 'archiveinbox'
     supported_from = EXCHANGE_2013_SP1
-    __slots__ = tuple()
 
 
 class ArchiveMsgFolderRoot(WellknownFolder):
@@ -209,85 +197,73 @@ class ArchiveMsgFolderRoot(WellknownFolder):
 class ArchiveRecoverableItemsDeletions(WellknownFolder):
     DISTINGUISHED_FOLDER_ID = 'archiverecoverableitemsdeletions'
     supported_from = EXCHANGE_2010_SP1
-    __slots__ = tuple()
 
 
 class ArchiveRecoverableItemsPurges(WellknownFolder):
     DISTINGUISHED_FOLDER_ID = 'archiverecoverableitemspurges'
     supported_from = EXCHANGE_2010_SP1
-    __slots__ = tuple()
 
 
 class ArchiveRecoverableItemsRoot(WellknownFolder):
     DISTINGUISHED_FOLDER_ID = 'archiverecoverableitemsroot'
     supported_from = EXCHANGE_2010_SP1
-    __slots__ = tuple()
 
 
 class ArchiveRecoverableItemsVersions(WellknownFolder):
     DISTINGUISHED_FOLDER_ID = 'archiverecoverableitemsversions'
     supported_from = EXCHANGE_2010_SP1
-    __slots__ = tuple()
 
 
 class Conflicts(WellknownFolder):
     DISTINGUISHED_FOLDER_ID = 'conflicts'
     supported_from = EXCHANGE_2013
-    __slots__ = tuple()
 
 
 class ConversationHistory(WellknownFolder):
     DISTINGUISHED_FOLDER_ID = 'conversationhistory'
     supported_from = EXCHANGE_2013
-    __slots__ = tuple()
 
 
 class Directory(WellknownFolder):
     DISTINGUISHED_FOLDER_ID = 'directory'
     supported_from = EXCHANGE_2013_SP1
-    __slots__ = tuple()
 
 
 class Favorites(WellknownFolder):
     CONTAINER_CLASS = 'IPF.Note'
     DISTINGUISHED_FOLDER_ID = 'favorites'
     supported_from = EXCHANGE_2013
-    __slots__ = tuple()
 
 
 class IMContactList(WellknownFolder):
     CONTAINER_CLASS = 'IPF.Contact.MOC.ImContactList'
     DISTINGUISHED_FOLDER_ID = 'imcontactlist'
     supported_from = EXCHANGE_2013
-    __slots__ = tuple()
 
 
 class Journal(WellknownFolder):
     CONTAINER_CLASS = 'IPF.Journal'
     DISTINGUISHED_FOLDER_ID = 'journal'
-    __slots__ = tuple()
 
 
 class LocalFailures(WellknownFolder):
     DISTINGUISHED_FOLDER_ID = 'localfailures'
     supported_from = EXCHANGE_2013
-    __slots__ = tuple()
 
 
 class MsgFolderRoot(WellknownFolder):
-    """Also known as the 'Top of Information Store' folder"""
+    """Also known as the 'Top of Information Store' folder."""
+
     DISTINGUISHED_FOLDER_ID = 'msgfolderroot'
     LOCALIZED_NAMES = {
         'zh_CN': ('信息存储顶部',),
     }
-    __slots__ = tuple()
 
 
 class MyContacts(WellknownFolder):
     CONTAINER_CLASS = 'IPF.Note'
     DISTINGUISHED_FOLDER_ID = 'mycontacts'
     supported_from = EXCHANGE_2013
-    __slots__ = tuple()
 
 
 class Notes(WellknownFolder):
@@ -296,20 +272,17 @@ class Notes(WellknownFolder):
     LOCALIZED_NAMES = {
         'da_DK': ('Noter',),
     }
-    __slots__ = tuple()
 
 
 class PeopleConnect(WellknownFolder):
     DISTINGUISHED_FOLDER_ID = 'peopleconnect'
     supported_from = EXCHANGE_2013
-    __slots__ = tuple()
 
 
 class QuickContacts(WellknownFolder):
     CONTAINER_CLASS = 'IPF.Contact.MOC.QuickContacts'
     DISTINGUISHED_FOLDER_ID = 'quickcontacts'
     supported_from = EXCHANGE_2013
-    __slots__ = tuple()
 
 
 class RecipientCache(Contacts):
@@ -318,49 +291,41 @@ class RecipientCache(Contacts):
     supported_from = EXCHANGE_2013
 
     LOCALIZED_NAMES = {}
-    __slots__ = tuple()
 
 
 class RecoverableItemsDeletions(WellknownFolder):
     DISTINGUISHED_FOLDER_ID = 'recoverableitemsdeletions'
     supported_from = EXCHANGE_2010_SP1
-    __slots__ = tuple()
 
 
 class RecoverableItemsPurges(WellknownFolder):
     DISTINGUISHED_FOLDER_ID = 'recoverableitemspurges'
     supported_from = EXCHANGE_2010_SP1
-    __slots__ = tuple()
 
 
 class RecoverableItemsRoot(WellknownFolder):
     DISTINGUISHED_FOLDER_ID = 'recoverableitemsroot'
     supported_from = EXCHANGE_2010_SP1
-    __slots__ = tuple()
 
 
 class RecoverableItemsVersions(WellknownFolder):
     DISTINGUISHED_FOLDER_ID = 'recoverableitemsversions'
     supported_from = EXCHANGE_2010_SP1
-    __slots__ = tuple()
 
 
 class SearchFolders(WellknownFolder):
     DISTINGUISHED_FOLDER_ID = 'searchfolders'
-    __slots__ = tuple()
 
 
 class ServerFailures(WellknownFolder):
     DISTINGUISHED_FOLDER_ID = 'serverfailures'
     supported_from = EXCHANGE_2013
-    __slots__ = tuple()
 
 
 class SyncIssues(WellknownFolder):
     CONTAINER_CLASS = 'IPF.Note'
     DISTINGUISHED_FOLDER_ID = 'syncissues'
     supported_from = EXCHANGE_2013
-    __slots__ = tuple()
 
 
 class ToDoSearch(WellknownFolder):
@@ -371,7 +336,6 @@ class ToDoSearch(WellknownFolder):
     LOCALIZED_NAMES = {
         None: ('To-Do Search',),
     }
-    __slots__ = tuple()
 
 
 class VoiceMail(WellknownFolder):
@@ -380,273 +344,274 @@ class VoiceMail(WellknownFolder):
     LOCALIZED_NAMES = {
         None: ('Voice Mail',),
     }
-    __slots__ = tuple()
 
 
-class NonDeleteableFolderMixin:
+class NonDeletableFolderMixin:
+    """A mixin for non-wellknown folders than that are not deletable."""
+
     @property
-    def is_deleteable(self):
+    def is_deletable(self):
         return False
 
 
-class AllContacts(NonDeleteableFolderMixin, Contacts):
+class AllContacts(NonDeletableFolderMixin, Contacts):
     CONTAINER_CLASS = 'IPF.Note'
 
     LOCALIZED_NAMES = {
         None: ('AllContacts',),
     }
-    __slots__ = tuple()
 
 
-class AllItems(NonDeleteableFolderMixin, Folder):
+class AllItems(NonDeletableFolderMixin, Folder):
     CONTAINER_CLASS = 'IPF'
 
     LOCALIZED_NAMES = {
         None: ('AllItems',),
     }
-    __slots__ = tuple()
 
 
-class Audits(NonDeleteableFolderMixin, Folder):
+class Audits(NonDeletableFolderMixin, Folder):
     LOCALIZED_NAMES = {
         None: ('Audits',),
     }
     get_folder_allowed = False
-    __slots__ = tuple()
 
 
-class CalendarLogging(NonDeleteableFolderMixin, Folder):
+class CalendarLogging(NonDeletableFolderMixin, Folder):
     LOCALIZED_NAMES = {
         None: ('Calendar Logging',),
     }
-    __slots__ = tuple()
 
 
-class CommonViews(NonDeleteableFolderMixin, Folder):
+class CommonViews(NonDeletableFolderMixin, Folder):
     DEFAULT_ITEM_TRAVERSAL_DEPTH = ASSOCIATED
     LOCALIZED_NAMES = {
         None: ('Common Views',),
     }
-    __slots__ = tuple()
 
 
-class ConversationSettings(NonDeleteableFolderMixin, Folder):
+class Companies(NonDeletableFolderMixin, Contacts):
+    DISTINGUISHED_FOLDER_ID = None
+    CONTAINTER_CLASS = 'IPF.Contact.Company'
+    LOCALIZED_NAMES = {
+        None: ('Companies',),
+    }
+
+
+class ConversationSettings(NonDeletableFolderMixin, Folder):
     CONTAINER_CLASS = 'IPF.Configuration'
     LOCALIZED_NAMES = {
         'da_DK': ('Indstillinger for samtalehandlinger',),
     }
-    __slots__ = tuple()
 
 
-class DefaultFoldersChangeHistory(NonDeleteableFolderMixin, Folder):
+class DefaultFoldersChangeHistory(NonDeletableFolderMixin, Folder):
     CONTAINER_CLASS = 'IPM.DefaultFolderHistoryItem'
     LOCALIZED_NAMES = {
         None: ('DefaultFoldersChangeHistory',),
     }
-    __slots__ = tuple()
 
 
-class DeferredAction(NonDeleteableFolderMixin, Folder):
+class DeferredAction(NonDeletableFolderMixin, Folder):
     LOCALIZED_NAMES = {
         None: ('Deferred Action',),
     }
-    __slots__ = tuple()
 
 
-class ExchangeSyncData(NonDeleteableFolderMixin, Folder):
+class ExchangeSyncData(NonDeletableFolderMixin, Folder):
     LOCALIZED_NAMES = {
         None: ('ExchangeSyncData',),
     }
-    __slots__ = tuple()
 
 
-class Files(NonDeleteableFolderMixin, Folder):
+class Files(NonDeletableFolderMixin, Folder):
     CONTAINER_CLASS = 'IPF.Files'
 
     LOCALIZED_NAMES = {
         'da_DK': ('Filer',),
     }
-    __slots__ = tuple()
 
 
-class FreebusyData(NonDeleteableFolderMixin, Folder):
+class FreebusyData(NonDeletableFolderMixin, Folder):
     LOCALIZED_NAMES = {
         None: ('Freebusy Data',),
     }
-    __slots__ = tuple()
 
 
-class Friends(NonDeleteableFolderMixin, Contacts):
+class Friends(NonDeletableFolderMixin, Contacts):
     CONTAINER_CLASS = 'IPF.Note'
 
     LOCALIZED_NAMES = {
         'de_DE': ('Bekannte',),
     }
-    __slots__ = tuple()
 
 
-class GALContacts(NonDeleteableFolderMixin, Contacts):
+class GALContacts(NonDeletableFolderMixin, Contacts):
     DISTINGUISHED_FOLDER_ID = None
     CONTAINER_CLASS = 'IPF.Contact.GalContacts'
 
     LOCALIZED_NAMES = {
         None: ('GAL Contacts',),
     }
-    __slots__ = tuple()
 
 
-class GraphAnalytics(NonDeleteableFolderMixin, Folder):
+class GraphAnalytics(NonDeletableFolderMixin, Folder):
     CONTAINER_CLASS = 'IPF.StoreItem.GraphAnalytics'
     LOCALIZED_NAMES = {
         None: ('GraphAnalytics',),
     }
-    __slots__ = tuple()
 
 
-class Location(NonDeleteableFolderMixin, Folder):
+class Location(NonDeletableFolderMixin, Folder):
     LOCALIZED_NAMES = {
         None: ('Location',),
     }
-    __slots__ = tuple()
 
 
-class MailboxAssociations(NonDeleteableFolderMixin, Folder):
+class MailboxAssociations(NonDeletableFolderMixin, Folder):
     LOCALIZED_NAMES = {
         None: ('MailboxAssociations',),
     }
-    __slots__ = tuple()
 
 
-class MyContactsExtended(NonDeleteableFolderMixin, Contacts):
+class MyContactsExtended(NonDeletableFolderMixin, Contacts):
     CONTAINER_CLASS = 'IPF.Note'
     LOCALIZED_NAMES = {
         None: ('MyContactsExtended',),
     }
-    __slots__ = tuple()
 
 
-class ParkedMessages(NonDeleteableFolderMixin, Folder):
+class OrganizationalContacts(NonDeletableFolderMixin, Contacts):
+    DISTINGUISHED_FOLDER_ID = None
+    CONTAINTER_CLASS = 'IPF.Contact.OrganizationalContacts'
+    LOCALIZED_NAMES = {
+        None: ('Organizational Contacts',),
+    }
+
+
+class ParkedMessages(NonDeletableFolderMixin, Folder):
     CONTAINER_CLASS = None
     LOCALIZED_NAMES = {
         None: ('ParkedMessages',),
     }
-    __slots__ = tuple()
 
 
-class PassThroughSearchResults(NonDeleteableFolderMixin, Folder):
+class PassThroughSearchResults(NonDeletableFolderMixin, Folder):
     CONTAINER_CLASS = 'IPF.StoreItem.PassThroughSearchResults'
     LOCALIZED_NAMES = {
         None: ('Pass-Through Search Results',),
     }
-    __slots__ = tuple()
 
 
-class PdpProfileV2Secured(NonDeleteableFolderMixin, Folder):
+class PeopleCentricConversationBuddies(NonDeletableFolderMixin, Contacts):
+    DISTINGUISHED_FOLDER_ID = None
+    CONTAINTER_CLASS = 'IPF.Contact.PeopleCentricConversationBuddies'
+    LOCALIZED_NAMES = {
+        None: ('PeopleCentricConversation Buddies',),
+    }
+
+
+class PdpProfileV2Secured(NonDeletableFolderMixin, Folder):
     CONTAINER_CLASS = 'IPF.StoreItem.PdpProfileSecured'
     LOCALIZED_NAMES = {
         None: ('PdpProfileV2Secured',),
     }
-    __slots__ = tuple()
 
 
-class Reminders(NonDeleteableFolderMixin, Folder):
+class Reminders(NonDeletableFolderMixin, Folder):
     CONTAINER_CLASS = 'Outlook.Reminder'
     LOCALIZED_NAMES = {
         'da_DK': ('Påmindelser',),
     }
-    __slots__ = tuple()
 
 
-class RSSFeeds(NonDeleteableFolderMixin, Folder):
+class RSSFeeds(NonDeletableFolderMixin, Folder):
     CONTAINER_CLASS = 'IPF.Note.OutlookHomepage'
     LOCALIZED_NAMES = {
         None: ('RSS Feeds',),
     }
-    __slots__ = tuple()
 
 
-class Schedule(NonDeleteableFolderMixin, Folder):
+class Schedule(NonDeletableFolderMixin, Folder):
     LOCALIZED_NAMES = {
         None: ('Schedule',),
     }
-    __slots__ = tuple()
 
 
-class Sharing(NonDeleteableFolderMixin, Folder):
+class Sharing(NonDeletableFolderMixin, Folder):
     CONTAINER_CLASS = 'IPF.Note'
     LOCALIZED_NAMES = {
         None: ('Sharing',),
     }
-    __slots__ = tuple()
 
 
-class Shortcuts(NonDeleteableFolderMixin, Folder):
+class Shortcuts(NonDeletableFolderMixin, Folder):
     LOCALIZED_NAMES = {
         None: ('Shortcuts',),
     }
-    __slots__ = tuple()
 
 
-class Signal(NonDeleteableFolderMixin, Folder):
+class Signal(NonDeletableFolderMixin, Folder):
     CONTAINER_CLASS = 'IPF.StoreItem.Signal'
     LOCALIZED_NAMES = {
         None: ('Signal',),
     }
-    __slots__ = tuple()
 
 
-class SmsAndChatsSync(NonDeleteableFolderMixin, Folder):
+class SmsAndChatsSync(NonDeletableFolderMixin, Folder):
     CONTAINER_CLASS = 'IPF.SmsAndChatsSync'
     LOCALIZED_NAMES = {
         None: ('SmsAndChatsSync',),
     }
-    __slots__ = tuple()
 
 
-class SpoolerQueue(NonDeleteableFolderMixin, Folder):
+class SpoolerQueue(NonDeletableFolderMixin, Folder):
     LOCALIZED_NAMES = {
         None: ('Spooler Queue',),
     }
-    __slots__ = tuple()
 
 
-class System(NonDeleteableFolderMixin, Folder):
+class System(NonDeletableFolderMixin, Folder):
     LOCALIZED_NAMES = {
         None: ('System',),
     }
     get_folder_allowed = False
-    __slots__ = tuple()
 
 
-class TemporarySaves(NonDeleteableFolderMixin, Folder):
+class System1(NonDeletableFolderMixin, Folder):
+    LOCALIZED_NAMES = {
+        None: ('System1',),
+    }
+    get_folder_allowed = False
+
+
+class TemporarySaves(NonDeletableFolderMixin, Folder):
     LOCALIZED_NAMES = {
         None: ('TemporarySaves',),
     }
-    __slots__ = tuple()
 
 
-class Views(NonDeleteableFolderMixin, Folder):
+class Views(NonDeletableFolderMixin, Folder):
     LOCALIZED_NAMES = {
         None: ('Views',),
     }
-    __slots__ = tuple()
 
 
-class WorkingSet(NonDeleteableFolderMixin, Folder):
+class WorkingSet(NonDeletableFolderMixin, Folder):
     LOCALIZED_NAMES = {
         None: ('Working Set',),
     }
-    __slots__ = tuple()
 
 
 # Folders that return 'ErrorDeleteDistinguishedFolder' when we try to delete them. I can't find any official docs
 # listing these folders.
-NON_DELETEABLE_FOLDERS = [
+NON_DELETABLE_FOLDERS = [
     AllContacts,
     AllItems,
     Audits,
     CalendarLogging,
     CommonViews,
+    Companies,
     ConversationSettings,
     DefaultFoldersChangeHistory,
     DeferredAction,
@@ -659,8 +624,10 @@ NON_DELETEABLE_FOLDERS = [
     Location,
     MailboxAssociations,
     MyContactsExtended,
+    OrganizationalContacts,
     ParkedMessages,
     PassThroughSearchResults,
+    PeopleCentricConversationBuddies,
     PdpProfileV2Secured,
     Reminders,
     RSSFeeds,
@@ -671,6 +638,7 @@ NON_DELETEABLE_FOLDERS = [
     SmsAndChatsSync,
     SpoolerQueue,
     System,
+    System1,
     TemporarySaves,
     Views,
     WorkingSet,
