@@ -1,21 +1,21 @@
 import logging
+from threading import Lock
 
-from ..errors import ErrorAccessDenied, ErrorFolderNotFound, ErrorNoPublicFolderReplicaAvailable, ErrorItemNotFound, \
-    ErrorInvalidOperation
-from ..fields import EffectiveRightsField
-from ..properties import Fields
-from ..version import EXCHANGE_2007_SP1, EXCHANGE_2010_SP1
+from .base import BaseFolder, MISSING_FOLDER_ERRORS
 from .collections import FolderCollection
-from .base import BaseFolder
-from .known_folders import MsgFolderRoot, NON_DELETEABLE_FOLDERS, WELLKNOWN_FOLDERS_IN_ROOT, \
+from .known_folders import MsgFolderRoot, NON_DELETABLE_FOLDERS, WELLKNOWN_FOLDERS_IN_ROOT, \
     WELLKNOWN_FOLDERS_IN_ARCHIVE_ROOT
 from .queryset import SingleFolderQuerySet, SHALLOW
+from ..errors import ErrorAccessDenied, ErrorFolderNotFound, ErrorInvalidOperation
+from ..fields import EffectiveRightsField
+from ..properties import EWSMeta
+from ..version import EXCHANGE_2007_SP1, EXCHANGE_2010_SP1
 
 log = logging.getLogger(__name__)
 
 
-class RootOfHierarchy(BaseFolder):
-    """Base class for folders that implement the root of a folder hierarchy"""
+class RootOfHierarchy(BaseFolder, metaclass=EWSMeta):
+    """Base class for folders that implement the root of a folder hierarchy."""
 
     # A list of wellknown, or "distinguished", folders that are belong in this folder hierarchy. See
     # https://docs.microsoft.com/en-us/dotnet/api/microsoft.exchange.webservices.data.wellknownfoldername
@@ -23,15 +23,15 @@ class RootOfHierarchy(BaseFolder):
     # 'RootOfHierarchy' subclasses must not be in this list.
     WELLKNOWN_FOLDERS = []
 
-    LOCAL_FIELDS = Fields(
-        # This folder type also has 'folder:PermissionSet' on some server versions, but requesting it sometimes causes
-        # 'ErrorAccessDenied', as reported by some users. Ignore it entirely for root folders - it's usefulness is
-        # deemed minimal at best.
-        EffectiveRightsField('effective_rights', field_uri='folder:EffectiveRights', is_read_only=True,
-                             supported_from=EXCHANGE_2007_SP1),
-    )
-    FIELDS = BaseFolder.FIELDS + LOCAL_FIELDS
-    __slots__ = tuple(f.name for f in LOCAL_FIELDS) + ('_account', '_subfolders')
+    _subfolders_lock = Lock()
+
+    # This folder type also has 'folder:PermissionSet' on some server versions, but requesting it sometimes causes
+    # 'ErrorAccessDenied', as reported by some users. Ignore it entirely for root folders - it's usefulness is
+    # deemed minimal at best.
+    effective_rights = EffectiveRightsField(field_uri='folder:EffectiveRights', is_read_only=True,
+                                            supported_from=EXCHANGE_2007_SP1)
+
+    __slots__ = '_account', '_subfolders'
 
     # A special folder that acts as the top of a folder hierarchy. Finds and caches subfolders at arbitrary depth.
     def __init__(self, **kwargs):
@@ -51,10 +51,6 @@ class RootOfHierarchy(BaseFolder):
     def parent(self):
         return None
 
-    def refresh(self):
-        self._subfolders = None
-        super().refresh()
-
     @classmethod
     def register(cls, *args, **kwargs):
         if cls is not RootOfHierarchy:
@@ -67,8 +63,10 @@ class RootOfHierarchy(BaseFolder):
             raise TypeError('For folder roots, custom fields must be registered on the RootOfHierarchy class')
         return super().deregister(*args, **kwargs)
 
-    def get_folder(self, folder_id):
-        return self._folders_map.get(folder_id, None)
+    def get_folder(self, folder):
+        if not folder.id:
+            raise ValueError("'folder' must have an ID")
+        return self._folders_map.get(folder.id, None)
 
     def add_folder(self, folder):
         if not folder.id:
@@ -89,7 +87,8 @@ class RootOfHierarchy(BaseFolder):
             pass
 
     def clear_cache(self):
-        self._subfolders = None
+        with self._subfolders_lock:
+            self._subfolders = None
 
     def get_children(self, folder):
         for f in self._folders_map.values():
@@ -100,7 +99,10 @@ class RootOfHierarchy(BaseFolder):
 
     @classmethod
     def get_distinguished(cls, account):
-        """Gets the distinguished folder for this folder class"""
+        """Get the distinguished folder for this folder class.
+
+        :param account:
+        """
         if not cls.DISTINGUISHED_FOLDER_ID:
             raise ValueError('Class %s must have a DISTINGUISHED_FOLDER_ID value' % cls)
         try:
@@ -108,17 +110,18 @@ class RootOfHierarchy(BaseFolder):
                 account=account,
                 folder=cls(account=account, name=cls.DISTINGUISHED_FOLDER_ID, is_distinguished=True)
             )
-        except ErrorFolderNotFound:
+        except MISSING_FOLDER_ERRORS:
             raise ErrorFolderNotFound('Could not find distinguished folder %s' % cls.DISTINGUISHED_FOLDER_ID)
 
     def get_default_folder(self, folder_cls):
-        # Returns the distinguished folder instance of type folder_cls belonging to this account. If no distinguished
-        # folder was found, try as best we can to return the default folder of type 'folder_cls'
+        """Return the distinguished folder instance of type folder_cls belonging to this account. If no distinguished
+        folder was found, try as best we can to return the default folder of type 'folder_cls'
+        """
         if not folder_cls.DISTINGUISHED_FOLDER_ID:
             raise ValueError("'folder_cls' %s must have a DISTINGUISHED_FOLDER_ID value" % folder_cls)
         # Use cached distinguished folder instance, but only if cache has already been prepped. This is an optimization
         # for accessing e.g. 'account.contacts' without fetching all folders of the account.
-        if self._subfolders:
+        if self._subfolders is not None:
             for f in self._folders_map.values():
                 # Require exact class, to not match subclasses, e.g. RecipientCache instead of Contacts
                 if f.__class__ == folder_cls and f.is_distinguished:
@@ -133,56 +136,54 @@ class RootOfHierarchy(BaseFolder):
             fld = folder_cls(root=self, name=folder_cls.DISTINGUISHED_FOLDER_ID, is_distinguished=True)
             fld.test_access()
             return self._folders_map.get(fld.id, fld)  # Use cached instance if available
-        except ErrorFolderNotFound:
+        except MISSING_FOLDER_ERRORS:
             # The Exchange server does not return a distinguished folder of this type
             pass
-        raise ErrorFolderNotFound('No useable default %s folders' % folder_cls)
+        raise ErrorFolderNotFound('No usable default %s folders' % folder_cls)
 
     @property
     def _folders_map(self):
         if self._subfolders is not None:
             return self._subfolders
 
-        # Map root, and all subfolders of root, at arbitrary depth by folder ID. First get distinguished folders, so we
-        # are sure to apply the correct Folder class, then fetch all subfolders of this root.
-        folders_map = {self.id: self}
-        distinguished_folders = [
-            cls(root=self, name=cls.DISTINGUISHED_FOLDER_ID, is_distinguished=True)
-            for cls in self.WELLKNOWN_FOLDERS
-            if cls.get_folder_allowed and cls.supports_version(self.account.version)
-        ]
-        for f in FolderCollection(account=self.account, folders=distinguished_folders).resolve():
-            if isinstance(f, (ErrorFolderNotFound, ErrorNoPublicFolderReplicaAvailable)):
-                # This is just a distinguished folder the server does not have
-                continue
-            if isinstance(f, ErrorInvalidOperation):
-                # This is probably a distinguished folder the server does not have. We previously tested the exact
-                # error message (f.value), but some Exchange servers return localized error messages, so that's not
-                # possible to do reliably.
-                continue
-            if isinstance(f, ErrorItemNotFound):
-                # Another way of telling us that this is a distinguished folder the server does not have
-                continue
-            if isinstance(f, ErrorAccessDenied):
-                # We may not have GetFolder access, either to this folder or at all
-                continue
-            if isinstance(f, Exception):
-                raise f
-            folders_map[f.id] = f
-        for f in SingleFolderQuerySet(account=self.account, folder=self).depth(
-                self.DEFAULT_FOLDER_TRAVERSAL_DEPTH
-        ).all():
-            if isinstance(f, ErrorAccessDenied):
-                # We may not have FindFolder access, or GetFolder access, either to this folder or at all
-                continue
-            if isinstance(f, Exception):
-                raise f
-            if f.id in folders_map:
-                # Already exists. Probably a distinguished folder
-                continue
-            folders_map[f.id] = f
-        self._subfolders = folders_map
-        return folders_map
+        with self._subfolders_lock:
+            # Map root, and all subfolders of root, at arbitrary depth by folder ID. First get distinguished folders,
+            # so we are sure to apply the correct Folder class, then fetch all subfolders of this root.
+            folders_map = {self.id: self}
+            distinguished_folders = [
+                cls(root=self, name=cls.DISTINGUISHED_FOLDER_ID, is_distinguished=True)
+                for cls in self.WELLKNOWN_FOLDERS
+                if cls.get_folder_allowed and cls.supports_version(self.account.version)
+            ]
+            for f in FolderCollection(account=self.account, folders=distinguished_folders).resolve():
+                if isinstance(f, MISSING_FOLDER_ERRORS):
+                    # This is just a distinguished folder the server does not have
+                    continue
+                if isinstance(f, ErrorInvalidOperation):
+                    # This is probably a distinguished folder the server does not have. We previously tested the exact
+                    # error message (f.value), but some Exchange servers return localized error messages, so that's not
+                    # possible to do reliably.
+                    continue
+                if isinstance(f, ErrorAccessDenied):
+                    # We may not have GetFolder access, either to this folder or at all
+                    continue
+                if isinstance(f, Exception):
+                    raise f
+                folders_map[f.id] = f
+            for f in SingleFolderQuerySet(account=self.account, folder=self).depth(
+                    self.DEFAULT_FOLDER_TRAVERSAL_DEPTH
+            ).all():
+                if isinstance(f, ErrorAccessDenied):
+                    # We may not have FindFolder access, or GetFolder access, either to this folder or at all
+                    continue
+                if isinstance(f, Exception):
+                    raise f
+                if f.id in folders_map:
+                    # Already exists. Probably a distinguished folder
+                    continue
+                folders_map[f.id] = f
+            self._subfolders = folders_map
+            return folders_map
 
     @classmethod
     def from_xml(cls, elem, account):
@@ -192,11 +193,12 @@ class RootOfHierarchy(BaseFolder):
 
     @classmethod
     def folder_cls_from_folder_name(cls, folder_name, locale):
-        """Returns the folder class that matches a localized folder name.
+        """Return the folder class that matches a localized folder name.
 
-        locale is a string, e.g. 'da_DK'
+        :param folder_name:
+        :param locale: a string, e.g. 'da_DK'
         """
-        for folder_cls in cls.WELLKNOWN_FOLDERS + NON_DELETEABLE_FOLDERS:
+        for folder_cls in cls.WELLKNOWN_FOLDERS + NON_DELETABLE_FOLDERS:
             if folder_name.lower() in folder_cls.localized_names(locale):
                 return folder_cls
         raise KeyError()
@@ -209,10 +211,10 @@ class RootOfHierarchy(BaseFolder):
 
 
 class Root(RootOfHierarchy):
-    """The root of the standard folder hierarchy"""
+    """The root of the standard folder hierarchy."""
+
     DISTINGUISHED_FOLDER_ID = 'root'
     WELLKNOWN_FOLDERS = WELLKNOWN_FOLDERS_IN_ROOT
-    __slots__ = tuple()
 
     @property
     def tois(self):
@@ -223,7 +225,7 @@ class Root(RootOfHierarchy):
     def get_default_folder(self, folder_cls):
         try:
             return super().get_default_folder(folder_cls)
-        except ErrorFolderNotFound:
+        except MISSING_FOLDER_ERRORS:
             pass
 
         # Try to pick a suitable default folder. we do this by:
@@ -240,12 +242,13 @@ class Root(RootOfHierarchy):
                 log.debug('Found cached %s folder with default distinguished name', folder_cls)
                 return f
 
-        # Try direct children of TOIS first. TOIS might not exist.
-        try:
-            return self._get_candidate(folder_cls=folder_cls, folder_coll=self.tois.children)
-        except ErrorFolderNotFound:
-            # No candidates, or TOIS does ot exist
-            pass
+        # Try direct children of TOIS first, unless we're trying to get the TOIS folder. TOIS might not exist.
+        if folder_cls != MsgFolderRoot:
+            try:
+                return self._get_candidate(folder_cls=folder_cls, folder_coll=self.tois.children)
+            except MISSING_FOLDER_ERRORS:
+                # No candidates, or TOIS does not exist, or we don't have access
+                pass
 
         # No candidates in TOIS. Try direct children of root.
         return self._get_candidate(folder_cls=folder_cls, folder_coll=self.children)
@@ -268,15 +271,15 @@ class Root(RootOfHierarchy):
             else:
                 log.debug('Found cached %s folder with localized name', folder_cls)
             return candidates[0]
-        raise ErrorFolderNotFound('No useable default %s folders' % folder_cls)
+        raise ErrorFolderNotFound('No usable default %s folders' % folder_cls)
 
 
 class PublicFoldersRoot(RootOfHierarchy):
-    """The root of the public folders hierarchy. Not available on all mailboxes"""
+    """The root of the public folders hierarchy. Not available on all mailboxes."""
+
     DISTINGUISHED_FOLDER_ID = 'publicfoldersroot'
     DEFAULT_FOLDER_TRAVERSAL_DEPTH = SHALLOW
     supported_from = EXCHANGE_2007_SP1
-    __slots__ = tuple()
 
     def get_children(self, folder):
         # EWS does not allow deep traversal of public folders, so self._folders_map will only populate the top-level
@@ -286,8 +289,7 @@ class PublicFoldersRoot(RootOfHierarchy):
         children = list(super().get_children(folder=folder))
         if children:
             # Return a generator like our parent does
-            for f in children:
-                yield f
+            yield from children
             return
 
         # Also return early if the server told us that there are no child folders.
@@ -307,16 +309,16 @@ class PublicFoldersRoot(RootOfHierarchy):
             pass
 
         # Let's update the cache atomically, to avoid partial reads of the cache.
-        self._subfolders.update(children_map)
+        with self._subfolders_lock:
+            self._subfolders.update(children_map)
 
         # Child folders have been cached now. Try super().get_children() again.
-        for f in super().get_children(folder=folder):
-            yield f
+        yield from super().get_children(folder=folder)
 
 
 class ArchiveRoot(RootOfHierarchy):
-    """The root of the archive folders hierarchy. Not available on all mailboxes"""
+    """The root of the archive folders hierarchy. Not available on all mailboxes."""
+
     DISTINGUISHED_FOLDER_ID = 'archiveroot'
     supported_from = EXCHANGE_2010_SP1
     WELLKNOWN_FOLDERS = WELLKNOWN_FOLDERS_IN_ARCHIVE_ROOT
-    __slots__ = tuple()
