@@ -3,7 +3,7 @@ import logging
 from ..extended_properties import ExtendedProperty
 from ..fields import BooleanField, ExtendedPropertyField, BodyField, MailboxField, MailboxListField, EWSElementField, \
     CharField, IdElementField, AttachmentField
-from ..properties import InvalidField, IdChangeKeyMixIn, EWSElement, ReferenceItemId, ItemId, Fields
+from ..properties import InvalidField, IdChangeKeyMixIn, EWSElement, ReferenceItemId, ItemId, EWSMeta
 from ..services import CreateItem
 from ..util import require_account
 from ..version import EXCHANGE_2007_SP1
@@ -62,18 +62,21 @@ MOVE_TO_DELETED_ITEMS = 'MoveToDeletedItems'
 DELETE_TYPE_CHOICES = (HARD_DELETE, SOFT_DELETE, MOVE_TO_DELETED_ITEMS)
 
 
-class RegisterMixIn(IdChangeKeyMixIn):
-    """Base class for classes that can change their list of supported fields dynamically"""
+class RegisterMixIn(IdChangeKeyMixIn, metaclass=EWSMeta):
+    """Base class for classes that can change their list of supported fields dynamically."""
 
     # This class implements dynamic fields on an element class, so we need to include __dict__ in __slots__
-    __slots__ = ('__dict__',)
+    __slots__ = '__dict__',
 
     INSERT_AFTER_FIELD = None
 
     @classmethod
     def register(cls, attr_name, attr_cls):
-        """
-        Register a custom extended property in this item class so they can be accessed just like any other attribute
+        """Register a custom extended property in this item class so they can be accessed just like any other attribute
+
+        :param attr_name:
+        :param attr_cls:
+        :return:
         """
         if not cls.INSERT_AFTER_FIELD:
             raise ValueError('Class %s is missing INSERT_AFTER_FIELD value' % cls)
@@ -96,8 +99,10 @@ class RegisterMixIn(IdChangeKeyMixIn):
 
     @classmethod
     def deregister(cls, attr_name):
-        """
-        De-register an extended property that has been registered with register()
+        """De-register an extended property that has been registered with register().
+
+        :param attr_name:
+        :return:
         """
         try:
             field = cls.get_field_by_fieldname(attr_name)
@@ -108,20 +113,22 @@ class RegisterMixIn(IdChangeKeyMixIn):
         cls.remove_field(field)
 
 
-class BaseItem(RegisterMixIn):
-    """Base class for all other classes that implement EWS items"""
+class BaseItem(RegisterMixIn, metaclass=EWSMeta):
+    """Base class for all other classes that implement EWS items."""
+
     ID_ELEMENT_CLS = ItemId
+    _id = IdElementField(field_uri='item:ItemId', value_cls=ID_ELEMENT_CLS)
 
-    FIELDS = Fields(
-        IdElementField('_id', field_uri='item:ItemId', value_cls=ID_ELEMENT_CLS),
-    )
-
-    __slots__ = tuple(f.name for f in FIELDS) + ('account', 'folder')
+    __slots__ = 'account', 'folder'
 
     def __init__(self, **kwargs):
-        # 'account' is optional but allows calling 'send()' and 'delete()'
-        # 'folder' is optional but allows calling 'save()'. If 'folder' has an account, and 'account' is not set,
-        # we use folder.account.
+        """Pick out optional 'account' and 'folder' kwargs, and pass the rest to the parent class.
+
+        :param kwargs:
+            'account' is optional but allows calling 'send()' and 'delete()'
+            'folder' is optional but allows calling 'save()'. If 'folder' has an account, and 'account' is not set,
+                we use folder.account.
+        """
         from ..folders import BaseFolder
         from ..account import Account
         self.account = kwargs.pop('account', None)
@@ -145,28 +152,24 @@ class BaseItem(RegisterMixIn):
         item.account = account
         return item
 
-    def to_id_xml(self, version):
-        return self._id.to_xml(version=version)
 
+class BaseReplyItem(EWSElement, metaclass=EWSMeta):
+    """Base class for reply/forward elements that share the same fields."""
 
-class BaseReplyItem(EWSElement):
-    """Base class for reply/forward elements that share the same fields"""
-    FIELDS = Fields(
-        CharField('subject', field_uri='Subject'),
-        BodyField('body', field_uri='Body'),  # Accepts and returns Body or HTMLBody instances
-        MailboxListField('to_recipients', field_uri='ToRecipients'),
-        MailboxListField('cc_recipients', field_uri='CcRecipients'),
-        MailboxListField('bcc_recipients', field_uri='BccRecipients'),
-        BooleanField('is_read_receipt_requested', field_uri='IsReadReceiptRequested'),
-        BooleanField('is_delivery_receipt_requested', field_uri='IsDeliveryReceiptRequested'),
-        MailboxField('author', field_uri='From'),
-        EWSElementField('reference_item_id', value_cls=ReferenceItemId),
-        BodyField('new_body', field_uri='NewBodyContent'),  # Accepts and returns Body or HTMLBody instances
-        MailboxField('received_by', field_uri='ReceivedBy', supported_from=EXCHANGE_2007_SP1),
-        MailboxField('received_by_representing', field_uri='ReceivedRepresenting', supported_from=EXCHANGE_2007_SP1),
-    )
+    subject = CharField(field_uri='Subject')
+    body = BodyField(field_uri='Body')  # Accepts and returns Body or HTMLBody instances
+    to_recipients = MailboxListField(field_uri='ToRecipients')
+    cc_recipients = MailboxListField(field_uri='CcRecipients')
+    bcc_recipients = MailboxListField(field_uri='BccRecipients')
+    is_read_receipt_requested = BooleanField(field_uri='IsReadReceiptRequested')
+    is_delivery_receipt_requested = BooleanField(field_uri='IsDeliveryReceiptRequested')
+    author = MailboxField(field_uri='From')
+    reference_item_id = EWSElementField(value_cls=ReferenceItemId)
+    new_body = BodyField(field_uri='NewBodyContent')  # Accepts and returns Body or HTMLBody instances
+    received_by = MailboxField(field_uri='ReceivedBy', supported_from=EXCHANGE_2007_SP1)
+    received_by_representing = MailboxField(field_uri='ReceivedRepresenting', supported_from=EXCHANGE_2007_SP1)
 
-    __slots__ = tuple(f.name for f in FIELDS) + ('account',)
+    __slots__ = 'account',
 
     def __init__(self, **kwargs):
         # 'account' is optional but allows calling 'send()' and 'save()'
@@ -178,44 +181,37 @@ class BaseReplyItem(EWSElement):
 
     @require_account
     def send(self, save_copy=True, copy_to_folder=None):
-        if copy_to_folder:
-            if not save_copy:
-                raise AttributeError("'save_copy' must be True when 'copy_to_folder' is set")
+        if copy_to_folder and not save_copy:
+            raise AttributeError("'save_copy' must be True when 'copy_to_folder' is set")
         message_disposition = SEND_AND_SAVE_COPY if save_copy else SEND_ONLY
-        CreateItem(account=self.account).get(
+        return CreateItem(account=self.account).get(
             items=[self],
             folder=copy_to_folder,
             message_disposition=message_disposition,
             send_meeting_invitations=SEND_TO_NONE,
-            expect_result=False,
         )
 
     @require_account
     def save(self, folder):
+        """Save the item for later modification. You may want to use account.drafts as the folder.
+
+        :param folder:
+        :return:
         """
-        save reply/forward and retrieve the item result for further modification,
-        you may want to use account.drafts as the folder.
-        """
-        res = CreateItem(account=self.account).get(
+        return CreateItem(account=self.account).get(
             items=[self],
             folder=folder,
             message_disposition=SAVE_ONLY,
             send_meeting_invitations=SEND_TO_NONE,
         )
-        return BulkCreateResult.from_xml(elem=res, account=self)
 
 
 class BulkCreateResult(BaseItem):
-    """A dummy class to store return values from a CreateItem service call"""
-    LOCAL_FIELDS = Fields(
-        AttachmentField('attachments', field_uri='item:Attachments'),  # ItemAttachment or FileAttachment
-    )
-    FIELDS = BaseItem.FIELDS + LOCAL_FIELDS
+    """A dummy class to store return values from a CreateItem service call."""
 
-    __slots__ = tuple(f.name for f in LOCAL_FIELDS)
+    attachments = AttachmentField(field_uri='item:Attachments')  # ItemAttachment or FileAttachment
 
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
-        # pylint: disable=access-member-before-definition
         if self.attachments is None:
             self.attachments = []
