@@ -1,85 +1,81 @@
 import logging
 
+from .base import BaseItem, SAVE_ONLY, SEND_AND_SAVE_COPY, ID_ONLY, SEND_TO_NONE, \
+    AUTO_RESOLVE, SOFT_DELETE, HARD_DELETE, ALL_OCCURRENCIES, MOVE_TO_DELETED_ITEMS
 from ..fields import BooleanField, IntegerField, TextField, CharListField, ChoiceField, URIField, BodyField, \
     DateTimeField, MessageHeaderField, AttachmentField, Choice, EWSElementField, EffectiveRightsField, CultureField, \
     CharField, MimeContentField, FieldPath
-from ..properties import ConversationId, ParentFolderId, ReferenceItemId, OccurrenceItemId, RecurringMasterItemId,\
-    Fields
+from ..properties import ConversationId, ParentFolderId, ReferenceItemId, OccurrenceItemId, RecurringMasterItemId, \
+    ResponseObjects, Fields
 from ..services import GetItem, CreateItem, UpdateItem, DeleteItem, MoveItem, CopyItem, ArchiveItem
 from ..util import is_iterable, require_account, require_id
 from ..version import EXCHANGE_2010, EXCHANGE_2013
-from .base import BaseItem, BulkCreateResult, SAVE_ONLY, SEND_ONLY, SEND_AND_SAVE_COPY, ID_ONLY, SEND_TO_NONE, \
-    AUTO_RESOLVE, SOFT_DELETE, HARD_DELETE, ALL_OCCURRENCIES, MOVE_TO_DELETED_ITEMS
 
 log = logging.getLogger(__name__)
 
 
 class Item(BaseItem):
-    """
-    MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/item
-    """
+    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/item"""
+
     ELEMENT_NAME = 'Item'
 
-    LOCAL_FIELDS = Fields(
-        MimeContentField('mime_content', field_uri='item:MimeContent', is_read_only_after_send=True),
-        EWSElementField('parent_folder_id', field_uri='item:ParentFolderId', value_cls=ParentFolderId,
-                        is_read_only=True),
-        CharField('item_class', field_uri='item:ItemClass', is_read_only=True),
-        CharField('subject', field_uri='item:Subject'),
-        ChoiceField('sensitivity', field_uri='item:Sensitivity', choices={
-            Choice('Normal'), Choice('Personal'), Choice('Private'), Choice('Confidential')
-        }, is_required=True, default='Normal'),
-        TextField('text_body', field_uri='item:TextBody', is_read_only=True, supported_from=EXCHANGE_2013),
-        BodyField('body', field_uri='item:Body'),  # Accepts and returns Body or HTMLBody instances
-        AttachmentField('attachments', field_uri='item:Attachments'),  # ItemAttachment or FileAttachment
-        DateTimeField('datetime_received', field_uri='item:DateTimeReceived', is_read_only=True),
-        IntegerField('size', field_uri='item:Size', is_read_only=True),  # Item size in bytes
-        CharListField('categories', field_uri='item:Categories'),
-        ChoiceField('importance', field_uri='item:Importance', choices={
-            Choice('Low'), Choice('Normal'), Choice('High')
-        }, is_required=True, default='Normal'),
-        TextField('in_reply_to', field_uri='item:InReplyTo'),
-        BooleanField('is_submitted', field_uri='item:IsSubmitted', is_read_only=True),
-        BooleanField('is_draft', field_uri='item:IsDraft', is_read_only=True),
-        BooleanField('is_from_me', field_uri='item:IsFromMe', is_read_only=True),
-        BooleanField('is_resend', field_uri='item:IsResend', is_read_only=True),
-        BooleanField('is_unmodified', field_uri='item:IsUnmodified', is_read_only=True),
-        MessageHeaderField('headers', field_uri='item:InternetMessageHeaders', is_read_only=True),
-        DateTimeField('datetime_sent', field_uri='item:DateTimeSent', is_read_only=True),
-        DateTimeField('datetime_created', field_uri='item:DateTimeCreated', is_read_only=True),
-        # Placeholder for ResponseObjects
-        DateTimeField('reminder_due_by', field_uri='item:ReminderDueBy', is_required_after_save=True,
-                      is_searchable=False),
-        BooleanField('reminder_is_set', field_uri='item:ReminderIsSet', is_required=True, default=False),
-        IntegerField('reminder_minutes_before_start', field_uri='item:ReminderMinutesBeforeStart',
-                     is_required_after_save=True, min=0, default=0),
-        TextField('display_cc', field_uri='item:DisplayCc', is_read_only=True),
-        TextField('display_to', field_uri='item:DisplayTo', is_read_only=True),
-        BooleanField('has_attachments', field_uri='item:HasAttachments', is_read_only=True),
-        # ExtendedProperty fields go here
-        CultureField('culture', field_uri='item:Culture', is_required_after_save=True, is_searchable=False),
-        EffectiveRightsField('effective_rights', field_uri='item:EffectiveRights', is_read_only=True),
-        CharField('last_modified_name', field_uri='item:LastModifiedName', is_read_only=True),
-        DateTimeField('last_modified_time', field_uri='item:LastModifiedTime', is_read_only=True),
-        BooleanField('is_associated', field_uri='item:IsAssociated', is_read_only=True, supported_from=EXCHANGE_2010),
-        URIField('web_client_read_form_query_string', field_uri='item:WebClientReadFormQueryString',
-                 is_read_only=True, supported_from=EXCHANGE_2010),
-        URIField('web_client_edit_form_query_string', field_uri='item:WebClientEditFormQueryString',
-                 is_read_only=True, supported_from=EXCHANGE_2010),
-        EWSElementField('conversation_id', field_uri='item:ConversationId', value_cls=ConversationId,
-                        is_read_only=True, supported_from=EXCHANGE_2010),
-        BodyField('unique_body', field_uri='item:UniqueBody', is_read_only=True, supported_from=EXCHANGE_2010),
-    )
-    FIELDS = LOCAL_FIELDS[0:1] + BaseItem.FIELDS + LOCAL_FIELDS[1:]
+    mime_content = MimeContentField(field_uri='item:MimeContent', is_read_only_after_send=True)
+    _id = BaseItem.FIELDS['_id']
+    parent_folder_id = EWSElementField(field_uri='item:ParentFolderId', value_cls=ParentFolderId, is_read_only=True)
+    item_class = CharField(field_uri='item:ItemClass', is_read_only=True)
+    subject = CharField(field_uri='item:Subject')
+    sensitivity = ChoiceField(field_uri='item:Sensitivity', choices={
+        Choice('Normal'), Choice('Personal'), Choice('Private'), Choice('Confidential')
+    }, is_required=True, default='Normal')
+    text_body = TextField(field_uri='item:TextBody', is_read_only=True, supported_from=EXCHANGE_2013)
+    body = BodyField(field_uri='item:Body')  # Accepts and returns Body or HTMLBody instances
+    attachments = AttachmentField(field_uri='item:Attachments')  # ItemAttachment or FileAttachment
+    datetime_received = DateTimeField(field_uri='item:DateTimeReceived', is_read_only=True)
+    size = IntegerField(field_uri='item:Size', is_read_only=True)  # Item size in bytes
+    categories = CharListField(field_uri='item:Categories')
+    importance = ChoiceField(field_uri='item:Importance', choices={
+        Choice('Low'), Choice('Normal'), Choice('High')
+    }, is_required=True, default='Normal')
+    in_reply_to = TextField(field_uri='item:InReplyTo')
+    is_submitted = BooleanField(field_uri='item:IsSubmitted', is_read_only=True)
+    is_draft = BooleanField(field_uri='item:IsDraft', is_read_only=True)
+    is_from_me = BooleanField(field_uri='item:IsFromMe', is_read_only=True)
+    is_resend = BooleanField(field_uri='item:IsResend', is_read_only=True)
+    is_unmodified = BooleanField(field_uri='item:IsUnmodified', is_read_only=True)
+    headers = MessageHeaderField(field_uri='item:InternetMessageHeaders', is_read_only=True)
+    datetime_sent = DateTimeField(field_uri='item:DateTimeSent', is_read_only=True)
+    datetime_created = DateTimeField(field_uri='item:DateTimeCreated', is_read_only=True)
+    response_objects = EWSElementField(field_uri='item:ResponseObjects', value_cls=ResponseObjects,
+                                       is_read_only=True,)
+    # Placeholder for ResponseObjects
+    reminder_due_by = DateTimeField(field_uri='item:ReminderDueBy', is_required_after_save=True, is_searchable=False)
+    reminder_is_set = BooleanField(field_uri='item:ReminderIsSet', is_required=True, default=False)
+    reminder_minutes_before_start = IntegerField(field_uri='item:ReminderMinutesBeforeStart',
+                                                 is_required_after_save=True, min=0, default=0)
+    display_cc = TextField(field_uri='item:DisplayCc', is_read_only=True)
+    display_to = TextField(field_uri='item:DisplayTo', is_read_only=True)
+    has_attachments = BooleanField(field_uri='item:HasAttachments', is_read_only=True)
+    # ExtendedProperty fields go here
+    culture = CultureField(field_uri='item:Culture', is_required_after_save=True, is_searchable=False)
+    effective_rights = EffectiveRightsField(field_uri='item:EffectiveRights', is_read_only=True)
+    last_modified_name = CharField(field_uri='item:LastModifiedName', is_read_only=True)
+    last_modified_time = DateTimeField(field_uri='item:LastModifiedTime', is_read_only=True)
+    is_associated = BooleanField(field_uri='item:IsAssociated', is_read_only=True, supported_from=EXCHANGE_2010)
+    web_client_read_form_query_string = URIField(field_uri='item:WebClientReadFormQueryString', is_read_only=True,
+                                                 supported_from=EXCHANGE_2010)
+    web_client_edit_form_query_string = URIField(field_uri='item:WebClientEditFormQueryString', is_read_only=True,
+                                                 supported_from=EXCHANGE_2010)
+    conversation_id = EWSElementField(field_uri='item:ConversationId', value_cls=ConversationId,
+                                      is_read_only=True, supported_from=EXCHANGE_2010)
+    unique_body = BodyField(field_uri='item:UniqueBody', is_read_only=True, supported_from=EXCHANGE_2010)
 
-    __slots__ = tuple(f.name for f in LOCAL_FIELDS)
+    FIELDS = Fields()
 
     # Used to register extended properties
     INSERT_AFTER_FIELD = 'has_attachments'
 
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
-        # pylint: disable=access-member-before-definition
         if self.attachments:
             for a in self.attachments:
                 if a.parent_item:
@@ -92,6 +88,7 @@ class Item(BaseItem):
             self.attachments = []
 
     def save(self, update_fields=None, conflict_resolution=AUTO_RESOLVE, send_meeting_invitations=SEND_TO_NONE):
+        from .task import Task
         if self.id:
             item_id, changekey = self._update(
                 update_fieldnames=update_fields,
@@ -99,9 +96,14 @@ class Item(BaseItem):
                 conflict_resolution=conflict_resolution,
                 send_meeting_invitations=send_meeting_invitations
             )
-            if self.id != item_id and not isinstance(self._id, (OccurrenceItemId, RecurringMasterItemId)):
+            if self.id != item_id \
+                    and not isinstance(self._id, (OccurrenceItemId, RecurringMasterItemId)) \
+                    and not isinstance(self, Task):
                 # When we update an item with an OccurrenceItemId as ID, EWS returns the ID of the occurrence, so
                 # the ID of this item changes.
+                #
+                # When we update certain fields on a task, the ID may change. A full description is available at
+                # https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/updateitem-operation-task
                 raise ValueError("'id' mismatch in returned update response")
             # Don't check that changekeys are different. No-op saves will sometimes leave the changekey intact
             self._id = self.ID_ELEMENT_CLS(item_id, changekey)
@@ -128,17 +130,14 @@ class Item(BaseItem):
 
     @require_account
     def _create(self, message_disposition, send_meeting_invitations):
-        # Return a BulkCreateResult because we want to return the ID of both the main item *and* attachments
-        res = CreateItem(account=self.account).get(
+        # Return a BulkCreateResult because we want to return the ID of both the main item *and* attachments. In send
+        # and send-and-save-copy mode, the server does not return an ID, so we just return True.
+        return CreateItem(account=self.account).get(
             items=[self],
             folder=self.folder,
             message_disposition=message_disposition,
             send_meeting_invitations=send_meeting_invitations,
-            expect_result=message_disposition not in (SEND_ONLY, SEND_AND_SAVE_COPY),
         )
-        if res is None:
-            return
-        return BulkCreateResult.from_xml(elem=res, account=self)
 
     def _update_fieldnames(self):
         from .contact import Contact, DistributionList
@@ -151,10 +150,11 @@ class Item(BaseItem):
             if f.is_read_only:
                 # These cannot be changed
                 continue
-            if f.is_required or f.is_required_after_save:
-                if getattr(self, f.name) is None or (f.is_list and not getattr(self, f.name)):
-                    # These are required and cannot be deleted
-                    continue
+            if (f.is_required or f.is_required_after_save) and (
+                    getattr(self, f.name) is None or (f.is_list and not getattr(self, f.name))
+            ):
+                # These are required and cannot be deleted
+                continue
             if not self.is_draft and f.is_read_only_after_send:
                 # These cannot be changed when the item is no longer a draft
                 continue
@@ -174,7 +174,7 @@ class Item(BaseItem):
         if not update_fieldnames:
             # The fields to update was not specified explicitly. Update all fields where update is possible
             update_fieldnames = self._update_fieldnames()
-        res = UpdateItem(account=self.account).get(
+        return UpdateItem(account=self.account).get(
             items=[(self, update_fieldnames)],
             message_disposition=message_disposition,
             conflict_resolution=conflict_resolution,
@@ -182,9 +182,6 @@ class Item(BaseItem):
             suppress_read_receipts=True,
             expect_result=message_disposition != SEND_AND_SAVE_COPY,
         )
-        if res is None:
-            return
-        return Item.id_from_xml(res)
 
     @require_id
     def refresh(self):
@@ -193,9 +190,7 @@ class Item(BaseItem):
         additional_fields = {
             FieldPath(field=f) for f in Folder(root=self.account.root).allowed_item_fields(version=self.account.version)
         }
-
-        elem = GetItem(account=self.account).get(items=[self], additional_fields=additional_fields, shape=ID_ONLY)
-        res = Folder.item_model_from_tag(elem.tag).from_xml(elem=elem, account=self.account)
+        res = GetItem(account=self.account).get(items=[self], additional_fields=additional_fields, shape=ID_ONLY)
         if self.id != res.id and not isinstance(self._id, (OccurrenceItemId, RecurringMasterItemId)):
             # When we refresh an item with an OccurrenceItemId as ID, EWS returns the ID of the occurrence, so
             # the ID of this item changes.
@@ -205,19 +200,16 @@ class Item(BaseItem):
         # 'parent_item' should point to 'self', not 'fresh_item'. That way, 'fresh_item' can be garbage collected.
         for a in self.attachments:
             a.parent_item = self
-        del res
+        return self
 
     @require_id
     def copy(self, to_folder):
-        res = CopyItem(account=self.account).get(
+        # If 'to_folder' is a public folder or a folder in a different mailbox then None is returned
+        return CopyItem(account=self.account).get(
             items=[self],
             to_folder=to_folder,
             expect_result=None,
         )
-        if res is None:
-            # Assume 'to_folder' is a public folder or a folder in a different mailbox
-            return
-        return Item.id_from_xml(res)
 
     @require_id
     def move(self, to_folder):
@@ -230,7 +222,7 @@ class Item(BaseItem):
             # Assume 'to_folder' is a public folder or a folder in a different mailbox
             self._id = None
             return
-        self._id = self.ID_ELEMENT_CLS(*Item.id_from_xml(res))
+        self._id = self.ID_ELEMENT_CLS(*res)
         self.folder = to_folder
 
     def move_to_trash(self, send_meeting_cancellations=SEND_TO_NONE, affected_task_occurrences=ALL_OCCURRENCIES,
@@ -268,7 +260,7 @@ class Item(BaseItem):
 
     @require_id
     def archive(self, to_folder):
-        return ArchiveItem(account=self.account).get(items=[self], to_folder=to_folder)
+        return ArchiveItem(account=self.account).get(items=[self], to_folder=to_folder, expect_result=True)
 
     def attach(self, attachments):
         """Add an attachment, or a list of attachments, to this item. If the item has already been saved, the
@@ -276,6 +268,8 @@ class Item(BaseItem):
         be created on the server when the item is saved.
 
         Adding attachments to an existing item will update the changekey of the item.
+
+        :param attachments:
         """
         if not is_iterable(attachments, generators_allowed=True):
             attachments = [attachments]
@@ -294,6 +288,8 @@ class Item(BaseItem):
         simply not be created on the server the item is saved.
 
         Removing attachments from an existing item will update the changekey of the item.
+
+        :param attachments:
         """
         if not is_iterable(attachments, generators_allowed=True):
             attachments = [attachments]
