@@ -4,11 +4,12 @@ A protocol is an endpoint for EWS service connections. It contains all necessary
 Protocols should be accessed through an Account, and are either created from a default Configuration or autodiscovered
 when creating an Account.
 """
+import abc
 import datetime
 import logging
 import os
-from threading import Lock
 from queue import LifoQueue, Empty, Full
+from threading import Lock
 
 import requests.adapters
 import requests.sessions
@@ -17,8 +18,9 @@ from oauthlib.oauth2 import BackendApplicationClient, WebApplicationClient
 from requests_oauthlib import OAuth2Session
 
 from .credentials import OAuth2AuthorizationCodeCredentials, OAuth2Credentials
-from .errors import TransportError, SessionPoolMinSizeReached
-from .properties import FreeBusyViewOptions, MailboxData, TimeWindow, TimeZone
+from .errors import TransportError, SessionPoolMinSizeReached, SessionPoolMaxSizeReached, RateLimitError, CASError, \
+    ErrorInvalidSchemaVersionForMailboxVersion, UnauthorizedError, MalformedResponseError
+from .properties import FreeBusyViewOptions, MailboxData, TimeWindow, TimeZone, RoomList, DLMailbox
 from .services import GetServerTimeZones, GetRoomLists, GetRooms, ResolveNames, GetUserAvailability, \
     GetSearchableMailboxes, ExpandDL, ConvertId
 from .transport import get_auth_instance, get_service_authtype, NTLM, OAUTH2, CREDENTIALS_REQUIRED, DEFAULT_HEADERS
@@ -32,7 +34,7 @@ def close_connections():
 
 
 class BaseProtocol:
-    """Base class for Protocol which implements the bare essentials"""
+    """Base class for Protocol which implements the bare essentials."""
 
     # The maximum number of sessions (== TCP connections, see below) we will open to this service endpoint. Keep this
     # low unless you have an agreement with the Exchange admin on the receiving end to hammer the server and
@@ -41,12 +43,12 @@ class BaseProtocol:
     SESSION_POOLSIZE = 1
     # We want only 1 TCP connection per Session object. We may have lots of different credentials hitting the server and
     # each credential needs its own session (NTLM auth will only send credentials once and then secure the connection,
-    # so a connection can only handle requests for one credential). Having multiple connections ser Session could
+    # so a connection can only handle requests for one credential). Having multiple connections per Session could
     # quickly exhaust the maximum number of concurrent connections the Exchange server allows from one client.
     CONNECTIONS_PER_SESSION = 1
     # The number of times a session may be reused before creating a new session object. 'None' means "infinite".
     # Discarding sessions after a certain number of usages may limit memory leaks in the Session object.
-    MAX_SESSION_USAGE_COUNT = 100
+    MAX_SESSION_USAGE_COUNT = None
     # Timeout for HTTP requests
     TIMEOUT = 120
 
@@ -63,16 +65,13 @@ class BaseProtocol:
         if not config.service_endpoint:
             raise AttributeError("'config.service_endpoint' must be set")
         self.config = config
-        self._session_pool_size = self.SESSION_POOLSIZE
-
-        # Autodetect authentication type if necessary
-        if self.config.auth_type is None:
-            self.config.auth_type = self.get_auth_type()
+        self._session_pool_size = 0
+        self._session_pool_maxsize = config.max_connections or self.SESSION_POOLSIZE
 
         # Try to behave nicely with the remote server. We want to keep the connection open between requests.
         # We also want to re-use sessions, to avoid the NTLM auth handshake on every request. We must know the
-        # authentication method to create a session pool.
-        self._session_pool = self._create_session_pool()
+        # authentication method to create sessions.
+        self._session_pool = LifoQueue()
         self._session_pool_lock = Lock()
 
     @property
@@ -90,11 +89,10 @@ class BaseProtocol:
     @credentials.setter
     def credentials(self, value):
         # We are updating credentials, but that doesn't automatically propagate to the session objects. The simplest
-        # solution is to just kill the session pool and rebuild it.
+        # solution is to just kill the sessions in the pool.
         with self._session_pool_lock:
             self.config._credentials = value
             self.close()
-            self._session_pool = self._create_session_pool()
 
     @property
     def retry_policy(self):
@@ -114,7 +112,7 @@ class BaseProtocol:
     def __setstate__(self, state):
         # Restore the session pool and lock
         self.__dict__.update(state)
-        self._session_pool = self._create_session_pool()
+        self._session_pool = LifoQueue()
         self._session_pool_lock = Lock()
 
     def __del__(self):
@@ -129,7 +127,9 @@ class BaseProtocol:
         log.debug('Server %s: Closing sessions', self.server)
         while True:
             try:
-                self._session_pool.get(block=False).close()
+                session = self._session_pool.get(block=False)
+                self.close_session(session)
+                self._session_pool_size -= 1
             except Empty:
                 break
 
@@ -143,21 +143,25 @@ class BaseProtocol:
             max_retries=0,
         )
 
-    def get_auth_type(self):
-        # Autodetect and return authentication type
-        raise NotImplementedError()
-
-    def _create_session_pool(self):
-        # Create a pool to reuse sessions containing connections to the server
-        session_pool = LifoQueue(maxsize=self._session_pool_size)
-        for _ in range(self._session_pool_size):
-            session_pool.put(self.create_session(), block=False)
-        return session_pool
-
     @property
     def session_pool_size(self):
         return self._session_pool_size
 
+    def increase_poolsize(self):
+        """Increases the session pool size. We increase by one session per call."""
+        # Create a single session and insert it into the pool. We need to protect this with a lock while we are changing
+        # the pool size variable, to avoid race conditions. We must not exceed the pool size limit.
+        if self._session_pool_size == self._session_pool_maxsize:
+            raise SessionPoolMaxSizeReached('Session pool size cannot be increased further')
+        with self._session_pool_lock:
+            if self._session_pool_size >= self._session_pool_maxsize:
+                log.debug('Session pool size was increased in another thread')
+                return
+            log.debug('Server %s: Increasing session pool size from %s to %s', self.server, self._session_pool_size,
+                      self._session_pool_size + 1)
+            self._session_pool.put(self.create_session(), block=False)
+            self._session_pool_size += 1
+
     def decrease_poolsize(self):
         """Decreases the session pool size in response to error messages from the server requesting to rate-limit
         requests. We decrease by one session per call.
@@ -170,23 +174,35 @@ class BaseProtocol:
             if self._session_pool_size <= 1:
                 log.debug('Session pool size was decreased in another thread')
                 return
-            log.warning('Lowering session pool size from %s to %s', self._session_pool_size,
+            log.warning('Server %s: Decreasing session pool size from %s to %s', self.server, self._session_pool_size,
                         self._session_pool_size - 1)
-            self.get_session().close()
+            session = self.get_session()
+            self.close_session(session)
             self._session_pool_size -= 1
 
     def get_session(self):
+        # Try to get a session from the queue. If the queue is empty, try to add one more session to the queue. If the
+        # queue is already at its max, wait until a session becomes available.
         _timeout = 60  # Rate-limit messages about session starvation
-        while True:
+        try:
+            session = self._session_pool.get(block=False)
+            log.debug('Server %s: Got session immediately', self.server)
+        except Empty:
             try:
-                log.debug('Server %s: Waiting for session', self.server)
-                session = self._session_pool.get(timeout=_timeout)
-                log.debug('Server %s: Got session %s', self.server, session.session_id)
-                session.usage_count += 1
-                return session
-            except Empty:
-                # This is normal when we have many worker threads starving for available sessions
-                log.debug('Server %s: No sessions available for %s seconds', self.server, _timeout)
+                self.increase_poolsize()
+            except SessionPoolMaxSizeReached:
+                pass
+            while True:
+                try:
+                    log.debug('Server %s: Waiting for session', self.server)
+                    session = self._session_pool.get(timeout=_timeout)
+                    break
+                except Empty:
+                    # This is normal when we have many worker threads starving for available sessions
+                    log.debug('Server %s: No sessions available for %s seconds', self.server, _timeout)
+        log.debug('Server %s: Got session %s', self.server, session.session_id)
+        session.usage_count += 1
+        return session
 
     def release_session(self, session):
         # This should never fail, as we don't have more sessions than the queue contains
@@ -199,18 +215,21 @@ class BaseProtocol:
         except Full:
             log.debug('Server %s: Session pool was already full %s', self.server, session.session_id)
 
+    @staticmethod
+    def close_session(session):
+        session.close()
+        del session
+
     def retire_session(self, session):
         # The session is useless. Close it completely and place a fresh session in the pool
         log.debug('Server %s: Retiring session %s', self.server, session.session_id)
-        session.close()
-        del session
+        self.close_session(session)
         self.release_session(self.create_session())
 
     def renew_session(self, session):
         # The session is useless. Close it completely and place a fresh session in the pool
         log.debug('Server %s: Renewing session %s', self.server, session.session_id)
-        session.close()
-        del session
+        self.close_session(session)
         return self.create_session()
 
     def refresh_credentials(self, session):
@@ -224,7 +243,7 @@ class BaseProtocol:
                 # they're the same as the session was created with. If
                 # this isn't the case, we can just go ahead with a new
                 # session using the already-updated credentials.
-                self.credentials.refresh()
+                self.credentials.refresh(session=session)
         return self.renew_session(session)
 
     def create_session(self):
@@ -233,7 +252,7 @@ class BaseProtocol:
         if self.credentials is None:
             if self.auth_type in CREDENTIALS_REQUIRED:
                 raise ValueError('Auth type %r requires credentials' % self.auth_type)
-            session = self.raw_session()
+            session = self.raw_session(self.service_endpoint)
             session.auth = get_auth_instance(auth_type=self.auth_type)
         else:
             with self.credentials.lock:
@@ -250,7 +269,7 @@ class BaseProtocol:
                         username = '\\' + self.credentials.username
                     else:
                         username = self.credentials.username
-                    session = self.raw_session()
+                    session = self.raw_session(self.service_endpoint)
                     session.auth = get_auth_instance(auth_type=self.auth_type, username=username,
                                                      password=self.credentials.password)
 
@@ -263,7 +282,9 @@ class BaseProtocol:
 
     def create_oauth2_session(self):
         if self.auth_type != OAUTH2:
-            raise ValueError('Auth type must be %r for credentials type OAuth2Credentials' % OAUTH2)
+            raise ValueError(
+                'Auth type must be %r for credentials type %s' % (OAUTH2, self.credentials.__class__.__name__)
+            )
 
         has_token = False
         scope = ['https://outlook.office365.com/.default']
@@ -310,12 +331,12 @@ class BaseProtocol:
             token_url = 'https://login.microsoftonline.com/%s/oauth2/v2.0/token' % self.credentials.tenant_id
             client = BackendApplicationClient(client_id=self.credentials.client_id)
 
-        session = self.raw_session(client, session_params)
+        session = self.raw_session(self.service_endpoint, oauth2_client=client, oauth2_session_params=session_params)
         if not has_token:
             # Fetch the token explicitly -- it doesn't occur implicitly
             token = session.fetch_token(token_url=token_url, client_id=self.credentials.client_id,
                                         client_secret=self.credentials.client_secret, scope=scope,
-                                        **token_params)
+                                        timeout=self.TIMEOUT, **token_params)
             # Allow the credentials object to update its copy of the new
             # token, and give the application an opportunity to cache it
             self.credentials.on_token_auto_refreshed(token)
@@ -324,15 +345,14 @@ class BaseProtocol:
         return session
 
     @classmethod
-    def raw_session(cls, oauth2_client=None, oauth2_session_params=None):
+    def raw_session(cls, prefix, oauth2_client=None, oauth2_session_params=None):
         if oauth2_client:
             session = OAuth2Session(client=oauth2_client, **(oauth2_session_params or {}))
         else:
             session = requests.sessions.Session()
         session.headers.update(DEFAULT_HEADERS)
         session.headers['User-Agent'] = cls.USERAGENT
-        session.mount('http://', adapter=cls.get_adapter())
-        session.mount('https://', adapter=cls.get_adapter())
+        session.mount(prefix, adapter=cls.get_adapter())
         return session
 
     def __repr__(self):
@@ -340,6 +360,8 @@ class BaseProtocol:
 
 
 class CachingProtocol(type):
+    """A metaclass for Protocol that caches Protocol instances based on a server+username key."""
+
     _protocol_cache = {}
     _protocol_cache_lock = Lock()
 
@@ -351,55 +373,83 @@ class CachingProtocol(type):
         # We ignore auth_type from kwargs in the cache key. We trust caller to supply the correct auth_type - otherwise
         # __init__ will guess the correct auth type.
 
-        # We may be using multiple different credentials and changing our minds on TLS verification. This key
-        # combination should be safe.
-        _protocol_cache_key = kwargs['config'].service_endpoint, kwargs['config'].credentials
+        config = kwargs['config']
+        _protocol_cache_key = cls._cache_key(config)
 
-        protocol = cls._protocol_cache.get(_protocol_cache_key)
-        if isinstance(protocol, Exception):
-            # The input data leads to a TransportError. Re-throw
-            raise protocol
-        if protocol is not None:
+        try:
+            protocol, _ = cls._protocol_cache[_protocol_cache_key]
+        except KeyError:
+            pass
+        else:
+            if isinstance(protocol, Exception):
+                # The input data leads to a TransportError. Re-throw
+                raise protocol
             return protocol
 
         # Acquire lock to guard against multiple threads competing to cache information. Having a per-server lock is
         # probably overkill although it would reduce lock contention.
         log.debug('Waiting for _protocol_cache_lock')
         with cls._protocol_cache_lock:
-            protocol = cls._protocol_cache.get(_protocol_cache_key)
-            if isinstance(protocol, Exception):
-                # Someone got ahead of us while holding the lock, but the input data leads to a TransportError. Re-throw
-                raise protocol
-            if protocol is not None:
-                # Someone got ahead of us while holding the lock
+            try:
+                protocol, _ = cls._protocol_cache[_protocol_cache_key]
+            except KeyError:
+                pass
+            else:
+                if isinstance(protocol, Exception):
+                    # We already tried this combination, possibly in a different competing thread, but the input
+                    # data leads to a TransportError.
+                    raise protocol
                 return protocol
+
             log.debug("Protocol __call__ cache miss. Adding key '%s'", str(_protocol_cache_key))
             try:
                 protocol = super().__call__(*args, **kwargs)
             except TransportError as e:
                 # This can happen if, for example, autodiscover supplies us with a bogus EWS endpoint
                 log.warning('Failed to create cached protocol with key %s: %s', _protocol_cache_key, e)
-                cls._protocol_cache[_protocol_cache_key] = e
+                cls._protocol_cache[_protocol_cache_key] = e, datetime.datetime.now()
                 raise e
-            cls._protocol_cache[_protocol_cache_key] = protocol
+            cls._protocol_cache[_protocol_cache_key] = protocol, datetime.datetime.now()
         return protocol
 
+    @staticmethod
+    def _cache_key(config):
+        # We may be using multiple different credentials for the same service endpoint. This key combination should be
+        # safe.
+        return config.service_endpoint, config.credentials
+
+    def __getitem__(cls, config):
+        return cls._protocol_cache[cls._cache_key(config)]
+
+    def __delitem__(cls, config):
+        del cls._protocol_cache[cls._cache_key(config)]
+
     @classmethod
     def clear_cache(mcs):
-        for key, protocol in mcs._protocol_cache.items():
-            if isinstance(protocol, Exception):
-                continue
-            service_endpoint = key[0]
-            log.debug("Service endpoint '%s': Closing sessions", service_endpoint)
-            protocol.close()
-        mcs._protocol_cache.clear()
+        with mcs._protocol_cache_lock:
+            for key, (protocol, _) in mcs._protocol_cache.items():
+                if isinstance(protocol, Exception):
+                    continue
+                service_endpoint = key[0]
+                log.debug("Service endpoint '%s': Closing sessions", service_endpoint)
+                with protocol._session_pool_lock:
+                    protocol.close()
+            mcs._protocol_cache.clear()
 
 
 class Protocol(BaseProtocol, metaclass=CachingProtocol):
+    """A class to handle all the low-level communication with an Exchange server. Contains a session pool, knows how to
+    negotiate the authentication type of the server, refresh credentials, etc. Also contains methods for calling EWS
+    services that are not tied to an account.
+    """
+
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self._api_version_hint = None
         self._version_lock = Lock()
+        # Autodetect authentication type if necessary
+        if self.config.auth_type is None:
+            self.config.auth_type = self.get_auth_type()
 
     def get_auth_type(self):
         # Autodetect authentication type. We also set version hint here.
@@ -421,10 +471,12 @@ class Protocol(BaseProtocol, metaclass=CachingProtocol):
         return self.config.version
 
     def get_timezones(self, timezones=None, return_full_timezone_data=False):
-        """ Get timezone definitions from the server
+        """Get timezone definitions from the server.
 
         :param timezones: A list of EWSDateTime instances. If None, fetches all timezones from server
-        :param return_full_timezone_data: If true, also returns periods and transitions
+          (Default value = None)
+        :param return_full_timezone_data: If true, also returns periods and transitions (Default value = False)
+
         :return: A list of (tz_id, name, periods, transitions) tuples
         """
         return GetServerTimeZones(protocol=self).call(
@@ -432,16 +484,17 @@ class Protocol(BaseProtocol, metaclass=CachingProtocol):
         )
 
     def get_free_busy_info(self, accounts, start, end, merged_free_busy_interval=30, requested_view='DetailedMerged'):
-        """ Returns free/busy information for a list of accounts
+        """Return free/busy information for a list of accounts.
 
         :param accounts: A list of (account, attendee_type, exclude_conflicts) tuples, where account is either an
-        Account object or a string, attendee_type is a MailboxData.attendee_type choice, and exclude_conflicts is a
-        boolean.
+          Account object or a string, attendee_type is a MailboxData.attendee_type choice, and exclude_conflicts is a
+          boolean.
         :param start: The start datetime of the request
         :param end: The end datetime of the request
-        :param merged_free_busy_interval: The interval, in minutes, of merged free/busy information
+        :param merged_free_busy_interval: The interval, in minutes, of merged free/busy information (Default value = 30)
         :param requested_view: The type of information returned. Possible values are defined in the
-               FreeBusyViewOptions.requested_view choices.
+          FreeBusyViewOptions.requested_view choices. (Default value = 'DetailedMerged')
+
         :return: A generator of FreeBusyView objects
         """
         from .account import Account
@@ -486,16 +539,16 @@ class Protocol(BaseProtocol, metaclass=CachingProtocol):
         return GetRoomLists(protocol=self).call()
 
     def get_rooms(self, roomlist):
-        from .properties import RoomList
         return GetRooms(protocol=self).call(roomlist=RoomList(email_address=roomlist))
 
     def resolve_names(self, names, return_full_contact_data=False, search_scope=None, shape=None):
-        """ Resolve accounts on the server using partial account data, e.g. an email address or initials
+        """Resolve accounts on the server using partial account data, e.g. an email address or initials.
 
         :param names: A list of identifiers to query
-        :param return_full_contact_data: If True, returns full contact data
-        :param search_scope: The scope to perform the search. Must be one of SEARCH_SCOPE_CHOICES
-        :param shape:
+        :param return_full_contact_data: If True, returns full contact data (Default value = False)
+        :param search_scope: The scope to perform the search. Must be one of SEARCH_SCOPE_CHOICES (Default value = None)
+        :param shape: (Default value = None)
+
         :return: A list of Mailbox items or, if return_full_contact_data is True, tuples of (Mailbox, Contact) items
         """
         return list(ResolveNames(protocol=self).call(
@@ -504,22 +557,25 @@ class Protocol(BaseProtocol, metaclass=CachingProtocol):
         ))
 
     def expand_dl(self, distribution_list):
-        """ Expand distribution list into it's members
+        """Expand distribution list into it's members.
 
         :param distribution_list: SMTP address of the distribution list to expand, or a DLMailbox representing the list
+
         :return: List of Mailbox items that are members of the distribution list
         """
-        from .properties import DLMailbox
         if isinstance(distribution_list, str):
             distribution_list = DLMailbox(email_address=distribution_list, mailbox_type='PublicDL')
         return list(ExpandDL(protocol=self).call(distribution_list=distribution_list))
 
     def get_searchable_mailboxes(self, search_filter=None, expand_group_membership=False):
-        """This method is only available to users who have been assigned the Discovery Management RBAC role. See
+        """Call the GetSearchableMailboxes service to get mailboxes that can be searched.
+
+        This method is only available to users who have been assigned the Discovery Management RBAC role. See
         https://docs.microsoft.com/en-us/exchange/permissions-exo/permissions-exo
 
-        :param search_filter: Is set, must be a single email alias
-        :param expand_group_membership: If True, returned distribution lists are expanded
+        :param search_filter: Is set, must be a single email alias (Default value = None)
+        :param expand_group_membership: If True, returned distribution lists are expanded (Default value = False)
+
         :return: a list of SearchableMailbox, FailedMailbox or Exception instances
         """
         return list(GetSearchableMailboxes(protocol=self).call(
@@ -528,22 +584,14 @@ class Protocol(BaseProtocol, metaclass=CachingProtocol):
         ))
 
     def convert_ids(self, ids, destination_format):
-        """Converts item and folder IDs between multiple formats
+        """Convert item and folder IDs between multiple formats.
 
         :param ids: a list of AlternateId, AlternatePublicFolderId or AlternatePublicFolderItemId instances
         :param destination_format: A string
+
         :return: a generator of AlternateId, AlternatePublicFolderId or AlternatePublicFolderItemId instances
         """
-        from .properties import AlternateId, AlternatePublicFolderId, AlternatePublicFolderItemId
-        cls_map = {cls.response_tag(): cls for cls in (
-            AlternateId, AlternatePublicFolderId, AlternatePublicFolderItemId
-        )}
-        for i in ConvertId(protocol=self).call(items=ids, destination_format=destination_format):
-            if isinstance(i, Exception):
-                yield i
-            else:
-                id_cls = cls_map[i.tag]
-                yield id_cls.from_xml(i, account=None)
+        return ConvertId(protocol=self).call(items=ids, destination_format=destination_format)
 
     def __getstate__(self):
         # The lock and thread pool cannot be pickled
@@ -578,14 +626,16 @@ EWS auth: %s''' % (self.service_endpoint, fullname, api_version, build, self.aut
 
 class NoVerifyHTTPAdapter(requests.adapters.HTTPAdapter):
     """An HTTP adapter that ignores TLS validation errors. Use at own risk."""
+
     def cert_verify(self, conn, url, verify, cert):
         # pylint: disable=unused-argument
-        # We're overiding a method so we have to keep the signature
+        # We're overriding a method so we have to keep the signature
         super().cert_verify(conn=conn, url=url, verify=False, cert=cert)
 
 
 class TLSClientAuth(requests.adapters.HTTPAdapter):
-    """An HTTP adapter that implements Certificate Based Authentication (CBA)"""
+    """An HTTP adapter that implements Certificate Based Authentication (CBA)."""
+
     cert_file = None
 
     def init_poolmanager(self, *args, **kwargs):
@@ -593,28 +643,63 @@ class TLSClientAuth(requests.adapters.HTTPAdapter):
         return super().init_poolmanager(*args, **kwargs)
 
 
-class RetryPolicy:
-    """Stores retry logic used when faced with errors from the server"""
+class RetryPolicy(metaclass=abc.ABCMeta):
+    """Stores retry logic used when faced with errors from the server."""
+
     @property
+    @abc.abstractmethod
     def fail_fast(self):
         # Used to choose the error handling policy. When True, a fault-tolerant policy is used. False, a fail-fast
         # policy is used.
-        raise NotImplementedError()
+        pass
 
     @property
+    @abc.abstractmethod
     def back_off_until(self):
-        raise NotImplementedError()
+        pass
 
     @back_off_until.setter
+    @abc.abstractmethod
     def back_off_until(self, value):
-        raise NotImplementedError()
+        pass
 
+    @abc.abstractmethod
     def back_off(self, seconds):
-        raise NotImplementedError()
+        pass
+
+    @abc.abstractmethod
+    def may_retry_on_error(self, response, wait):
+        pass
+
+    def raise_response_errors(self, response):
+        cas_error = response.headers.get('X-CasErrorCode')
+        if cas_error:
+            if cas_error.startswith('CAS error:'):
+                # Remove unnecessary text
+                cas_error = cas_error.split(':', 1)[1].strip()
+            raise CASError(cas_error=cas_error, response=response)
+        if response.status_code == 500 and (b'The specified server version is invalid' in response.content or
+                                            b'ErrorInvalidSchemaVersionForMailboxVersion' in response.content):
+            # Another way of communicating invalid schema versions
+            raise ErrorInvalidSchemaVersionForMailboxVersion('Invalid server version')
+        if b'The referenced account is currently locked out' in response.content:
+            raise UnauthorizedError('The referenced account is currently locked out')
+        if response.status_code == 401 and self.fail_fast:
+            # This is a login failure
+            raise UnauthorizedError('Invalid credentials for %s' % response.url)
+        if 'TimeoutException' in response.headers:
+            # A header set by us on CONNECTION_ERRORS
+            raise response.headers['TimeoutException']
+        # This could be anything. Let higher layers handle this
+        raise MalformedResponseError(
+            'Unknown failure in response. Code: %s headers: %s content: %s'
+            % (response.status_code, response.headers, response.text)
+        )
 
 
 class FailFast(RetryPolicy):
-    """Fail immediately on server errors"""
+    """Fail immediately on server errors."""
+
     @property
     def fail_fast(self):
         return True
@@ -623,11 +708,22 @@ class FailFast(RetryPolicy):
     def back_off_until(self):
         return None
 
+    def back_off(self, seconds):
+        raise ValueError('Cannot back off with fail-fast policy')
+
+    def may_retry_on_error(self, response, wait):
+        log.debug('No retry: no fail-fast policy')
+        return False
+
 
 class FaultTolerance(RetryPolicy):
     """Enables fault-tolerant error handling. Tells internal methods to do an exponential back off when requests start
     failing, and wait up to max_wait seconds before failing.
     """
+
+    # Back off 60 seconds if we didn't get an explicit suggested value
+    DEFAULT_BACKOFF = 60
+
     def __init__(self, max_wait=3600):
         self.max_wait = max_wait
         self._back_off_until = None
@@ -650,7 +746,7 @@ class FaultTolerance(RetryPolicy):
 
     @property
     def back_off_until(self):
-        """Returns the back off value as a datetime. Resets the current back off value if it has expired."""
+        """Return the back off value as a datetime. Reset the current back off value if it has expired."""
         if self._back_off_until is None:
             return None
         with self._back_off_lock:
@@ -668,7 +764,38 @@ class FaultTolerance(RetryPolicy):
 
     def back_off(self, seconds):
         if seconds is None:
-            seconds = 60  # Back off 60 seconds if we didn't get an explicit suggested value
+            seconds = self.DEFAULT_BACKOFF
         value = datetime.datetime.now() + datetime.timedelta(seconds=seconds)
         with self._back_off_lock:
             self._back_off_until = value
+
+    def may_retry_on_error(self, response, wait):
+        if response.status_code not in (301, 302, 401, 500, 503):
+            # Don't retry if we didn't get a status code that we can hope to recover from
+            log.debug('No retry: wrong status code %s', response.status_code)
+            return False
+        if wait > self.max_wait:
+            # We lost patience. Session is cleaned up in outer loop
+            raise RateLimitError(
+                'Max timeout reached', url=response.url, status_code=response.status_code, total_wait=wait)
+        if response.status_code == 401:
+            # EWS sometimes throws 401's when it wants us to throttle connections. OK to retry.
+            return True
+        if response.headers.get('connection') == 'close':
+            # Connection closed. OK to retry.
+            return True
+        if response.status_code == 302 and response.headers.get('location', '').lower() \
+                == '/ews/genericerrorpage.htm?aspxerrorpath=/ews/exchange.asmx':
+            # The genericerrorpage.htm/internalerror.asp is ridiculous behaviour for random outages. OK to retry.
+            #
+            # Redirect to '/internalsite/internalerror.asp' or '/internalsite/initparams.aspx' is caused by e.g. TLS
+            # certificate f*ckups on the Exchange server. We should not retry those.
+            return True
+        if response.status_code == 503:
+            # Internal server error. OK to retry.
+            return True
+        if response.status_code == 500 and b"Server Error in '/EWS' Application" in response.content:
+            # "Server Error in '/EWS' Application" has been seen in highly concurrent settings. OK to retry.
+            log.debug('Retry allowed: conditions met')
+            return True
+        return False
