@@ -1,10 +1,12 @@
+import abc
+import logging
+import warnings
 from copy import deepcopy
 from itertools import islice
-import logging
 
 from .errors import MultipleObjectsReturned, DoesNotExist
-from .items import CalendarItem, ID_ONLY
 from .fields import FieldPath, FieldOrder
+from .items import CalendarItem, ID_ONLY
 from .properties import InvalidField
 from .restriction import Q
 from .services import CHUNK_SIZE
@@ -14,33 +16,40 @@ log = logging.getLogger(__name__)
 
 
 class SearchableMixIn:
-    """Implements a search API for inheritance"""
+    """Implement a search API for inheritance."""
+
+    @abc.abstractmethod
     def get(self, *args, **kwargs):
-        raise NotImplementedError()
+        pass
 
+    @abc.abstractmethod
     def all(self):
-        raise NotImplementedError()
+        pass
 
+    @abc.abstractmethod
     def none(self):
-        raise NotImplementedError()
+        pass
 
+    @abc.abstractmethod
     def filter(self, *args, **kwargs):
-        raise NotImplementedError()
+        pass
 
+    @abc.abstractmethod
     def exclude(self, *args, **kwargs):
-        raise NotImplementedError()
+        pass
 
+    @abc.abstractmethod
     def people(self):
-        raise NotImplementedError()
+        pass
 
 
 class QuerySet(SearchableMixIn):
-    """
-    A Django QuerySet-like class for querying items. Defers queries until the QuerySet is consumed. Supports chaining to
+    """A Django QuerySet-like class for querying items. Defers queries until the QuerySet is consumed. Supports chaining to
     build up complex queries.
 
     Django QuerySet documentation: https://docs.djangoproject.com/en/dev/ref/models/querysets/
     """
+
     VALUES = 'values'
     VALUES_LIST = 'values_list'
     FLAT = 'flat'
@@ -59,7 +68,7 @@ class QuerySet(SearchableMixIn):
         if request_type not in self.REQUEST_TYPES:
             raise ValueError("'request_type' %r must be one of %s" % (request_type, self.REQUEST_TYPES))
         self.request_type = request_type
-        self.q = Q()  # Default to no restrictions. 'None' means 'return nothing'
+        self.q = Q()  # Default to no restrictions
         self.only_fields = None
         self.order_fields = None
         self.return_format = self.NONE
@@ -69,8 +78,6 @@ class QuerySet(SearchableMixIn):
         self.offset = 0
         self._depth = None
 
-        self._cache = None
-
     def _copy_self(self):
         # When we copy a queryset where the cache has already been filled, we don't copy the cache. Thus, a copied
         # queryset will fetch results from the server again.
@@ -81,7 +88,7 @@ class QuerySet(SearchableMixIn):
         # items = list(qs)
         # new_qs = qs.exclude(bar='baz')  # This should work, and should fetch from the server
         #
-        if not isinstance(self.q, (type(None), Q)):
+        if not isinstance(self.q, Q):
             raise ValueError("self.q value '%s' must be None or a Q instance" % self.q)
         if not isinstance(self.only_fields, (type(None), tuple)):
             raise ValueError("self.only_fields value '%s' must be None or a tuple" % self.only_fields)
@@ -91,7 +98,7 @@ class QuerySet(SearchableMixIn):
             raise ValueError("self.return_value '%s' must be one of %s" % (self.return_format, self.RETURN_TYPES))
         # Only mutable objects need to be deepcopied. Folder should be the same object
         new_qs = self.__class__(self.folder_collection, request_type=self.request_type)
-        new_qs.q = None if self.q is None else deepcopy(self.q)
+        new_qs.q = deepcopy(self.q)
         new_qs.only_fields = self.only_fields
         new_qs.order_fields = None if self.order_fields is None else deepcopy(self.order_fields)
         new_qs.return_format = self.return_format
@@ -102,10 +109,6 @@ class QuerySet(SearchableMixIn):
         new_qs._depth = self._depth
         return new_qs
 
-    @property
-    def is_cached(self):
-        return self._cache is not None
-
     def _get_field_path(self, field_path):
         from .items import Persona
         if self.request_type == self.PERSONA:
@@ -170,14 +173,11 @@ class QuerySet(SearchableMixIn):
         }[return_format](items)
 
     def _query(self):
-        from .items import Persona
         if self.only_fields is None:
             # We didn't restrict list of field paths. Get all fields from the server, including extended properties.
             if self.request_type == self.PERSONA:
-                additional_fields = {FieldPath(field=f) for f in Persona.supported_fields(
-                    version=self.folder_collection.account.version
-                ) if not f.is_complex}
-                complex_fields_requested = False
+                additional_fields = {}  # GetPersona doesn't take explicit fields. Don't bother calculating the list
+                complex_fields_requested = True
             else:
                 additional_fields = {FieldPath(field=f) for f in self.folder_collection.allowed_item_fields()}
                 complex_fields_requested = True
@@ -202,37 +202,33 @@ class QuerySet(SearchableMixIn):
         else:
             extra_order_fields = set()
 
+        find_kwargs = dict(
+            shape=ID_ONLY,  # Always use IdOnly here, because AllProperties doesn't actually get *all* properties
+            depth=self._depth,
+            additional_fields=additional_fields,
+            order_fields=order_fields,
+            page_size=self.page_size,
+            max_items=self.max_items,
+            offset=self.offset,
+        )
         if self.request_type == self.PERSONA:
-            if len(self.folder_collection) != 1:
-                raise ValueError('Personas can only be queried on a single folder')
-            items = list(self.folder_collection)[0].find_people(
-                self.q,
-                shape=ID_ONLY,
-                depth=self._depth,
-                additional_fields=additional_fields,
-                order_fields=order_fields,
-                page_size=self.page_size,
-                max_items=self.max_items,
-                offset=self.offset,
-            )
+            if complex_fields_requested:
+                find_kwargs['additional_fields'] = None
+                items = self.folder_collection.account.fetch_personas(
+                    ids=self.folder_collection.find_people(self.q, **find_kwargs)
+                )
+            else:
+                if not additional_fields:
+                    find_kwargs['additional_fields'] = None
+                items = self.folder_collection.find_people(self.q, **find_kwargs)
         else:
-            find_item_kwargs = dict(
-                shape=ID_ONLY,  # Always use IdOnly here, because AllProperties doesn't actually get *all* properties
-                depth=self._depth,
-                additional_fields=additional_fields,
-                order_fields=order_fields,
-                calendar_view=self.calendar_view,
-                page_size=self.page_size,
-                max_items=self.max_items,
-                offset=self.offset,
-            )
-
+            find_kwargs['calendar_view'] = self.calendar_view
             if complex_fields_requested:
                 # The FindItem service does not support complex field types. Tell find_items() to return
                 # (id, changekey) tuples, and pass that to fetch().
-                find_item_kwargs['additional_fields'] = None
+                find_kwargs['additional_fields'] = None
                 items = self.folder_collection.account.fetch(
-                    ids=self.folder_collection.find_items(self.q, **find_item_kwargs),
+                    ids=self.folder_collection.find_items(self.q, **find_kwargs),
                     only_fields=additional_fields,
                     chunk_size=self.page_size,
                 )
@@ -241,8 +237,8 @@ class QuerySet(SearchableMixIn):
                     # If additional_fields is the empty set, we only requested ID and changekey fields. We can then
                     # take a shortcut by using (shape=ID_ONLY, additional_fields=None) to tell find_items() to return
                     # (id, changekey) tuples. We'll post-process those later.
-                    find_item_kwargs['additional_fields'] = None
-                items = self.folder_collection.find_items(self.q, **find_item_kwargs)
+                    find_kwargs['additional_fields'] = None
+                items = self.folder_collection.find_items(self.q, **find_kwargs)
 
         if not must_sort_clientside:
             return items
@@ -252,7 +248,7 @@ class QuerySet(SearchableMixIn):
         # each sort operation if the field was marked as such.
         for f in reversed(self.order_fields):
             try:
-                items = sorted(items, key=lambda i: _get_value_or_default(i, f), reverse=f.reverse)
+                items = sorted(items, key=lambda i: _get_sort_value_or_default(i, f), reverse=f.reverse)
             except TypeError as e:
                 if 'unorderable types' not in e.args[0]:
                     raise
@@ -270,23 +266,11 @@ class QuerySet(SearchableMixIn):
         # Fill cache if this is the first iteration. Return an iterator over the results. Make this non-greedy by
         # filling the cache while we are iterating.
         #
-        # We don't set self._cache until the iterator is finished. Otherwise an interrupted iterator would leave the
-        # cache in an inconsistent state.
-        if self.is_cached:
-            for val in self._cache:
-                yield val
-            return
-
-        if self.q is None:
-            self._cache = []
+        if self.q.is_never():
             return
 
         log.debug('Initializing cache')
-        _cache = []
-        for val in self._format_items(items=self._query(), return_format=self.return_format):
-            _cache.append(val)
-            yield val
-        self._cache = _cache
+        yield from self._format_items(items=self._query(), return_format=self.return_format)
 
     """Do not implement __len__. The implementation of list() tries to preallocate memory by calling __len__ on the
     given sequence, before calling __iter__. If we implemented __len__, we would end up calling FindItems twice, once
@@ -296,8 +280,6 @@ class QuerySet(SearchableMixIn):
     a __len__ implementation should be cheap. That does not hold for self.count().
 
     def __len__(self):
-        if self.is_cached:
-            return len(self._cache)
         # This queryset has no cache yet. Call the optimized counting implementation
         return self.count()
     """
@@ -310,8 +292,6 @@ class QuerySet(SearchableMixIn):
         return self._getitem_slice(idx_or_slice)
 
     def _getitem_idx(self, idx):
-        if self.is_cached:
-            return self._cache[idx]
         if idx < 0:
             # Support negative indexes by reversing the queryset and negating the index value
             reverse_idx = -(idx+1)
@@ -330,10 +310,7 @@ class QuerySet(SearchableMixIn):
         if ((s.start or 0) < 0) or ((s.stop or 0) < 0) or ((s.step or 0) < 0):
             # islice() does not support negative start, stop and step. Make sure cache is full by iterating the full
             # query result, and then slice on the cache.
-            list(self.__iter__())
-            return self._cache[s]
-        if self.is_cached:
-            return islice(self.__iter__(), s.start, s.stop, s.step)
+            return list(self.__iter__())[s]
         # Optimize by setting an exact offset and max_items value
         new_qs = self._copy_self()
         if s.start is not None and s.stop is not None:
@@ -389,7 +366,7 @@ class QuerySet(SearchableMixIn):
             raise ValueError('values() requires at least one field name')
         return self._item_yielder(
             iterable=iterable,
-            item_func=lambda i: {f.path: f.get_value(i) for f in self.only_fields},
+            item_func=lambda i: {f.path: _get_value_or_default(f, i) for f in self.only_fields},
             id_only_func=lambda item_id, changekey: {'id': item_id},
             changekey_only_func=lambda item_id, changekey: {'changekey': changekey},
             id_and_changekey_func=lambda item_id, changekey: {'id': item_id, 'changekey': changekey},
@@ -400,7 +377,7 @@ class QuerySet(SearchableMixIn):
             raise ValueError('values_list() requires at least one field name')
         return self._item_yielder(
             iterable=iterable,
-            item_func=lambda i: tuple(f.get_value(i) for f in self.only_fields),
+            item_func=lambda i: tuple(_get_value_or_default(f, i) for f in self.only_fields),
             id_only_func=lambda item_id, changekey: (item_id,),
             changekey_only_func=lambda item_id, changekey: (changekey,),
             id_and_changekey_func=lambda item_id, changekey: (item_id, changekey),
@@ -409,10 +386,9 @@ class QuerySet(SearchableMixIn):
     def _as_flat_values_list(self, iterable):
         if not self.only_fields or len(self.only_fields) != 1:
             raise ValueError('flat=True requires exactly one field name')
-        flat_field_path = self.only_fields[0]
         return self._item_yielder(
             iterable=iterable,
-            item_func=flat_field_path.get_value,
+            item_func=lambda i: _get_value_or_default(self.only_fields[0], i),
             id_only_func=lambda item_id, changekey: item_id,
             changekey_only_func=lambda item_id, changekey: changekey,
             id_and_changekey_func=None,  # Can never be called
@@ -430,38 +406,36 @@ class QuerySet(SearchableMixIn):
     # foo_qs.filter(foo='baz')  # Should not be affected by the previous statement
     #
     def all(self):
-        """ Return everything, without restrictions """
+        """ """
         new_qs = self._copy_self()
         return new_qs
 
     def none(self):
-        """ Return a query that is guaranteed to be empty  """
+        """ """
         new_qs = self._copy_self()
-        new_qs.q = None
+        new_qs.q = Q(conn_type=Q.NEVER)
         return new_qs
 
     def filter(self, *args, **kwargs):
-        """ Return everything that matches these search criteria """
         new_qs = self._copy_self()
         q = Q(*args, **kwargs)
-        new_qs.q = q if new_qs.q is None else new_qs.q & q
+        new_qs.q = new_qs.q & q
         return new_qs
 
     def exclude(self, *args, **kwargs):
-        """ Return everything that does NOT match these search criteria """
         new_qs = self._copy_self()
         q = ~Q(*args, **kwargs)
-        new_qs.q = q if new_qs.q is None else new_qs.q & q
+        new_qs.q = new_qs.q & q
         return new_qs
 
     def people(self):
-        """ Changes the queryset to search the folder for Personas instead of Items """
+        """Change the queryset to search the folder for Personas instead of Items."""
         new_qs = self._copy_self()
         new_qs.request_type = self.PERSONA
         return new_qs
 
     def only(self, *args):
-        """ Fetch only the specified field names. All other item fields will be 'None' """
+        """Fetch only the specified field names. All other item fields will be 'None'."""
         try:
             only_fields = tuple(self._get_field_path(arg) for arg in args)
         except ValueError as e:
@@ -471,9 +445,11 @@ class QuerySet(SearchableMixIn):
         return new_qs
 
     def order_by(self, *args):
-        """ Return the query result sorted by the specified field names. Field names prefixed with '-' will be sorted
-        in reverse order. EWS only supports server-side sorting on a single field. Sorting on multiple fields is
-        implemented client-side and will therefore make the query greedy """
+        """
+
+        :return: The QuerySet in reverse order. EWS only supports server-side sorting on a single field. Sorting on
+          multiple fields is implemented client-side and will therefore make the query greedy.
+        """
         try:
             order_fields = tuple(self._get_field_order(arg) for arg in args)
         except ValueError as e:
@@ -483,7 +459,7 @@ class QuerySet(SearchableMixIn):
         return new_qs
 
     def reverse(self):
-        """ Return the entire query result in reverse order """
+        """Reverses the ordering of the queryset."""
         if not self.order_fields:
             raise ValueError('Reversing only makes sense if there are order_by fields')
         new_qs = self._copy_self()
@@ -492,7 +468,6 @@ class QuerySet(SearchableMixIn):
         return new_qs
 
     def values(self, *args):
-        """ Return the values of the specified field names as dicts """
         try:
             only_fields = tuple(self._get_field_path(arg) for arg in args)
         except ValueError as e:
@@ -503,10 +478,9 @@ class QuerySet(SearchableMixIn):
         return new_qs
 
     def values_list(self, *args, **kwargs):
-        """ Return the values of the specified field names as lists. If called with flat=True and only one field name,
-        return only this value instead of a list.
-
-        Allow an arbitrary list of fields in *args, possibly ending with flat=True|False"""
+        """Return the values of the specified field names as a list of lists. If called with flat=True and only one
+        field name, returns a list of values.
+        """
         flat = kwargs.pop('flat', False)
         if kwargs:
             raise AttributeError('Unknown kwargs: %s' % kwargs)
@@ -522,33 +496,28 @@ class QuerySet(SearchableMixIn):
         return new_qs
 
     def depth(self, depth):
-        """Specify the search depth (SHALLOW, ASSOCIATED or DEEP)
+        """Specify the search depth. Possible values are: SHALLOW, ASSOCIATED or DEEP.
+
+        :param depth:
         """
         new_qs = self._copy_self()
         new_qs._depth = depth
         return new_qs
 
+    def iterator(self):
+        # Return an iterator over the results
+        warnings.warn('QuerySet no longer caches results. .iterator() is a no-op.', DeprecationWarning, stacklevel=2)
+        return self.__iter__()
+
     ###########################
     #
     # Methods that end chaining
     #
     ###########################
 
-    def iterator(self):
-        """ Return the query result as an iterator, without caching the result """
-        if self.q is None:
-            return []
-        if self.is_cached:
-            return self._cache
-        # Return an iterator that doesn't bother with caching
-        return self._format_items(items=self._query(), return_format=self.return_format)
-
     def get(self, *args, **kwargs):
-        """ Assume the query will return exactly one item. Return that item """
-        if self.is_cached and not args and not kwargs:
-            # We can only safely use the cache if get() is called without args
-            items = self._cache
-        elif not args and set(kwargs.keys()) in ({'id'}, {'id', 'changekey'}):
+        """Assume the query will return exactly one item. Return that item."""
+        if not args and set(kwargs) in ({'id'}, {'id', 'changekey'}):
             # We allow calling get(id=..., changekey=...) to get a single item, but only if exactly these two
             # kwargs are present.
             account = self.folder_collection.account
@@ -565,137 +534,159 @@ class QuerySet(SearchableMixIn):
         return items[0]
 
     def count(self, page_size=1000):
-        """ Get the query count, with as little effort as possible 'page_size' is the number of items to
-        fetch from the server per request. We're only fetching the IDs, so keep it high"""
-        if self.is_cached:
-            return len(self._cache)
+        """Get the query count, with as little effort as possible 'page_size' is the number of items to
+        fetch from the server per request. We're only fetching the IDs, so keep it high
+
+        :param page_size:  (Default value = 1000)
+        """
         new_qs = self._copy_self()
-        new_qs.only_fields = tuple()
+        new_qs.only_fields = ()
         new_qs.order_fields = None
         new_qs.return_format = self.NONE
         new_qs.page_size = page_size
         return len(list(new_qs.__iter__()))
 
     def exists(self):
-        """ Find out if the query contains any hits, with as little effort as possible """
-        if self.is_cached:
-            return len(self._cache) > 0
+        """Find out if the query contains any hits, with as little effort as possible."""
         new_qs = self._copy_self()
         new_qs.max_items = 1
         return new_qs.count(page_size=1) > 0
 
     def _id_only_copy_self(self):
         new_qs = self._copy_self()
-        new_qs.only_fields = tuple()
+        new_qs.only_fields = ()
         new_qs.order_fields = None
         new_qs.return_format = self.NONE
         return new_qs
 
     def delete(self, page_size=1000, **delete_kwargs):
-        """ Delete the items matching the query, with as little effort as possible. 'page_size' is the number of items
-        to fetch and delete per request. We're only fetching the IDs, so keep it high"""
-        if self.is_cached:
-            ids = self._cache
-        else:
-            ids = self._id_only_copy_self()
-            ids.page_size = page_size
-        res = self.folder_collection.account.bulk_delete(
+        """Delete the items matching the query, with as little effort as possible. 'page_size' is the number of items
+        to fetch and delete per request. We're only fetching the IDs, so keep it high.
+
+        :param page_size:  (Default value = 1000)
+        :param delete_kwargs:
+        """
+        ids = self._id_only_copy_self()
+        ids.page_size = page_size
+        return self.folder_collection.account.bulk_delete(
             ids=ids,
             chunk_size=page_size,
             **delete_kwargs
         )
-        self._cache = None  # Invalidate the cache, regardless of the results
-        return res
 
     def send(self, page_size=1000, **send_kwargs):
-        """ Send the items matching the query, with as little effort as possible. 'page_size' is the number of items
-        to fetch and send per request. We're only fetching the IDs, so keep it high"""
-        if self.is_cached:
-            ids = self._cache
-        else:
-            ids = self._id_only_copy_self()
-            ids.page_size = page_size
-        res = self.folder_collection.account.bulk_send(
+        """Send the items matching the query, with as little effort as possible. 'page_size' is the number of items
+        to fetch and send per request. We're only fetching the IDs, so keep it high.
+
+        :param page_size:  (Default value = 1000)
+        :param send_kwargs:
+        """
+        ids = self._id_only_copy_self()
+        ids.page_size = page_size
+        return self.folder_collection.account.bulk_send(
             ids=ids,
             chunk_size=page_size,
             **send_kwargs
         )
-        self._cache = None  # Invalidate the cache, regardless of the results
-        return res
 
     def copy(self, to_folder, page_size=1000, **copy_kwargs):
-        """ Copy the items matching the query, with as little effort as possible. 'page_size' is the number of items
-        to fetch and copy per request. We're only fetching the IDs, so keep it high"""
-        if self.is_cached:
-            ids = self._cache
-        else:
-            ids = self._id_only_copy_self()
-            ids.page_size = page_size
-        res = self.folder_collection.account.bulk_copy(
+        """Copy the items matching the query, with as little effort as possible. 'page_size' is the number of items
+        to fetch and copy per request. We're only fetching the IDs, so keep it high.
+
+        :param to_folder:
+        :param page_size:  (Default value = 1000)
+        :param copy_kwargs:
+        """
+        ids = self._id_only_copy_self()
+        ids.page_size = page_size
+        return self.folder_collection.account.bulk_copy(
             ids=ids,
             to_folder=to_folder,
             chunk_size=page_size,
             **copy_kwargs
         )
-        self._cache = None  # Invalidate the cache, regardless of the results
-        return res
 
     def move(self, to_folder, page_size=1000):
-        """ Move the items matching the query, with as little effort as possible. 'page_size' is the number of items
-        to fetch and move per request. We're only fetching the IDs, so keep it high"""
-        if self.is_cached:
-            ids = self._cache
-        else:
-            ids = self._id_only_copy_self()
-            ids.page_size = page_size
-        res = self.folder_collection.account.bulk_move(
+        """Move the items matching the query, with as little effort as possible. 'page_size' is the number of items
+        to fetch and move per request. We're only fetching the IDs, so keep it high.
+
+        :param to_folder:
+        :param page_size: (Default value = 1000)
+        """
+        ids = self._id_only_copy_self()
+        ids.page_size = page_size
+        return self.folder_collection.account.bulk_move(
             ids=ids,
             to_folder=to_folder,
             chunk_size=page_size,
         )
-        self._cache = None  # Invalidate the cache after delete, regardless of the results
-        return res
 
     def archive(self, to_folder, page_size=1000):
-        """ Archive the items matching the query, with as little effort as possible. 'page_size' is the number of items
-        to fetch and move per request. We're only fetching the IDs, so keep it high"""
-        if self.is_cached:
-            ids = self._cache
-        else:
-            ids = self._id_only_copy_self()
-            ids.page_size = page_size
-        res = self.folder_collection.account.bulk_archive(
+        """Archive the items matching the query, with as little effort as possible. 'page_size' is the number of items
+        to fetch and move per request. We're only fetching the IDs, so keep it high.
+
+        :param to_folder:
+        :param page_size: (Default value = 1000)
+        """
+        ids = self._id_only_copy_self()
+        ids.page_size = page_size
+        return self.folder_collection.account.bulk_archive(
             ids=ids,
             to_folder=to_folder,
             chunk_size=page_size,
         )
-        self._cache = None  # Invalidate the cache after delete, regardless of the results
-        return res
+
+    def mark_as_junk(self, page_size=1000, **mark_as_junk_kwargs):
+        """Mark the items matching the query as junk, with as little effort as possible. 'page_size' is the number of
+        items to fetch and mark per request. We're only fetching the IDs, so keep it high.
+
+        :param page_size:  (Default value = 1000)
+        :param mark_as_junk_kwargs:
+        """
+        ids = self._id_only_copy_self()
+        ids.page_size = page_size
+        return self.folder_collection.account.bulk_mark_as_junk(
+            ids=ids,
+            chunk_size=page_size,
+            **mark_as_junk_kwargs
+        )
 
     def __str__(self):
         fmt_args = [('q', str(self.q)), ('folders', '[%s]' % ', '.join(str(f) for f in self.folder_collection.folders))]
-        if self.is_cached:
-            fmt_args.append(('len', str(len(self._cache))))
         return self.__class__.__name__ + '(%s)' % ', '.join('%s=%s' % (k, v) for k, v in fmt_args)
 
 
-def _get_value_or_default(item, field_order):
+def _get_value_or_default(field, item):
+    # When we request specific fields using .values() or .values_list(), the incoming item type may not have the field
+    # we are requesting. Return None when this happens instead of raising an AttributeError.
+    try:
+        return field.get_value(item)
+    except AttributeError:
+        return None
+
+
+def _get_sort_value_or_default(item, field_order):
     # Python can only sort values when <, > and = are implemented for the two types. Try as best we can to sort
     # items, even when the item may have a None value for the field in question, or when the item is an
-    # Exception. If the field to be sorted by does not have a default value, try creating an empty instance if the
-    # field value class. If that doesn't work, there's really nothing we can do about it; we'll raise an error. If
-    # it does, we sort all None values and exceptions as the default value.
-    default = field_order.field_path.field.default or field_order.field_path.field.value_cls()
+    # Exception. In that case, we calculate a default value and sort all None values and exceptions as the default
+    # value.
     if isinstance(item, Exception):
-        return default
-    val = field_order.field_path.get_value(item)
+        return _default_field_value(field_order.field_path.field)
+    val = field_order.field_path.get_sort_value(item)
     if val is None:
-        return default
+        return _default_field_value(field_order.field_path.field)
     return val
 
 
+def _default_field_value(field):
+    """Return the default value of a field. If the field does not have a default value, try creating an empty instance
+    of the field value class. If that doesn't work, there's really nothing we can do about it; we'll raise an error.
+    """
+    return field.default or ([field.value_cls()] if field.is_list else field.value_cls())
+
+
 def _rinse_item(i, fields_to_nullify):
-    # Set fields in fields_to_nullify to None. Make sure to accept exceptions.
+    """Set fields in fields_to_nullify to None. Make sure to accept exceptions."""
     if isinstance(i, Exception):
         return i
     for f in fields_to_nullify:
