@@ -1,8 +1,8 @@
-import base64
-from collections import OrderedDict
 import logging
+from collections import OrderedDict
+from copy import copy
 
-from .properties import InvalidField
+from .fields import InvalidField, FieldPath, DateTimeBackedDateField
 from .util import create_element, xml_to_str, value_to_xml_text, is_iterable
 from .version import EXCHANGE_2010
 
@@ -16,7 +16,8 @@ class Q:
     AND = 'AND'
     OR = 'OR'
     NOT = 'NOT'
-    CONN_TYPES = {AND, OR, NOT}
+    NEVER = 'NEVER'  # This is not specified by EWS. We use it for queries that will never match, e.g. 'foo__in=()'
+    CONN_TYPES = {AND, OR, NOT, NEVER}
 
     # EWS Operators
     EQ = '=='
@@ -54,7 +55,7 @@ class Q:
                     LOOKUP_IEXACT, LOOKUP_CONTAINS, LOOKUP_ICONTAINS, LOOKUP_STARTSWITH, LOOKUP_ISTARTSWITH,
                     LOOKUP_EXISTS}
 
-    __slots__ = ('conn_type', 'field_path', 'op', 'value', 'children', 'query_string')
+    __slots__ = 'conn_type', 'field_path', 'op', 'value', 'children', 'query_string'
 
     def __init__(self, *args, **kwargs):
         self.conn_type = kwargs.pop('conn_type', self.AND)
@@ -67,41 +68,34 @@ class Q:
         # Parsing of args and kwargs may require child elements
         self.children = []
 
-        # Remove any empty Q elements in args before proceeding
-        args = tuple(a for a in args if not (isinstance(a, self.__class__) and a.is_empty()))
+        # Check for query string as the only argument
+        if not kwargs and len(args) == 1 and isinstance(args[0], str):
+            self.query_string = args[0]
+            args = ()
 
-        # Check for query string, or Q object containing query string, as the only argument
-        if len(args) == 1 and not kwargs:
-            if isinstance(args[0], str):
-                self.query_string = args[0]
-                return
-            if isinstance(args[0], self.__class__) and args[0].query_string:
-                self.query_string = args[0].query_string
-                return
-
-        # Parse args which must be Q objects
+        # Parse args which must now be Q objects
         for q in args:
             if not isinstance(q, self.__class__):
                 raise ValueError("Non-keyword arg %r must be a Q instance" % q)
-            if q.query_string:
-                raise ValueError(
-                    'A query string cannot be combined with other restrictions (args: %r, kwargs: %r)' % (args, kwargs)
-                )
-            self.children.append(q)
+        self.children.extend(args)
 
         # Parse keyword args and extract the filter
         is_single_kwarg = len(args) == 0 and len(kwargs) == 1
         for key, value in kwargs.items():
-            children = self._get_children_from_kwarg(key=key, value=value, is_single_kwarg=is_single_kwarg)
-            self.children.extend(children)
+            self.children.extend(
+                self._get_children_from_kwarg(key=key, value=value, is_single_kwarg=is_single_kwarg)
+            )
 
-        if len(self.children) == 1 and self.field_path is None and self.conn_type != self.NOT:
-            # We only have one child and no expression on ourselves, so we are a no-op. Flatten by taking over the child
-            self._promote()
+        # Simplify this object
+        self.reduce()
+
+        # Final sanity check
+        self._check_integrity()
 
     def _get_children_from_kwarg(self, key, value, is_single_kwarg=False):
-        # Generates Q objects corresponding to a single keyword argument. Makes this a leaf if there are no children to
-        # generate.
+        """Generate Q objects corresponding to a single keyword argument. Make this a leaf if there are no children to
+        generate.
+        """
         key_parts = key.rsplit('__', 1)
         if len(key_parts) == 2 and key_parts[1] in self.LOOKUP_TYPES:
             # This is a kwarg with a lookup at the end
@@ -121,14 +115,6 @@ class Q:
                     self.__class__(**{'%s__lte' % field_path: value[1]}),
                 ]
 
-            if lookup == self.LOOKUP_IN:
-                # EWS doesn't have an '__in' operator. Allow '__in' lookups on list and non-list field types,
-                # specifying a list value. We'll emulate it as a set of OR'ed exact matches.
-                if not is_iterable(value, generators_allowed=True):
-                    raise ValueError("Value for lookup %r must be a list" % key)
-                children = [self.__class__(**{field_path: v}) for v in value]
-                return [self.__class__(*children, conn_type=self.OR)]
-
             # Filtering on list types is a bit quirky. The only lookup type I have found to work is:
             #
             #     item:Categories == 'foo' AND item:Categories == 'bar' AND ...
@@ -136,8 +122,8 @@ class Q:
             #     item:Categories == 'foo' OR item:Categories == 'bar' OR ...
             #
             # The former returns items that have all these categories, but maybe also others. The latter returns
-            # items that have at least one of these categories. This translates to the 'contains' and 'in' lookups.
-            # Both versions are case-insensitive.
+            # items that have at least one of these categories. This translates to the 'contains' and 'in' lookups,
+            # respectively. Both versions are case-insensitive.
             #
             # Exact matching and case-sensitive or partial-string matching is not possible since that requires the
             # 'Contains' element which only supports matching on string elements, not arrays.
@@ -145,8 +131,24 @@ class Q:
             # Exact matching of categories (i.e. match ['a', 'b'] but not ['a', 'b', 'c']) could be implemented by
             # post-processing items by fetch the categories field unconditionally and removing the items that don't
             # have an exact match.
+            if lookup == self.LOOKUP_IN:
+                # EWS doesn't have an '__in' operator. Allow '__in' lookups on list and non-list field types,
+                # specifying a list value. We'll emulate it as a set of OR'ed exact matches.
+                if not is_iterable(value, generators_allowed=True):
+                    raise ValueError("Value for lookup %r must be a list" % key)
+                children = [self.__class__(**{field_path: v}) for v in value]
+                if not children:
+                    # This is an '__in' operator with an empty list as the value. We interpret it to mean "is foo
+                    # contained in the empty set?" which is always false. Mark this Q object as such.
+                    return [self.__class__(conn_type=self.NEVER)]
+                return [self.__class__(*children, conn_type=self.OR)]
+
             if lookup == self.LOOKUP_CONTAINS and is_iterable(value, generators_allowed=True):
-                # '__contains' lookups on list field types
+                # A '__contains' lookup with an list as the value ony makes sense for list fields, since exact match
+                # on multiple distinct values will always fail for single-value fields.
+                #
+                # An empty list as value is allowed. We interpret it to mean "are all values in the empty set contained
+                # in foo?" which is always true.
                 children = [self.__class__(**{field_path: v}) for v in value]
                 return [self.__class__(*children, conn_type=self.AND)]
 
@@ -166,12 +168,40 @@ class Q:
         self.value = value
         return []
 
+    def reduce(self):
+        """Simplify this object, if possible."""
+        self._reduce_children()
+        self._promote()
+
+    def _reduce_children(self):
+        """Look at the children of this object and remove unnecessary items."""
+        children = self.children
+        if any((isinstance(a, self.__class__) and a.is_never()) for a in children):
+            # We have at least one 'never' arg
+            if self.conn_type == self.AND:
+                # Remove all other args since nothing we AND together with a 'never' arg can change the result
+                children = [self.__class__(conn_type=self.NEVER)]
+            elif self.conn_type == self.OR:
+                # Remove all 'never' args because all other args will decide the result. Keep one 'never' arg in case
+                # all args are 'never' args.
+                children = [a for a in children if not (isinstance(a, self.__class__) and a.is_never())]
+                if not children:
+                    children = [self.__class__(conn_type=self.NEVER)]
+            elif self.conn_type == self.NOT:
+                # Let's interpret 'not never' to mean 'always'. Remove all 'never' args
+                children = [a for a in children if not (isinstance(a, self.__class__) and a.is_never())]
+
+        # Remove any empty Q elements in args before proceeding
+        children = [a for a in children if not (isinstance(a, self.__class__) and a.is_empty())]
+        self.children = children
+
     def _promote(self):
-        # Flatten by taking over the only child
-        if len(self.children) != 1:
-            raise ValueError('Can only flatten when child count is 1')
-        if self.field_path is not None:
-            raise ValueError("Can only flatten when 'field_path' is not set")
+        """When we only have one child and no expression on ourselves, we are a no-op. Flatten by taking over the only
+        child.
+        """
+        if len(self.children) != 1 or self.field_path is not None or self.conn_type == self.NOT:
+            return
+
         q = self.children[0]
         self.conn_type = q.conn_type
         self.field_path = q.field_path
@@ -181,8 +211,9 @@ class Q:
         self.children = q.children
 
     def clean(self, version):
-        # Do some basic checks on the attributes, using a generic folder. to_xml() does a really good job of
-        # validating. There's no reason to replicate much of that here.
+        """Do some basic checks on the attributes, using a generic folder. to_xml() does a really good job of
+        validating. There's no reason to replicate much of that here.
+        """
         from .folders import Folder
         self.to_xml(folders=[Folder()], version=version, applies_to=Restriction.ITEMS)
 
@@ -274,11 +305,18 @@ class Q:
         return not self.children
 
     def is_empty(self):
-        return self.is_leaf() and self.field_path is None and self.query_string is None
+        """Return True if this object is without any restrictions at all."""
+        return self.is_leaf() and self.field_path is None and self.query_string is None and self.conn_type != self.NEVER
+
+    def is_never(self):
+        """Return True if this object has a restriction that will never match anything."""
+        return self.conn_type == self.NEVER
 
     def expr(self):
         if self.is_empty():
             return None
+        if self.is_never():
+            return self.NEVER
         if self.query_string:
             return self.query_string
         if self.is_leaf():
@@ -298,6 +336,7 @@ class Q:
 
     def to_xml(self, folders, version, applies_to):
         if self.query_string:
+            self._check_integrity()
             if version.build < EXCHANGE_2010:
                 raise NotImplementedError('QueryString filtering is only supported for Exchange 2010 servers and later')
             elem = create_element('m:QueryString')
@@ -314,6 +353,10 @@ class Q:
     def _check_integrity(self):
         if self.is_empty():
             return
+        if self.conn_type == self.NEVER:
+            if any([self.field_path, self.op, self.value, self.children]):
+                raise ValueError("'never' queries cannot be combined with other settings")
+            return
         if self.query_string:
             if any([self.field_path, self.op, self.value, self.children]):
                 raise ValueError('Query strings cannot be combined with other settings')
@@ -321,14 +364,18 @@ class Q:
         if self.conn_type not in self.CONN_TYPES:
             raise ValueError("'conn_type' %s must be one of %s" % (self.conn_type, self.CONN_TYPES))
         if not self.is_leaf():
+            for q in self.children:
+                if q.query_string and len(self.children) > 1:
+                    raise ValueError(
+                        'A query string cannot be combined with other restrictions'
+                    )
             return
         if not self.field_path:
             raise ValueError("'field_path' must be set")
         if self.op not in self.OP_TYPES:
             raise ValueError("'op' %s must be one of %s" % (self.op, self.OP_TYPES))
-        if self.op == self.EXISTS:
-            if self.value is not True:
-                raise ValueError("'value' must be True when operator is EXISTS")
+        if self.op == self.EXISTS and self.value is not True:
+            raise ValueError("'value' must be True when operator is EXISTS")
         if self.value is None:
             raise ValueError('Value for filter on field path "%s" cannot be None' % self.field_path)
         if is_iterable(self.value, generators_allowed=True):
@@ -352,7 +399,6 @@ class Q:
 
     def _get_field_path(self, folders, applies_to, version):
         # Convert the string field path to a real FieldPath object. The path is validated using the given folders.
-        from .fields import FieldPath
         for folder in folders:
             try:
                 if applies_to == Restriction.FOLDERS:
@@ -374,30 +420,27 @@ class Q:
             return None
         clean_field = field_path.subfield if (field_path.subfield and field_path.label) else field_path.field
         if clean_field.is_list:
-            # With __contains, we allow filtering by only one value even though the field is a list type
+            # __contains and __in are implemented as multiple leaves, with one value per leaf. clean() on list fields
+            # only works on lists, so clean a one-element list.
             return clean_field.clean(value=[self.value], version=version)[0]
-        else:
-            return clean_field.clean(value=self.value, version=version)
+        return clean_field.clean(value=self.value, version=version)
 
     def xml_elem(self, folders, version, applies_to):
         # Recursively build an XML tree structure of this Q object. If this is an empty leaf (the equivalent of Q()),
         # return None.
         from .indexed_properties import SingleFieldIndexedElement
-        from .extended_properties import ExtendedProperty
-        from .fields import DateTimeBackedDateField
         # Don't check self.value just yet. We want to return error messages on the field path first, and then the value.
         # This is done in _get_field_path() and _get_clean_value(), respectively.
         self._check_integrity()
         if self.is_empty():
             return None
+        if self.is_never():
+            raise ValueError("EWS does not support 'never' queries")
         if self.is_leaf():
             elem = self._op_to_xml(self.op)
             field_path = self._get_field_path(folders, applies_to=applies_to, version=version)
             clean_value = self._get_clean_value(field_path=field_path, version=version)
-            if issubclass(field_path.field.value_cls, ExtendedProperty) and field_path.field.value_cls.is_binary_type():
-                # We need to base64-encode binary data
-                clean_value = base64.b64encode(clean_value.value).decode('ascii')
-            elif issubclass(field_path.field.value_cls, SingleFieldIndexedElement) and not field_path.label:
+            if issubclass(field_path.field.value_cls, SingleFieldIndexedElement) and not field_path.label:
                 # We allow a filter shortcut of e.g. email_addresses__contains=EmailAddress(label='Foo', ...) instead of
                 # email_addresses__Foo_email_address=.... Set FieldPath label now so we can generate the field_uri.
                 field_path.label = clean_value.label
@@ -445,29 +488,26 @@ class Q:
         # ~ operator. If op has an inverse, change op. Else return a new Q with conn_type NOT
         if self.conn_type == self.NOT:
             # This is NOT NOT. Change to AND
-            self.conn_type = self.AND
-            if len(self.children) == 1 and self.field_path is None:
-                self._promote()
-            return self
+            new = copy(self)
+            new.conn_type = self.AND
+            new.reduce()
+            return new
         if self.is_leaf():
-            if self.op == self.EQ:
-                self.op = self.NE
-                return self
-            if self.op == self.NE:
-                self.op = self.EQ
-                return self
-            if self.op == self.GT:
-                self.op = self.LTE
-                return self
-            if self.op == self.GTE:
-                self.op = self.LT
-                return self
-            if self.op == self.LT:
-                self.op = self.GTE
-                return self
-            if self.op == self.LTE:
-                self.op = self.GT
-                return self
+            inverse_ops = {
+                self.EQ: self.NE,
+                self.NE: self.EQ,
+                self.GT: self.LTE,
+                self.GTE: self.LT,
+                self.LT: self.GTE,
+                self.LTE: self.GT,
+            }
+            try:
+                new = copy(self)
+                new.op = inverse_ops[self.op]
+                new.reduce()
+                return new
+            except KeyError:
+                pass
         return self.__class__(self, conn_type=self.NOT)
 
     def __eq__(self, other):
@@ -483,6 +523,8 @@ class Q:
         if self.is_leaf():
             if self.query_string:
                 return self.__class__.__name__ + '(%r)' % self.query_string
+            if self.is_never():
+                return self.__class__.__name__ + '(conn_type=%r)' % (self.conn_type)
             return self.__class__.__name__ + '(%s %s %r)' % (self.field_path, self.op, self.value)
         sorted_children = tuple(sorted(self.children, key=lambda i: i.field_path or ''))
         if self.conn_type == self.NOT or len(self.children) > 1:
@@ -491,10 +533,7 @@ class Q:
 
 
 class Restriction:
-    """
-    Implements an EWS Restriction type.
-
-    """
+    """Implement an EWS Restriction type."""
 
     # The type of item the restriction applies to
     FOLDERS = 'folders'
@@ -520,7 +559,5 @@ class Restriction:
         return self.q.to_xml(folders=self.folders, version=version, applies_to=self.applies_to)
 
     def __str__(self):
-        """
-        Prints the XML syntax tree
-        """
+        """Print the XML syntax tree."""
         return xml_to_str(self.to_xml(version=self.folders[0].account.version))
