@@ -1,16 +1,15 @@
-from ..util import create_element
 from .common import EWSAccountService, create_folder_ids_element
+from ..util import create_element
 
 
 class DeleteFolder(EWSAccountService):
-    """
-    MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/deletefolder-operation
-    """
+    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/deletefolder-operation"""
+
     SERVICE_NAME = 'DeleteFolder'
-    element_container_name = None  # DeleteFolder doesn't return a response object, just status in XML attrs
+    returns_elements = False
 
     def call(self, folders, delete_type):
-        return self._get_elements(payload=self.get_payload(folders=folders, delete_type=delete_type))
+        return self._chunked_get_elements(self.get_payload, items=folders, delete_type=delete_type)
 
     def get_payload(self, folders, delete_type):
         deletefolder = create_element('m:%s' % self.SERVICE_NAME, attrs=dict(DeleteType=delete_type))
