@@ -1,19 +1,19 @@
 from collections import OrderedDict
 
-from ..util import create_element
 from .common import EWSAccountService, create_folder_ids_element
+from ..util import create_element
 
 
 class EmptyFolder(EWSAccountService):
-    """
-    MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/emptyfolder
-    """
+    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/emptyfolder-operation"""
+
     SERVICE_NAME = 'EmptyFolder'
-    element_container_name = None  # EmptyFolder doesn't return a response object, just status in XML attrs
+    returns_elements = False
 
     def call(self, folders, delete_type, delete_sub_folders):
-        return self._get_elements(payload=self.get_payload(folders=folders, delete_type=delete_type,
-                                                           delete_sub_folders=delete_sub_folders))
+        return self._chunked_get_elements(
+            self.get_payload, items=folders, delete_type=delete_type, delete_sub_folders=delete_sub_folders
+        )
 
     def get_payload(self, folders, delete_type, delete_sub_folders):
         emptyfolder = create_element(
