@@ -1,21 +1,24 @@
+from .common import EWSService
+from ..properties import Room
 from ..util import create_element, set_xml_value, MNS
 from ..version import EXCHANGE_2010
-from .common import EWSService
 
 
 class GetRooms(EWSService):
-    """
-    MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/getrooms
-    """
+    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/getrooms-operation"""
+
     SERVICE_NAME = 'GetRooms'
     element_container_name = '{%s}Rooms' % MNS
+    supported_from = EXCHANGE_2010
 
     def call(self, roomlist):
-        from ..properties import Room
-        if self.protocol.version.build < EXCHANGE_2010:
-            raise NotImplementedError('%s is only supported for Exchange 2010 servers and later' % self.SERVICE_NAME)
+        return self._elems_to_objs(self._get_elements(payload=self.get_payload(roomlist=roomlist)))
 
-        for elem in self._get_elements(payload=self.get_payload(roomlist=roomlist)):
+    def _elems_to_objs(self, elems):
+        for elem in elems:
+            if isinstance(elem, Exception):
+                yield elem
+                continue
             yield Room.from_xml(elem=elem, account=None)
 
     def get_payload(self, roomlist):
