@@ -1,19 +1,18 @@
-from ..util import create_element, set_xml_value
 from .common import EWSAccountService, create_item_ids_element
+from ..util import create_element, set_xml_value
 
 
 class SendItem(EWSAccountService):
-    """
-    MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/senditem-operation
-    """
+    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/senditem-operation"""
+
     SERVICE_NAME = 'SendItem'
-    element_container_name = None  # SendItem doesn't return a response object, just status in XML attrs
+    returns_elements = False
 
     def call(self, items, saved_item_folder):
-        from ..folders import BaseFolder, FolderId, DistinguishedFolderId
-        if saved_item_folder and not isinstance(saved_item_folder, (BaseFolder, FolderId, DistinguishedFolderId)):
+        from ..folders import BaseFolder, FolderId
+        if saved_item_folder and not isinstance(saved_item_folder, (BaseFolder, FolderId)):
             raise ValueError("'saved_item_folder' %r must be a Folder or FolderId instance" % saved_item_folder)
-        return self._get_elements(payload=self.get_payload(items=items, saved_item_folder=saved_item_folder))
+        return self._chunked_get_elements(self.get_payload, items=items, saved_item_folder=saved_item_folder)
 
     def get_payload(self, items, saved_item_folder):
         senditem = create_element(
