@@ -1,18 +1,16 @@
 from collections import OrderedDict
-import logging
 
+from .common import EWSAccountService, to_item_id
 from ..ewsdatetime import EWSDate
+from ..fields import FieldPath, IndexedField
+from ..properties import ItemId
 from ..util import create_element, set_xml_value, MNS
-from ..version import EXCHANGE_2010, EXCHANGE_2013_SP1
-from .common import EWSAccountService, EWSPooledMixIn, to_item_id
-
-log = logging.getLogger(__name__)
+from ..version import EXCHANGE_2013_SP1
 
 
-class UpdateItem(EWSAccountService, EWSPooledMixIn):
-    """
-    MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/updateitem
-    """
+class UpdateItem(EWSAccountService):
+    """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/updateitem-operation"""
+
     SERVICE_NAME = 'UpdateItem'
     element_container_name = '{%s}Items' % MNS
 
@@ -36,7 +34,8 @@ class UpdateItem(EWSAccountService, EWSPooledMixIn):
             raise ValueError("'suppress_read_receipts' %s must be True or False" % suppress_read_receipts)
         if message_disposition == SEND_ONLY:
             raise ValueError('Cannot send-only existing objects. Use SendItem service instead')
-        return self._pool_requests(payload_func=self.get_payload, **dict(
+        return self._elems_to_objs(self._chunked_get_elements(
+            self.get_payload,
             items=items,
             conflict_resolution=conflict_resolution,
             message_disposition=message_disposition,
@@ -44,6 +43,14 @@ class UpdateItem(EWSAccountService, EWSPooledMixIn):
             suppress_read_receipts=suppress_read_receipts,
         ))
 
+    def _elems_to_objs(self, elems):
+        from ..items import Item
+        for elem in elems:
+            if isinstance(elem, (Exception, type(None))):
+                yield elem
+                continue
+            yield Item.id_from_xml(elem)
+
     def _delete_item_elem(self, field_path):
         deleteitemfield = create_element('t:DeleteItemField')
         return set_xml_value(deleteitemfield, field_path, version=self.account.version)
@@ -51,10 +58,10 @@ class UpdateItem(EWSAccountService, EWSPooledMixIn):
     def _set_item_elem(self, item_model, field_path, value):
         setitemfield = create_element('t:SetItemField')
         set_xml_value(setitemfield, field_path, version=self.account.version)
-        folderitem = create_element(item_model.request_tag())
+        item_elem = create_element(item_model.request_tag())
         field_elem = field_path.field.to_xml(value, version=self.account.version)
-        set_xml_value(folderitem, field_elem, version=self.account.version)
-        setitemfield.append(folderitem)
+        set_xml_value(item_elem, field_elem, version=self.account.version)
+        setitemfield.append(item_elem)
         return setitemfield
 
     @staticmethod
@@ -78,80 +85,67 @@ class UpdateItem(EWSAccountService, EWSPooledMixIn):
         if item.__class__ == CalendarItem:
             # For CalendarItem items where we update 'start' or 'end', we want to update internal timezone fields
             item.clean_timezone_fields(version=self.account.version)  # Possibly also sets timezone values
-            meeting_tz_field, start_tz_field, end_tz_field = CalendarItem.timezone_fields()
-            if self.account.version.build < EXCHANGE_2010:
-                if 'start' in fieldnames_copy or 'end' in fieldnames_copy:
-                    fieldnames_copy.append(meeting_tz_field.name)
-            else:
-                if 'start' in fieldnames_copy:
-                    fieldnames_copy.append(start_tz_field.name)
-                if 'end' in fieldnames_copy:
-                    fieldnames_copy.append(end_tz_field.name)
-        else:
-            meeting_tz_field, start_tz_field, end_tz_field = None, None, None
+            for field_name in ('start', 'end'):
+                if field_name in fieldnames_copy:
+                    tz_field_name = item.tz_field_for_field_name(field_name).name
+                    if tz_field_name not in fieldnames_copy:
+                        fieldnames_copy.append(tz_field_name)
 
         for field in self._sorted_fields(item_model=item.__class__, fieldnames=fieldnames_copy):
             if field.is_read_only:
                 raise ValueError('%s is a read-only field' % field.name)
-            value = self._get_item_value(item, field, meeting_tz_field, start_tz_field, end_tz_field)
+            value = self._get_item_value(item, field)
             if value is None or (field.is_list and not value):
                 # A value of None or [] means we want to remove this field from the item
-                for elem in self._get_delete_item_elems(field=field):
-                    yield elem
+                yield from self._get_delete_item_elems(field=field)
             else:
-                for elem in self._get_set_item_elems(item_model=item.__class__, field=field, value=value):
-                    yield elem
+                yield from self._get_set_item_elems(item_model=item.__class__, field=field, value=value)
 
-    def _get_item_value(self, item, field, meeting_tz_field, start_tz_field, end_tz_field):
+    def _get_item_value(self, item, field):
         from ..items import CalendarItem
         value = field.clean(getattr(item, field.name), version=self.account.version)  # Make sure the value is OK
         if item.__class__ == CalendarItem:
             # For CalendarItem items where we update 'start' or 'end', we want to send values in the local timezone
-            if field.name in ('start', 'end') and type(value) == EWSDate:
-                # EWS always expects a datetime
-                return item.date_to_datetime(field_name=field.name)
-            if self.account.version.build < EXCHANGE_2010:
-                if field.name in ('start', 'end'):
-                    value = value.astimezone(getattr(item, meeting_tz_field.name))
-            else:
-                if field.name == 'start':
-                    value = value.astimezone(getattr(item, start_tz_field.name))
-                elif field.name == 'end':
-                    value = value.astimezone(getattr(item, end_tz_field.name))
+            if field.name in ('start', 'end'):
+                if type(value) is EWSDate:
+                    # EWS always expects a datetime
+                    return item.date_to_datetime(field_name=field.name)
+                tz_field_name = item.tz_field_for_field_name(field.name).name
+                return value.astimezone(getattr(item, tz_field_name))
         return value
 
     def _get_delete_item_elems(self, field):
-        from ..fields import FieldPath
         if field.is_required or field.is_required_after_save:
             raise ValueError('%s is a required field and may not be deleted' % field.name)
         for field_path in FieldPath(field=field).expand(version=self.account.version):
             yield self._delete_item_elem(field_path=field_path)
 
     def _get_set_item_elems(self, item_model, field, value):
-        from ..fields import FieldPath, IndexedField
-        from ..indexed_properties import MultiFieldIndexedElement
         if isinstance(field, IndexedField):
-            # TODO: Maybe the set/delete logic should extend into subfields, not just overwrite the whole item.
+            # Generate either set or delete elements for all combinations of labels and subfields
+            supported_labels = field.value_cls.get_field_by_fieldname('label')\
+                .supported_choices(version=self.account.version)
+            seen_labels = set()
+            subfields = field.value_cls.supported_fields(version=self.account.version)
             for v in value:
-                # TODO: We should also delete the labels that no longer exist in the list
-                if issubclass(field.value_cls, MultiFieldIndexedElement):
-                    # We have subfields. Generate SetItem XML for each subfield. SetItem only accepts items that
-                    # have the one value set that we want to change. Create a new IndexedField object that has
-                    # only that value set.
-                    for subfield in field.value_cls.supported_fields(version=self.account.version):
+                seen_labels.add(v.label)
+                for subfield in subfields:
+                    field_path = FieldPath(field=field, label=v.label, subfield=subfield)
+                    subfield_value = getattr(v, subfield.name)
+                    if not subfield_value:
+                        # Generate delete elements for blank subfield values
+                        yield self._delete_item_elem(field_path=field_path)
+                    else:
+                        # Generate set elements for non-null subfield values
                         yield self._set_item_elem(
                             item_model=item_model,
-                            field_path=FieldPath(field=field, label=v.label, subfield=subfield),
-                            value=field.value_cls(**{'label': v.label, subfield.name: getattr(v, subfield.name)}),
+                            field_path=field_path,
+                            value=field.value_cls(**{'label': v.label, subfield.name: subfield_value}),
                         )
-                else:
-                    # The simpler IndexedFields with only one subfield
-                    subfield = field.value_cls.value_field(version=self.account.version)
-                    yield self._set_item_elem(
-                        item_model=item_model,
-                        field_path=FieldPath(field=field, label=v.label, subfield=subfield),
-                        value=v,
-                    )
+                # Generate delete elements for all subfields of all labels not mentioned in the list of values
+                for label in (label for label in supported_labels if label not in seen_labels):
+                    for subfield in subfields:
+                        yield self._delete_item_elem(field_path=FieldPath(field=field, label=label, subfield=subfield))
         else:
             yield self._set_item_elem(item_model=item_model, field_path=FieldPath(field=field), value=value)
 
@@ -160,7 +154,6 @@ class UpdateItem(EWSAccountService, EWSPooledMixIn):
         # Takes a list of (Item, fieldnames) tuples where 'Item' is a instance of a subclass of Item and 'fieldnames'
         # are the attribute names that were updated. Returns the XML for an UpdateItem call.
         # an UpdateItem request.
-        from ..properties import ItemId
         if self.account.version.build >= EXCHANGE_2013_SP1:
             updateitem = create_element(
                 'm:%s' % self.SERVICE_NAME,
@@ -183,10 +176,11 @@ class UpdateItem(EWSAccountService, EWSPooledMixIn):
         itemchanges = create_element('m:ItemChanges')
         version = self.account.version
         for item, fieldnames in items:
+            if not item.account:
+                item.account = self.account
             if not fieldnames:
                 raise ValueError('"fieldnames" must not be empty')
             itemchange = create_element('t:ItemChange')
-            log.debug('Updating item %s fields %s', item, fieldnames)
             set_xml_value(itemchange, to_item_id(item, ItemId, version=version), version=version)
             updates = create_element('t:Updates')
             for elem in self._get_item_update_elems(item=item, fieldnames=fieldnames):
