@@ -1,29 +1,29 @@
-from .ewsdatetime import UTC_NOW
+import datetime
+
+from .ewsdatetime import UTC
 from .fields import DateTimeField, MessageField, ChoiceField, Choice
-from .properties import EWSElement, OutOfOffice, Fields
+from .properties import EWSElement, OutOfOffice
 from .util import create_element, set_xml_value
 
 
 class OofSettings(EWSElement):
     """MSDN: https://docs.microsoft.com/en-us/exchange/client-developer/web-service-reference/oofsettings"""
+
     ELEMENT_NAME = 'OofSettings'
     REQUEST_ELEMENT_NAME = 'UserOofSettings'
 
     ENABLED = 'Enabled'
     SCHEDULED = 'Scheduled'
     DISABLED = 'Disabled'
-    FIELDS = Fields(
-        ChoiceField('state', field_uri='OofState', is_required=True,
-                    choices={Choice(ENABLED), Choice(SCHEDULED), Choice(DISABLED)}),
-        ChoiceField('external_audience', field_uri='ExternalAudience',
-                    choices={Choice('None'), Choice('Known'), Choice('All')}, default='All'),
-        DateTimeField('start', field_uri='StartTime'),
-        DateTimeField('end', field_uri='EndTime'),
-        MessageField('internal_reply', field_uri='InternalReply'),
-        MessageField('external_reply', field_uri='ExternalReply'),
-    )
+    STATE_CHOICES = (ENABLED, SCHEDULED, DISABLED)
 
-    __slots__ = tuple(f.name for f in FIELDS)
+    state = ChoiceField(field_uri='OofState', is_required=True, choices={Choice(c) for c in STATE_CHOICES})
+    external_audience = ChoiceField(field_uri='ExternalAudience',
+                                    choices={Choice('None'), Choice('Known'), Choice('All')}, default='All')
+    start = DateTimeField(field_uri='StartTime')
+    end = DateTimeField(field_uri='EndTime')
+    internal_reply = MessageField(field_uri='InternalReply')
+    external_reply = MessageField(field_uri='ExternalReply')
 
     def clean(self, version=None):
         super().clean(version=version)
@@ -32,7 +32,7 @@ class OofSettings(EWSElement):
                 raise ValueError("'start' and 'end' must be set when state is '%s'" % self.SCHEDULED)
             if self.start >= self.end:
                 raise ValueError("'start' must be before 'end'")
-            if self.end < UTC_NOW():
+            if self.end < datetime.datetime.now(tz=UTC):
                 raise ValueError("'end' must be in the future")
         if self.state != self.DISABLED and (not self.internal_reply or not self.external_reply):
             raise ValueError("'internal_reply' and 'external_reply' must be set when state is not '%s'" % self.DISABLED)
