@@ -1,8 +1,7 @@
 import logging
 import re
 
-from .errors import TransportError, ErrorInvalidSchemaVersionForMailboxVersion, ErrorInvalidServerVersion, \
-    ErrorIncorrectSchemaVersion, ResponseMessageError
+from .errors import TransportError, ResponseMessageError
 from .util import xml_to_str, TNS
 
 log = logging.getLogger(__name__)
@@ -39,9 +38,7 @@ API_VERSIONS = sorted({v[0] for v in VERSIONS.values()}, reverse=True)
 
 
 class Build:
-    """
-    Holds methods for working with build numbers
-    """
+    """Holds methods for working with build numbers."""
 
     # List of build numbers here: https://docs.microsoft.com/en-us/exchange/new-features/build-numbers-and-release-dates
     API_VERSION_MAP = {
@@ -65,7 +62,7 @@ class Build:
         },
     }
 
-    __slots__ = ('major_version', 'minor_version', 'major_build', 'minor_build')
+    __slots__ = 'major_version', 'minor_version', 'major_build', 'minor_build'
 
     def __init__(self, major_version, minor_version, major_build=0, minor_build=0):
         if not isinstance(major_version, int):
@@ -110,6 +107,8 @@ class Build:
             * The next 6 bits contain the minor version number
             * The next bit contains a flag. Can be ignored
             * The next 15 bits contain the major build number
+
+        :param s:
         """
         bin_s = '{:032b}'.format(int(s, 16))  # Convert string to 32-bit binary string
         major_version = int(bin_s[4:10], 2)
@@ -125,6 +124,9 @@ class Build:
         except KeyError:
             raise ValueError('API version for build %s is unknown' % self)
 
+    def fullname(self):
+        return VERSIONS[self.api_version()][1]
+
     def __cmp__(self, other):
         # __cmp__ is not a magic method in Python3. We'll just use it here to implement comparison operators
         c = (self.major_version > other.major_version) - (self.major_version < other.major_version)
@@ -181,10 +183,9 @@ EXCHANGE_O365 = Build(15, 20)
 
 
 class Version:
-    """
-    Holds information about the server version
-    """
-    __slots__ = ('build', 'api_version')
+    """Holds information about the server version."""
+
+    __slots__ = 'build', 'api_version'
 
     def __init__(self, build, api_version=None):
         if not isinstance(build, (Build, type(None))):
@@ -203,13 +204,15 @@ class Version:
 
     @classmethod
     def guess(cls, protocol, api_version_hint=None):
-        """
-        Tries to ask the server which version it has. We haven't set up an Account object yet, so we generate requests
+        """Ask the server which version it has. We haven't set up an Account object yet, so we generate requests
         by hand. We only need a response header containing a ServerVersionInfo element.
 
         To get API version and build numbers from the server, we need to send a valid SOAP request. We can't do that
         without a valid API version. To solve this chicken-and-egg problem, we try all possible API versions that this
         package supports, until we get a valid response.
+
+        :param protocol:
+        :param api_version_hint:  (Default value = None)
         """
         from .services import ResolveNames
         # The protocol doesn't have a version yet, so default to latest supported version if we don't have a hint.
@@ -224,13 +227,12 @@ class Version:
         name = str(protocol.credentials) if protocol.credentials and str(protocol.credentials) else 'DUMMY'
         try:
             list(ResolveNames(protocol=protocol).call(unresolved_entries=[name]))
-        except (ErrorInvalidSchemaVersionForMailboxVersion, ErrorInvalidServerVersion, ErrorIncorrectSchemaVersion):
-            raise TransportError('Unable to guess version')
-        except ResponseMessageError:
-            # We survived long enough to get a new version
-            pass
-        if not protocol.version.build:
-            raise AttributeError('Protocol should have a build number at this point')
+        except ResponseMessageError as e:
+            # We may have survived long enough to get a new version
+            if not protocol.config.version.build:
+                raise TransportError('No valid version headers found in response (%r)' % e)
+        if not protocol.config.version.build:
+            raise TransportError('No valid version headers found in response')
         return protocol.version
 
     @staticmethod
