@@ -1,3 +1,4 @@
+import abc
 from collections import namedtuple
 import datetime
 from decimal import Decimal
@@ -8,24 +9,28 @@ import time
 import unittest
 import unittest.util
 
-import pytz
 from yaml import safe_load
+try:
+    import zoneinfo
+except ImportError:
+    from backports import zoneinfo
 
 from exchangelib.account import Account
 from exchangelib.attachments import FileAttachment
 from exchangelib.configuration import Configuration
 from exchangelib.credentials import DELEGATE, Credentials
-from exchangelib.errors import UnknownTimeZone, AmbiguousTimeError, NonExistentTimeError
-from exchangelib.ewsdatetime import EWSDateTime, EWSDate, EWSTimeZone, UTC
+from exchangelib.errors import UnknownTimeZone
+from exchangelib.ewsdatetime import EWSTimeZone
 from exchangelib.fields import BooleanField, IntegerField, DecimalField, TextField, EmailAddressField, URIField, \
     ChoiceField, BodyField, DateTimeField, Base64Field, PhoneNumberField, EmailAddressesField, TimeZoneField, \
     PhysicalAddressField, ExtendedPropertyField, MailboxField, AttendeesField, AttachmentField, CharListField, \
     MailboxListField, EWSElementField, CultureField, CharField, TextListField, PermissionSetField, MimeContentField, \
     DateField, DateTimeBackedDateField
 from exchangelib.indexed_properties import EmailAddress, PhysicalAddress, PhoneNumber
-from exchangelib.properties import Attendee, Mailbox, PermissionSet, Permission, UserId
+from exchangelib.properties import Attendee, Mailbox, PermissionSet, Permission, UserId, CompleteName,\
+    ReminderMessageData
 from exchangelib.protocol import BaseProtocol, NoVerifyHTTPAdapter, FaultTolerance
-from exchangelib.recurrence import Recurrence, DailyPattern
+from exchangelib.recurrence import Recurrence, TaskRecurrence, DailyPattern, DailyRegeneration
 from exchangelib.util import DummyResponse
 
 mock_account = namedtuple('mock_account', ('protocol', 'version'))
@@ -46,12 +51,7 @@ def mock_session_exception(exc_cls):
     return raise_exc
 
 
-class MockResponse(DummyResponse):
-    def __init__(self, c):
-        super().__init__(url='', headers={}, request_headers={}, content=c)
-
-
-class TimedTestCase(unittest.TestCase):
+class TimedTestCase(unittest.TestCase, metaclass=abc.ABCMeta):
     SLOW_TEST_DURATION = 5  # Log tests that are slower than this value (in seconds)
 
     def setUp(self):
@@ -64,7 +64,7 @@ class TimedTestCase(unittest.TestCase):
             print("{:07.3f} : {}".format(t2, self.id()))
 
 
-class EWSTest(TimedTestCase):
+class EWSTest(TimedTestCase, metaclass=abc.ABCMeta):
     @classmethod
     def setUpClass(cls):
         # There's no official Exchange server we can test against, and we can't really provide credentials for our
@@ -87,7 +87,7 @@ class EWSTest(TimedTestCase):
             BaseProtocol.HTTP_ADAPTER_CLS = NoVerifyHTTPAdapter
 
         # Create an account shared by all tests
-        tz = EWSTimeZone.timezone('Europe/Copenhagen')
+        tz = zoneinfo.ZoneInfo('Europe/Copenhagen')
         cls.retry_policy = FaultTolerance(max_wait=600)
         config = Configuration(
             server=settings['server'],
@@ -103,7 +103,7 @@ class EWSTest(TimedTestCase):
         self.categories = [get_random_string(length=16, spaces=False, special=False)]
 
     def wipe_test_account(self):
-        # Deletes up all deleteable items in the test account. Not run in a normal test run
+        # Deletes up all deletable items in the test account. Not run in a normal test run
         self.account.root.wipe(page_size=100)
 
     def bulk_delete(self, ids):
@@ -147,7 +147,7 @@ class EWSTest(TimedTestCase):
         if isinstance(field, TextField):
             return get_random_string(400)
         if isinstance(field, MimeContentField):
-            return get_random_bytes(400)
+            return get_random_string(400).encode('utf-8')
         if isinstance(field, Base64Field):
             return get_random_bytes(400)
         if isinstance(field, BooleanField):
@@ -163,21 +163,19 @@ class EWSTest(TimedTestCase):
         if isinstance(field, DateTimeField):
             return get_random_datetime(tz=self.account.default_timezone)
         if isinstance(field, AttachmentField):
-            return [FileAttachment(name='my_file.txt', content=get_random_bytes(400))]
+            return [FileAttachment(name='my_file.txt', content=get_random_string(400).encode('utf-8'))]
         if isinstance(field, MailboxListField):
             # email_address must be a real account on the server(?)
             # TODO: Mailbox has multiple optional args but vals must match server account, so we can't easily test
             if get_random_bool():
                 return [Mailbox(email_address=self.account.primary_smtp_address)]
-            else:
-                return [self.account.primary_smtp_address]
+            return [self.account.primary_smtp_address]
         if isinstance(field, MailboxField):
             # email_address must be a real account on the server(?)
             # TODO: Mailbox has multiple optional args but vals must match server account, so we can't easily test
             if get_random_bool():
                 return Mailbox(email_address=self.account.primary_smtp_address)
-            else:
-                return self.account.primary_smtp_address
+            return self.account.primary_smtp_address
         if isinstance(field, AttendeesField):
             # Attendee must refer to a real mailbox on the server(?). We're only sure to have one
             if get_random_bool():
@@ -190,11 +188,9 @@ class EWSTest(TimedTestCase):
                     Attendee(mailbox=mbx, response_type='Accept',
                              last_response_time=get_random_datetime(tz=self.account.default_timezone))
                 ]
-            else:
-                if get_random_bool():
-                    return [Attendee(mailbox=mbx, response_type='Accept')]
-                else:
-                    return [self.account.primary_smtp_address]
+            if get_random_bool():
+                return [Attendee(mailbox=mbx, response_type='Accept')]
+            return [self.account.primary_smtp_address]
         if isinstance(field, EmailAddressesField):
             addrs = []
             for label in EmailAddress.get_field_by_fieldname('label').supported_choices(version=self.account.version):
@@ -222,12 +218,38 @@ class EWSTest(TimedTestCase):
         if isinstance(field, EWSElementField):
             if field.value_cls == Recurrence:
                 return Recurrence(pattern=DailyPattern(interval=5), start=get_random_date(), number=7)
+            if field.value_cls == TaskRecurrence:
+                return TaskRecurrence(pattern=DailyRegeneration(interval=5), start=get_random_date(), number=7)
+            if field.value_cls == ReminderMessageData:
+                start = get_random_time()
+                end = get_random_time(start_time=start)
+                return ReminderMessageData(
+                    reminder_text=get_random_string(16),
+                    location=get_random_string(16),
+                    start_time=start,
+                    end_time=end,
+                )
+        if field.value_cls == CompleteName:
+            return CompleteName(
+                title=get_random_string(16),
+                first_name=get_random_string(16),
+                middle_name=get_random_string(16),
+                last_name=get_random_string(16),
+                suffix=get_random_string(16),
+                initials=get_random_string(16),
+                full_name=get_random_string(16),
+                nickname=get_random_string(16),
+                yomi_first_name=get_random_string(16),
+                yomi_last_name=get_random_string(16),
+            )
         if isinstance(field, TimeZoneField):
             while True:
+                tz = zoneinfo.ZoneInfo(random.choice(tuple(zoneinfo.available_timezones())))
                 try:
-                    return EWSTimeZone.timezone(random.choice(pytz.all_timezones))
+                    EWSTimeZone.from_zoneinfo(tz)
                 except UnknownTimeZone:
-                    pass
+                    continue
+                return tz
         if isinstance(field, PermissionSetField):
             return PermissionSet(
                 permissions=[
@@ -254,7 +276,7 @@ def get_random_decimal(min_val=0, max_val=100):
 
 
 def get_random_choice(choices):
-    return random.sample(choices, 1)[0]
+    return random.sample(tuple(choices), 1)[0]
 
 
 def get_random_string(length, spaces=True, special=True):
@@ -271,8 +293,12 @@ def get_random_string(length, spaces=True, special=True):
     return res
 
 
-def get_random_bytes(*args, **kwargs):
-    return get_random_string(*args, **kwargs).encode('utf-8')
+def get_random_byte():
+    return get_random_bytes(1)
+
+
+def get_random_bytes(length):
+    return bytes(get_random_int(max_val=255) for _ in range(length))
 
 
 def get_random_url():
@@ -295,28 +321,39 @@ def get_random_email():
     ))
 
 
+def _total_minutes(tm):
+    return (tm.hour * 60) + tm.minute
+
+
+def get_random_time(start_time=datetime.time.min, end_time=datetime.time.max):
+    # Create a random time with minute precision.
+    random_minutes = random.randint(_total_minutes(start_time), _total_minutes(end_time))
+    return datetime.time(hour=random_minutes // 60, minute=random_minutes % 60)
+
+
 # The timezone we're testing (CET/CEST) had a DST date change in 1996 (see
 # https://en.wikipedia.org/wiki/Summer_Time_in_Europe). The Microsoft timezone definition on the server
-# does not observe that, but pytz does. So random datetimes before 1996 will fail tests randomly.
-def get_random_date(start_date=EWSDate(1996, 1, 1), end_date=EWSDate(2030, 1, 1)):
+# does not observe that, but IANA does. So random datetimes before 1996 will fail tests randomly.
+RANDOM_DATE_MIN = datetime.date(1996, 1, 1)
+RANDOM_DATE_MAX = datetime.date(2030, 1, 1)
+UTC = zoneinfo.ZoneInfo('UTC')
+
+
+def get_random_date(start_date=RANDOM_DATE_MIN, end_date=RANDOM_DATE_MAX):
     # Keep with a reasonable date range. A wider date range is unstable WRT timezones
-    return EWSDate.fromordinal(random.randint(start_date.toordinal(), end_date.toordinal()))
+    return datetime.date.fromordinal(random.randint(start_date.toordinal(), end_date.toordinal()))
 
 
-def get_random_datetime(start_date=EWSDate(1996, 1, 1), end_date=EWSDate(2030, 1, 1), tz=UTC):
+def get_random_datetime(start_date=RANDOM_DATE_MIN, end_date=RANDOM_DATE_MAX, tz=UTC):
     # Create a random datetime with minute precision. Both dates are inclusive.
     # Keep with a reasonable date range. A wider date range than the default values is unstable WRT timezones.
-    while True:
-        try:
-            random_date = get_random_date(start_date=start_date, end_date=end_date)
-            random_datetime = datetime.datetime.combine(random_date, datetime.time.min) \
-                + datetime.timedelta(minutes=random.randint(0, 60 * 24))
-            return tz.localize(EWSDateTime.from_datetime(random_datetime), is_dst=None)
-        except (AmbiguousTimeError, NonExistentTimeError):
-            pass
+    random_date = get_random_date(start_date=start_date, end_date=end_date)
+    random_datetime = datetime.datetime.combine(random_date, datetime.time.min) \
+        + datetime.timedelta(minutes=random.randint(0, 60 * 24))
+    return random_datetime.replace(tzinfo=tz)
 
 
-def get_random_datetime_range(start_date=EWSDate(1996, 1, 1), end_date=EWSDate(2030, 1, 1), tz=UTC):
+def get_random_datetime_range(start_date=RANDOM_DATE_MIN, end_date=RANDOM_DATE_MAX, tz=UTC):
     # Create two random datetimes.  Both dates are inclusive.
     # Keep with a reasonable date range. A wider date range than the default values is unstable WRT timezones.
     # Calendar items raise ErrorCalendarDurationIsTooLong if duration is > 5 years.
