@@ -1,17 +1,24 @@
 from collections import namedtuple
 import pickle
 
-from exchangelib import Account, Credentials, FaultTolerance, Message, FileAttachment, DELEGATE, Configuration
+from exchangelib.account import Account
+from exchangelib.attachments import FileAttachment
+from exchangelib.configuration import Configuration
+from exchangelib.credentials import Credentials, DELEGATE
 from exchangelib.errors import ErrorAccessDenied, ErrorFolderNotFound, UnauthorizedError
 from exchangelib.folders import Calendar
+from exchangelib.items import Message
 from exchangelib.properties import DelegateUser, UserId, DelegatePermissions
-from exchangelib.protocol import Protocol
+from exchangelib.protocol import Protocol, FaultTolerance
 from exchangelib.services import GetDelegate
+from exchangelib.version import Version, EXCHANGE_2007_SP1
 
-from .common import EWSTest, MockResponse
+from .common import EWSTest
 
 
 class AccountTest(EWSTest):
+    """Test features of the Account object."""
+
     def test_magic(self):
         self.account.fullname = 'John Doe'
         self.assertIn(self.account.primary_smtp_address, str(self.account))
@@ -50,18 +57,18 @@ class AccountTest(EWSTest):
         self.assertNotEqual(folder.id, None)
         self.assertEqual(folder.name.lower(), Calendar.localized_names(self.account.locale)[0])
 
-        class MockCalendar(Calendar):
+        class MockCalendar1(Calendar):
             @classmethod
             def get_distinguished(cls, root):
                 raise ErrorAccessDenied('foo')
 
         # Test an indirect folder lookup with FindItems
-        folder = self.account.root.get_default_folder(MockCalendar)
-        self.assertIsInstance(folder, MockCalendar)
+        folder = self.account.root.get_default_folder(MockCalendar1)
+        self.assertIsInstance(folder, MockCalendar1)
         self.assertEqual(folder.id, None)
-        self.assertEqual(folder.name, MockCalendar.DISTINGUISHED_FOLDER_ID)
+        self.assertEqual(folder.name, MockCalendar1.DISTINGUISHED_FOLDER_ID)
 
-        class MockCalendar(Calendar):
+        class MockCalendar2(Calendar):
             @classmethod
             def get_distinguished(cls, root):
                 raise ErrorFolderNotFound('foo')
@@ -69,15 +76,15 @@ class AccountTest(EWSTest):
         # Test using the one folder of this folder type
         with self.assertRaises(ErrorFolderNotFound):
             # This fails because there are no folders of type MockCalendar
-            self.account.root.get_default_folder(MockCalendar)
+            self.account.root.get_default_folder(MockCalendar2)
 
         _orig = Calendar.get_distinguished
         try:
-            Calendar.get_distinguished = MockCalendar.get_distinguished
+            Calendar.get_distinguished = MockCalendar2.get_distinguished
             folder = self.account.root.get_default_folder(Calendar)
             self.assertIsInstance(folder, Calendar)
             self.assertNotEqual(folder.id, None)
-            self.assertEqual(folder.name.lower(), MockCalendar.localized_names(self.account.locale)[0])
+            self.assertEqual(folder.name.lower(), MockCalendar2.localized_names(self.account.locale)[0])
         finally:
             Calendar.get_distinguished = _orig
 
@@ -113,52 +120,50 @@ class AccountTest(EWSTest):
             len(self.account.delegates),
             0
         )
+        self.assertGreaterEqual(
+            len(list(GetDelegate(account=self.account).call(user_ids=['foo@example.com'], include_permissions=True))),
+            0
+        )
 
-        xml = b'''<?xml version="1.0" encoding="utf-8"?>
-        <soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"
-                       xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
-                       xmlns:xsd="http://www.w3.org/2001/XMLSchema">
-          <soap:Header>
-            <t:ServerVersionInfo MajorBuildNumber="845" MajorVersion="15" MinorBuildNumber="22" MinorVersion="1"
-                                 Version="V2016_10_10"
-                                 xmlns:t="http://schemas.microsoft.com/exchange/services/2006/types" />
-          </soap:Header>
-          <soap:Body>
-            <m:GetDelegateResponse xmlns:t="http://schemas.microsoft.com/exchange/services/2006/types"
-                                   ResponseClass="Success"
-                                   xmlns:m="http://schemas.microsoft.com/exchange/services/2006/messages">
-              <m:ResponseCode>NoError</m:ResponseCode>
-              <m:ResponseMessages>
-                <m:DelegateUserResponseMessageType ResponseClass="Success">
-                  <m:ResponseCode>NoError</m:ResponseCode>
-                  <m:DelegateUser>
-                      <t:UserId>
-                        <t:SID>SOME_SID</t:SID>
-                        <t:PrimarySmtpAddress>foo@example.com</t:PrimarySmtpAddress>
-                        <t:DisplayName>Foo Bar</t:DisplayName>
-                      </t:UserId>
-                      <t:DelegatePermissions>
-                        <t:CalendarFolderPermissionLevel>Author</t:CalendarFolderPermissionLevel>
-                        <t:InboxFolderPermissionLevel>Reviewer</t:ContactsFolderPermissionLevel>
-                      </t:DelegatePermissions>
-                      <t:ReceiveCopiesOfMeetingMessages>false</t:ReceiveCopiesOfMeetingMessages>
-                    <t:ViewPrivateItems>true</t:ViewPrivateItems>
-                    </m:DelegateUser>
-                  </m:DelegateUserResponseMessageType>
-              </m:ResponseMessages>
-              <m:DeliverMeetingRequests>DelegatesAndMe</m:DeliverMeetingRequests>
-              </m:GetDelegateResponse>
-          </soap:Body>
-        </soap:Envelope>'''
+        xml = b'''\
+<?xml version="1.0" encoding="utf-8"?>
+<s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">
+  <s:Body>
+    <m:GetDelegateResponse ResponseClass="Success"
+            xmlns:t="http://schemas.microsoft.com/exchange/services/2006/types"
+            xmlns:m="http://schemas.microsoft.com/exchange/services/2006/messages">
+      <m:ResponseCode>NoError</m:ResponseCode>
+      <m:ResponseMessages>
+        <m:DelegateUserResponseMessageType ResponseClass="Success">
+          <m:ResponseCode>NoError</m:ResponseCode>
+          <m:DelegateUser>
+              <t:UserId>
+                <t:SID>SOME_SID</t:SID>
+                <t:PrimarySmtpAddress>foo@example.com</t:PrimarySmtpAddress>
+                <t:DisplayName>Foo Bar</t:DisplayName>
+              </t:UserId>
+              <t:DelegatePermissions>
+                <t:CalendarFolderPermissionLevel>Author</t:CalendarFolderPermissionLevel>
+                <t:InboxFolderPermissionLevel>Reviewer</t:ContactsFolderPermissionLevel>
+              </t:DelegatePermissions>
+              <t:ReceiveCopiesOfMeetingMessages>false</t:ReceiveCopiesOfMeetingMessages>
+            <t:ViewPrivateItems>true</t:ViewPrivateItems>
+            </m:DelegateUser>
+          </m:DelegateUserResponseMessageType>
+      </m:ResponseMessages>
+      <m:DeliverMeetingRequests>DelegatesAndMe</m:DeliverMeetingRequests>
+      </m:GetDelegateResponse>
+  </s:Body>
+</s:Envelope>'''
 
         MockTZ = namedtuple('EWSTimeZone', ['ms_id'])
         MockAccount = namedtuple('Account', ['access_type', 'primary_smtp_address', 'default_timezone', 'protocol'])
-        a = MockAccount(DELEGATE, 'foo@example.com', MockTZ('XXX'), protocol='foo')
+        MockProtocol = namedtuple('Protocol', ['version'])
+        p = MockProtocol(version=Version(build=EXCHANGE_2007_SP1))
+        a = MockAccount(DELEGATE, 'foo@example.com', MockTZ('XXX'), protocol=p)
 
         ws = GetDelegate(account=a)
-        header, body = ws._get_soap_parts(response=MockResponse(xml))
-        res = ws._get_elements_in_response(response=ws._get_soap_messages(body=body))
-        delegates = [DelegateUser.from_xml(elem=elem, account=a) for elem in res]
+        delegates = list(ws.parse(xml))
         self.assertListEqual(
             delegates,
             [
@@ -193,30 +198,25 @@ class AccountTest(EWSTest):
             autodiscover=False,
             locale='da_DK',
         )
+
         # Should fail when credentials are wrong, but UnauthorizedError is caught and retried. Mock the needed methods
-        import exchangelib.util
+        class Mock1(FaultTolerance):
+            def may_retry_on_error(self, response, wait):
+                if response.status_code == 401:
+                    return False
+                return super().may_retry_on_error(response, wait)
 
-        _orig1 = exchangelib.util._may_retry_on_error
-        _orig2 = exchangelib.util._raise_response_errors
+            def raise_response_errors(self, response):
+                if response.status_code == 401:
+                    raise UnauthorizedError('Invalid credentials for %s' % response.url)
+                return super().raise_response_errors(response)
 
-        def _mock1(response, retry_policy, wait):
-            if response.status_code == 401:
-                return False
-            return _orig1(response, retry_policy, wait)
-
-        def _mock2(response, protocol, log_msg, log_vals):
-            if response.status_code == 401:
-                raise UnauthorizedError('Invalid credentials for %s' % response.url)
-            return _orig2(response, protocol, log_msg, log_vals)
-
-        exchangelib.util._may_retry_on_error = _mock1
-        exchangelib.util._raise_response_errors = _mock2
         try:
+            account.protocol.config.retry_policy = Mock1()
             with self.assertRaises(UnauthorizedError):
                 account.root.refresh()
         finally:
-            exchangelib.util._may_retry_on_error = _orig1
-            exchangelib.util._raise_response_errors = _orig2
+            account.protocol.config.retry_policy = self.retry_policy
 
         # Cannot update from Configuration object
         with self.assertRaises(AttributeError):
