@@ -2,8 +2,7 @@ from exchangelib.attachments import FileAttachment, ItemAttachment, AttachmentId
 from exchangelib.errors import ErrorItemNotFound, ErrorInvalidIdMalformed
 from exchangelib.folders import Inbox
 from exchangelib.items import Item, Message
-from exchangelib.services import GetAttachment
-from exchangelib.util import chunkify, TNS
+from exchangelib.util import chunkify
 
 from .test_items.test_basics import BaseItemTest
 from .common import get_random_string
@@ -37,7 +36,7 @@ class AttachmentsTest(BaseItemTest):
         att1.parent_item = None
         att1.attachment_id = None
 
-    def test_attachment_properties(self):
+    def test_file_attachment_properties(self):
         binary_file_content = 'Hello from unicode æøå'.encode('utf-8')
         att1 = FileAttachment(name='my_file_1.txt', content=binary_file_content)
         self.assertIn("name='my_file_1.txt'", str(att1))
@@ -49,17 +48,63 @@ class AttachmentsTest(BaseItemTest):
         with self.assertRaises(ValueError):
             print(att1.content)  # Test property getter when we need to fetch the content
 
+    def test_item_attachment_properties(self):
         attached_item1 = self.get_test_item(folder=self.test_folder)
-        att2 = ItemAttachment(name='attachment1', item=attached_item1)
-        self.assertIn("name='attachment1'", str(att2))
-        att2.item = attached_item1  # Test property setter
-        self.assertEqual(att2.item, attached_item1)  # Test property getter
-        self.assertEqual(att2.item, attached_item1)  # Test property getter
-        att2.attachment_id = 'xxx'
-        self.assertEqual(att2.item, attached_item1)  # Test property getter when attachment_id is set
-        att2._item = None
+        att1 = ItemAttachment(name='attachment1', item=attached_item1)
+        self.assertIn("name='attachment1'", str(att1))
+        att1.item = attached_item1  # Test property setter
+        self.assertEqual(att1.item, attached_item1)  # Test property getter
+        self.assertEqual(att1.item, attached_item1)  # Test property getter
+        att1.attachment_id = 'xxx'
+        self.assertEqual(att1.item, attached_item1)  # Test property getter when attachment_id is set
+        att1._item = None
         with self.assertRaises(ValueError):
-            print(att2.item)  # Test property getter when we need to fetch the item
+            print(att1.item)  # Test property getter when we need to fetch the item
+
+    def test_item_attachments(self):
+        item = self.get_test_item(folder=self.test_folder)
+        attached_item1 = self.get_test_item(folder=self.test_folder)
+        att1 = ItemAttachment(name='attachment1', item=attached_item1)
+
+        # Test __init__(attachments=...) and attach() on new item
+        self.assertEqual(len(item.attachments), 0)
+        item.attach(att1)
+        self.assertEqual(len(item.attachments), 1)
+        item.save()
+        fresh_item = self.get_item_by_id(item)
+        self.assertEqual(len(fresh_item.attachments), 1)
+        fresh_attachments = sorted(fresh_item.attachments, key=lambda a: a.name)
+        self.assertEqual(fresh_attachments[0].name, 'attachment1')
+        self.assertEqual(fresh_attachments[0].item.subject, attached_item1.subject)
+        self.assertEqual(fresh_attachments[0].item.body, attached_item1.body)
+        # Same as 'body' because 'body' doesn't contain HTML
+        self.assertEqual(fresh_attachments[0].item.text_body, attached_item1.body)
+
+        # Test attach on saved object
+        att2 = ItemAttachment(name='attachment2', item=attached_item1)
+        self.assertEqual(len(item.attachments), 1)
+        item.attach(att2)
+        self.assertEqual(len(item.attachments), 2)
+        fresh_item = self.get_item_by_id(item)
+        self.assertEqual(len(fresh_item.attachments), 2)
+        fresh_attachments = sorted(fresh_item.attachments, key=lambda a: a.name)
+        self.assertEqual(fresh_attachments[0].name, 'attachment1')
+        self.assertEqual(fresh_attachments[0].item.subject, attached_item1.subject)
+        self.assertEqual(fresh_attachments[0].item.body, attached_item1.body)
+        self.assertEqual(fresh_attachments[1].name, 'attachment2')
+        self.assertEqual(fresh_attachments[1].item.subject, attached_item1.subject)
+        self.assertEqual(fresh_attachments[1].item.body, attached_item1.body)
+
+        # Test detach
+        item.detach(att1)
+        self.assertTrue(att1.attachment_id is None)
+        self.assertTrue(att1.parent_item is None)
+        fresh_item = self.get_item_by_id(item)
+        self.assertEqual(len(fresh_item.attachments), 1)
+        fresh_attachments = sorted(fresh_item.attachments, key=lambda a: a.name)
+        self.assertEqual(fresh_attachments[0].name, 'attachment2')
+        self.assertEqual(fresh_attachments[0].item.subject, attached_item1.subject)
+        self.assertEqual(fresh_attachments[0].item.body, attached_item1.body)
 
     def test_file_attachments(self):
         item = self.get_test_item(folder=self.test_folder)
@@ -71,26 +116,18 @@ class AttachmentsTest(BaseItemTest):
         item.attach(att1)
         self.assertEqual(len(item.attachments), 1)
         item.save()
-        fresh_item = list(self.account.fetch(ids=[item]))[0]
+        fresh_item = self.get_item_by_id(item)
         self.assertEqual(len(fresh_item.attachments), 1)
         fresh_attachments = sorted(fresh_item.attachments, key=lambda a: a.name)
         self.assertEqual(fresh_attachments[0].name, 'my_file_1.txt')
         self.assertEqual(fresh_attachments[0].content, binary_file_content)
 
-        # Test raw call to service
-        self.assertEqual(
-            list(GetAttachment(account=item.account).call(
-                items=[att1.attachment_id],
-                include_mime_content=False)
-            )[0].find('{%s}Content' % TNS).text,
-            'SGVsbG8gZnJvbSB1bmljb2RlIMOmw7jDpQ==')
-
         # Test attach on saved object
         att2 = FileAttachment(name='my_file_2.txt', content=binary_file_content)
         self.assertEqual(len(item.attachments), 1)
         item.attach(att2)
         self.assertEqual(len(item.attachments), 2)
-        fresh_item = list(self.account.fetch(ids=[item]))[0]
+        fresh_item = self.get_item_by_id(item)
         self.assertEqual(len(fresh_item.attachments), 2)
         fresh_attachments = sorted(fresh_item.attachments, key=lambda a: a.name)
         self.assertEqual(fresh_attachments[0].name, 'my_file_1.txt')
@@ -102,7 +139,7 @@ class AttachmentsTest(BaseItemTest):
         item.detach(att1)
         self.assertTrue(att1.attachment_id is None)
         self.assertTrue(att1.parent_item is None)
-        fresh_item = list(self.account.fetch(ids=[item]))[0]
+        fresh_item = self.get_item_by_id(item)
         self.assertEqual(len(fresh_item.attachments), 1)
         fresh_attachments = sorted(fresh_item.attachments, key=lambda a: a.name)
         self.assertEqual(fresh_attachments[0].name, 'my_file_2.txt')
@@ -116,12 +153,12 @@ class AttachmentsTest(BaseItemTest):
         item.save()
 
         # Test streaming file content
-        fresh_item = list(self.account.fetch(ids=[item]))[0]
+        fresh_item = self.get_item_by_id(item)
         with fresh_item.attachments[0].fp as fp:
             self.assertEqual(fp.read(), large_binary_file_content)
 
         # Test partial reads of streaming file content
-        fresh_item = list(self.account.fetch(ids=[item]))[0]
+        fresh_item = self.get_item_by_id(item)
         with fresh_item.attachments[0].fp as fp:
             chunked_reads = []
             buffer = fp.read(7)
@@ -157,7 +194,7 @@ class AttachmentsTest(BaseItemTest):
         att1 = FileAttachment(name='empty_file.txt', content=b'')
         item.attach(att1)
         item.save()
-        fresh_item = list(self.account.fetch(ids=[item]))[0]
+        fresh_item = self.get_item_by_id(item)
         self.assertEqual(
             fresh_item.attachments[0].content,
             b''
@@ -172,7 +209,7 @@ class AttachmentsTest(BaseItemTest):
         item.attach(file_attachment)
         item.save()
 
-        fresh_item = list(self.account.fetch(ids=[item]))[0]
+        fresh_item = self.get_item_by_id(item)
         self.assertSetEqual(
             {a.name for a in fresh_item.attachments},
             {'item_attachment', 'file_attachment'}
