@@ -5,16 +5,18 @@ from types import MethodType
 import dns
 import requests_mock
 
-from exchangelib import DELEGATE
+from exchangelib.account import Account
+from exchangelib.credentials import Credentials, DELEGATE
 import exchangelib.autodiscover.discovery
-from exchangelib import Credentials, NTLM, FailFast, Configuration, Account
 from exchangelib.autodiscover import close_connections, clear_cache, autodiscover_cache, AutodiscoverProtocol, \
     Autodiscovery
 from exchangelib.autodiscover.properties import Autodiscover
+from exchangelib.configuration import Configuration
 from exchangelib.errors import ErrorNonExistentMailbox, AutoDiscoverCircularRedirect, AutoDiscoverFailed
-from exchangelib.protocol import FaultTolerance
+from exchangelib.protocol import FaultTolerance, FailFast
+from exchangelib.transport import NTLM
 from exchangelib.util import get_domain
-from .common import EWSTest
+from .common import EWSTest, get_random_string
 
 
 class AutodiscoverTest(EWSTest):
@@ -22,7 +24,7 @@ class AutodiscoverTest(EWSTest):
         super().setUp()
 
         # Enable retries, to make tests more robust
-        Autodiscovery.INITIAL_RETRY_POLICY = FaultTolerance(max_wait=30)
+        Autodiscovery.INITIAL_RETRY_POLICY = FaultTolerance(max_wait=5)
         Autodiscovery.RETRY_WAIT = 5
 
         # Each test should start with a clean autodiscover cache
@@ -51,12 +53,12 @@ class AutodiscoverTest(EWSTest):
 </Autodiscover>''' % (self.account.primary_smtp_address.encode(), self.dummy_ews_endpoint.encode())
         self.dummy_ews_response = b'''\
 <?xml version='1.0' encoding='utf-8'?>
-<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
-  <soap:Header>
+<s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">
+  <s:Header>
     <h:ServerVersionInfo xmlns:h="http://schemas.microsoft.com/exchange/services/2006/types"
     MajorVersion="15" MinorVersion="1" MajorBuildNumber="1847" MinorBuildNumber="5" Version="V2017_07_11"/>
-  </soap:Header>
-  <soap:Body>
+  </s:Header>
+  <s:Body>
     <m:ResolveNamesResponse
     xmlns:m="http://schemas.microsoft.com/exchange/services/2006/messages"
     xmlns:t="http://schemas.microsoft.com/exchange/services/2006/types">
@@ -68,15 +70,14 @@ class AutodiscoverTest(EWSTest):
         </m:ResolveNamesResponseMessage>
       </m:ResponseMessages>
     </m:ResolveNamesResponse>
-  </soap:Body>
-</soap:Envelope>
+  </s:Body>
+</s:Envelope>
 '''
 
     @requests_mock.mock(real_http=False)  # Just make sure we don't issue any real HTTP here
     def test_magic(self, m):
         # Just test we don't fail when calling repr() and str(). Insert a dummy cache entry for testing
-        clear_cache()
-        c = Credentials('leet_user', 'cannaguess')
+        c = Credentials(get_random_string(8), get_random_string(8))
         autodiscover_cache[('example.com', c)] = AutodiscoverProtocol(config=Configuration(
             service_endpoint='https://example.com/Autodiscover/Autodiscover.xml',
             credentials=c,
@@ -92,7 +93,6 @@ class AutodiscoverTest(EWSTest):
 
     def test_autodiscover_empty_cache(self):
         # A live test of the entire process with an empty cache
-        clear_cache()
         ad_response, protocol = exchangelib.autodiscover.discovery.discover(
             email=self.account.primary_smtp_address,
             credentials=self.account.protocol.credentials,
@@ -123,8 +123,6 @@ class AutodiscoverTest(EWSTest):
             )
 
     def test_failed_login_via_account(self):
-        Autodiscovery.INITIAL_RETRY_POLICY = FaultTolerance(max_wait=10)
-        clear_cache()
         with self.assertRaises(AutoDiscoverFailed):
             Account(
                 primary_smtp_address=self.account.primary_smtp_address,
@@ -137,8 +135,7 @@ class AutodiscoverTest(EWSTest):
     @requests_mock.mock(real_http=False)  # Just make sure we don't issue any real HTTP here
     def test_close_autodiscover_connections(self, m):
         # A live test that we can close TCP connections
-        clear_cache()
-        c = Credentials('leet_user', 'cannaguess')
+        c = Credentials(get_random_string(8), get_random_string(8))
         autodiscover_cache[('example.com', c)] = AutodiscoverProtocol(config=Configuration(
             service_endpoint='https://example.com/Autodiscover/Autodiscover.xml',
             credentials=c,
@@ -151,8 +148,7 @@ class AutodiscoverTest(EWSTest):
     @requests_mock.mock(real_http=False)  # Just make sure we don't issue any real HTTP here
     def test_autodiscover_direct_gc(self, m):
         # Test garbage collection of the autodiscover cache
-        clear_cache()
-        c = Credentials('leet_user', 'cannaguess')
+        c = Credentials(get_random_string(8), get_random_string(8))
         autodiscover_cache[('example.com', c)] = AutodiscoverProtocol(config=Configuration(
             service_endpoint='https://example.com/Autodiscover/Autodiscover.xml',
             credentials=c,
@@ -187,7 +183,7 @@ class AutodiscoverTest(EWSTest):
         # Poison the cache with a failing autodiscover endpoint. discover() must handle this and rebuild the cache
         autodiscover_cache[discovery._cache_key] = AutodiscoverProtocol(config=Configuration(
             service_endpoint='https://example.com/Autodiscover/Autodiscover.xml',
-            credentials=Credentials('leet_user', 'cannaguess'),
+            credentials=Credentials(get_random_string(8), get_random_string(8)),
             auth_type=NTLM,
             retry_policy=FailFast(),
         ))
@@ -223,17 +219,22 @@ class AutodiscoverTest(EWSTest):
         autodiscover_cache[key] = namedtuple('P', ['service_endpoint', 'auth_type', 'retry_policy'])(1, 'bar', 'baz')
         # Check that it exists. 'in' goes directly to the file
         self.assertTrue(key in autodiscover_cache)
-        # Destroy the backing cache file(s)
+
+        # Check that we can recover from a destroyed file
         for db_file in glob.glob(autodiscover_cache._storage_file + '*'):
             with open(db_file, 'w') as f:
                 f.write('XXX')
-        # Check that we can recover from a destroyed file and that the entry no longer exists
+        self.assertFalse(key in autodiscover_cache)
+
+        # Check that we can recover from an empty file
+        for db_file in glob.glob(autodiscover_cache._storage_file + '*'):
+            with open(db_file, 'wb') as f:
+                f.write(b'')
         self.assertFalse(key in autodiscover_cache)
 
     @requests_mock.mock(real_http=False)  # Just make sure we don't issue any real HTTP here
     def test_autodiscover_from_account(self, m):
         # Test that autodiscovery via account creation works
-        clear_cache()
         # Mock the default endpoint that we test in step 1 of autodiscovery
         m.post(self.dummy_ad_endpoint, status_code=200, content=self.dummy_ad_response)
         # Also mock the EWS URL. We try to guess its auth method as part of autodiscovery
@@ -274,7 +275,6 @@ class AutodiscoverTest(EWSTest):
     def test_autodiscover_redirect(self, m):
         # Test various aspects of autodiscover redirection. Mock all HTTP responses because we can't force a live server
         # to send us into the correct code paths.
-        clear_cache()
         # Mock the default endpoint that we test in step 1 of autodiscovery
         m.post(self.dummy_ad_endpoint, status_code=200, content=self.dummy_ad_response)
         # Also mock the EWS URL. We try to guess its auth method as part of autodiscovery
@@ -306,7 +306,7 @@ class AutodiscoverTest(EWSTest):
 </Autodiscover>''')
         # Also mock the EWS URL. We try to guess its auth method as part of autodiscovery
         m.post('https://expr.example.com/EWS/Exchange.asmx', status_code=200)
-        ad_response, p = discovery.discover()
+        ad_response, _ = discovery.discover()
         self.assertEqual(ad_response.autodiscover_smtp_address, 'john@example.com')
 
         # Make sure we discover an address redirect to the same domain. We have to mock the same URL with two different
@@ -345,9 +345,9 @@ class AutodiscoverTest(EWSTest):
             dict(status_code=200, content=redirect_addr_content),
             dict(status_code=200, content=settings_content),
         ])
-        ad_response, p = discovery.discover()
+        ad_response, _ = discovery.discover()
         self.assertEqual(ad_response.autodiscover_smtp_address, 'redirected@%s' % self.domain)
-        self.assertEqual(ad_response.protocol.ews_url, 'https://redirected.%s/EWS/Exchange.asmx' % self.domain)
+        self.assertEqual(ad_response.ews_url, 'https://redirected.%s/EWS/Exchange.asmx' % self.domain)
 
         # Test that we catch circular redirects on the same domain with a primed cache. Just mock the endpoint to
         # return the same redirect response on every request.
@@ -373,72 +373,84 @@ class AutodiscoverTest(EWSTest):
             discovery.discover()
 
         # Test that we can handle being asked to redirect to an address on a different domain
+        # Don't use example.com to redirect - it does not resolve or answer on all ISPs
         m.post(self.dummy_ad_endpoint, status_code=200, content=b'''\
 <?xml version="1.0" encoding="utf-8"?>
 <Autodiscover xmlns="http://schemas.microsoft.com/exchange/autodiscover/responseschema/2006">
     <Response xmlns="http://schemas.microsoft.com/exchange/autodiscover/outlook/responseschema/2006a">
         <Account>
             <Action>redirectAddr</Action>
-            <RedirectAddr>john@example.com</RedirectAddr>
+            <RedirectAddr>john@httpbin.org</RedirectAddr>
         </Account>
     </Response>
 </Autodiscover>''')
-        m.post('https://example.com/Autodiscover/Autodiscover.xml', status_code=200, content=b'''\
+        m.post('https://httpbin.org/Autodiscover/Autodiscover.xml', status_code=200, content=b'''\
 <?xml version="1.0" encoding="utf-8"?>
 <Autodiscover xmlns="http://schemas.microsoft.com/exchange/autodiscover/responseschema/2006">
     <Response xmlns="http://schemas.microsoft.com/exchange/autodiscover/outlook/responseschema/2006a">
         <User>
-            <AutoDiscoverSMTPAddress>john@redirected.example.com</AutoDiscoverSMTPAddress>
+            <AutoDiscoverSMTPAddress>john@redirected.httpbin.org</AutoDiscoverSMTPAddress>
         </User>
         <Account>
             <AccountType>email</AccountType>
             <Action>settings</Action>
             <Protocol>
                 <Type>EXPR</Type>
-                <EwsUrl>https://redirected.example.com/EWS/Exchange.asmx</EwsUrl>
+                <EwsUrl>https://httpbin.org/EWS/Exchange.asmx</EwsUrl>
             </Protocol>
         </Account>
     </Response>
 </Autodiscover>''')
         # Also mock the EWS URL. We try to guess its auth method as part of autodiscovery
-        m.post('https://redirected.example.com/EWS/Exchange.asmx', status_code=200)
-        ad_response, p = discovery.discover()
-        self.assertEqual(ad_response.autodiscover_smtp_address, 'john@redirected.example.com')
-        self.assertEqual(ad_response.protocol.ews_url, 'https://redirected.example.com/EWS/Exchange.asmx')
+        m.post('https://httpbin.org/EWS/Exchange.asmx', status_code=200)
+        ad_response, _ = discovery.discover()
+        self.assertEqual(ad_response.autodiscover_smtp_address, 'john@redirected.httpbin.org')
+        self.assertEqual(ad_response.ews_url, 'https://httpbin.org/EWS/Exchange.asmx')
 
     def test_get_srv_records(self):
-        from exchangelib.autodiscover.discovery import _get_srv_records, SrvRecord
+        from exchangelib.autodiscover.discovery import SrvRecord
+        ad = Autodiscovery('foo@example.com')
         # Unknown domain
-        self.assertEqual(_get_srv_records('example.XXXXX'), [])
+        self.assertEqual(ad._get_srv_records('example.XXXXX'), [])
         # No SRV record
-        self.assertEqual(_get_srv_records('example.com'), [])
+        self.assertEqual(ad._get_srv_records('example.com'), [])
         # Finding a real server that has a correct SRV record is not easy. Mock it
         _orig = dns.resolver.Resolver
 
         class _Mock1:
-            def query(self, hostname, cat):
+            @staticmethod
+            def resolve(hostname, cat):
                 class A:
-                    def to_text(self):
+                    @staticmethod
+                    def to_text():
                         # Return a valid record
                         return '1 2 3 example.com.'
                 return [A()]
 
         dns.resolver.Resolver = _Mock1
+        del ad.resolver
         # Test a valid record
-        self.assertEqual(_get_srv_records('example.com.'), [SrvRecord(priority=1, weight=2, port=3, srv='example.com')])
+        self.assertEqual(
+            ad._get_srv_records('example.com.'),
+            [SrvRecord(priority=1, weight=2, port=3, srv='example.com')]
+        )
 
         class _Mock2:
-            def query(self, hostname, cat):
+            @staticmethod
+            def resolve(hostname, cat):
                 class A:
-                    def to_text(self):
+                    @staticmethod
+                    def to_text():
                         # Return malformed data
                         return 'XXXXXXX'
                 return [A()]
 
         dns.resolver.Resolver = _Mock2
+        del ad.resolver
         # Test an invalid record
-        self.assertEqual(_get_srv_records('example.com'), [])
+        self.assertEqual(ad._get_srv_records('example.com'), [])
         dns.resolver.Resolver = _orig
+        del ad.resolver
 
     def test_select_srv_host(self):
         from exchangelib.autodiscover.discovery import _select_srv_host, SrvRecord
@@ -534,7 +546,7 @@ class AutodiscoverTest(EWSTest):
     </Response>
 </Autodiscover>'''
         self.assertEqual(
-            Autodiscover.from_bytes(xml).response.protocol.ews_url,
+            Autodiscover.from_bytes(xml).response.ews_url,
             'https://expr.example.com/EWS/Exchange.asmx'
         )
 
@@ -557,7 +569,7 @@ class AutodiscoverTest(EWSTest):
     </Response>
 </Autodiscover>'''
         self.assertEqual(
-            Autodiscover.from_bytes(xml).response.protocol.ews_url,
+            Autodiscover.from_bytes(xml).response.ews_url,
             'https://exch.example.com/EWS/Exchange.asmx'
         )
 
@@ -580,4 +592,4 @@ class AutodiscoverTest(EWSTest):
     </Response>
 </Autodiscover>'''
         with self.assertRaises(ValueError):
-            Autodiscover.from_bytes(xml).response.protocol
+            Autodiscover.from_bytes(xml).response.ews_url
