@@ -1,8 +1,9 @@
-from exchangelib import Message, Mailbox, CalendarItem
-from exchangelib.extended_properties import ExtendedProperty
+from exchangelib.extended_properties import ExtendedProperty, Flag
+from exchangelib.items import Message, CalendarItem
 from exchangelib.folders import Inbox
+from exchangelib.properties import Mailbox
 
-from .common import get_random_int
+from .common import get_random_int, get_random_url
 from .test_items.test_basics import BaseItemTest
 
 
@@ -81,10 +82,6 @@ class ExtendedPropertyTest(BaseItemTest):
             self.ITEM_CLASS.deregister(attr_name=attr_name)
 
     def test_extended_property_with_tag(self):
-        class Flag(ExtendedProperty):
-            property_tag = 0x1090
-            property_type = 'Integer'
-
         attr_name = 'my_flag'
         self.ITEM_CLASS.register(attr_name=attr_name, attr_cls=Flag)
         try:
@@ -112,10 +109,6 @@ class ExtendedPropertyTest(BaseItemTest):
             InvalidProp('Foo').clean()  # property_tag is in protected range
 
     def test_extended_property_with_string_tag(self):
-        class Flag(ExtendedProperty):
-            property_tag = '0x1090'
-            property_type = 'Integer'
-
         attr_name = 'my_flag'
         self.ITEM_CLASS.register(attr_name=attr_name, attr_cls=Flag)
         try:
@@ -137,7 +130,7 @@ class ExtendedPropertyTest(BaseItemTest):
     def test_extended_distinguished_property(self):
         if self.ITEM_CLASS == CalendarItem:
             # MyMeeting is an extended prop version of the 'CalendarItem.uid' field. They don't work together.
-            raise self.skipTest("This extendedproperty doesn't work on CalendarItems")
+            self.skipTest("This extendedproperty doesn't work on CalendarItems")
 
         class MyMeeting(ExtendedProperty):
             distinguished_property_set_id = 'Meeting'
@@ -152,12 +145,12 @@ class ExtendedPropertyTest(BaseItemTest):
             prop_val = item.my_meeting
             self.assertTrue(isinstance(prop_val, bytes))
             item.save()
-            item = list(self.account.fetch(ids=[(item.id, item.changekey)]))[0]
+            item = self.get_item_by_id((item.id, item.changekey))
             self.assertEqual(prop_val, item.my_meeting, (prop_val, item.my_meeting))
             new_prop_val = self.random_val(self.ITEM_CLASS.get_field_by_fieldname(attr_name))
             item.my_meeting = new_prop_val
             item.save()
-            item = list(self.account.fetch(ids=[(item.id, item.changekey)]))[0]
+            item = self.get_item_by_id((item.id, item.changekey))
             self.assertEqual(new_prop_val, item.my_meeting)
         finally:
             self.ITEM_CLASS.deregister(attr_name=attr_name)
@@ -177,105 +170,126 @@ class ExtendedPropertyTest(BaseItemTest):
             prop_val = item.my_meeting_array
             self.assertTrue(isinstance(prop_val, list))
             item.save()
-            item = list(self.account.fetch(ids=[(item.id, item.changekey)]))[0]
+            item = self.get_item_by_id((item.id, item.changekey))
             self.assertEqual(prop_val, item.my_meeting_array)
             new_prop_val = self.random_val(self.ITEM_CLASS.get_field_by_fieldname(attr_name))
             item.my_meeting_array = new_prop_val
             item.save()
-            item = list(self.account.fetch(ids=[(item.id, item.changekey)]))[0]
+            item = self.get_item_by_id((item.id, item.changekey))
             self.assertEqual(new_prop_val, item.my_meeting_array)
         finally:
             self.ITEM_CLASS.deregister(attr_name=attr_name)
 
     def test_extended_property_validation(self):
-        """
-        if cls.property_type not in cls.PROPERTY_TYPES:
-            raise ValueError(
-                "'property_type' value '%s' must be one of %s" % (cls.property_type, sorted(cls.PROPERTY_TYPES))
-            )
-        """
         # Must not have property_set_id or property_tag
-        class TestProp(ExtendedProperty):
+        class TestProp1(ExtendedProperty):
             distinguished_property_set_id = 'XXX'
             property_set_id = 'YYY'
         with self.assertRaises(ValueError):
-            TestProp.validate_cls()
+            TestProp1.validate_cls()
 
         # Must have property_id or property_name
-        class TestProp(ExtendedProperty):
+        class TestProp2(ExtendedProperty):
             distinguished_property_set_id = 'XXX'
         with self.assertRaises(ValueError):
-            TestProp.validate_cls()
+            TestProp2.validate_cls()
 
         # distinguished_property_set_id must have a valid value
-        class TestProp(ExtendedProperty):
+        class TestProp3(ExtendedProperty):
             distinguished_property_set_id = 'XXX'
             property_id = 'YYY'
         with self.assertRaises(ValueError):
-            TestProp.validate_cls()
+            TestProp3.validate_cls()
 
         # Must not have distinguished_property_set_id or property_tag
-        class TestProp(ExtendedProperty):
+        class TestProp4(ExtendedProperty):
             property_set_id = 'XXX'
             property_tag = 'YYY'
         with self.assertRaises(ValueError):
-            TestProp.validate_cls()
+            TestProp4.validate_cls()
 
         # Must have property_id or property_name
-        class TestProp(ExtendedProperty):
+        class TestProp5(ExtendedProperty):
             property_set_id = 'XXX'
         with self.assertRaises(ValueError):
-            TestProp.validate_cls()
+            TestProp5.validate_cls()
 
         # property_tag is only compatible with property_type
-        class TestProp(ExtendedProperty):
+        class TestProp6(ExtendedProperty):
             property_tag = 'XXX'
             property_set_id = 'YYY'
         with self.assertRaises(ValueError):
-            TestProp.validate_cls()
+            TestProp6.validate_cls()
 
         # property_tag must be an integer or string that can be converted to int
-        class TestProp(ExtendedProperty):
+        class TestProp7(ExtendedProperty):
             property_tag = 'XXX'
         with self.assertRaises(ValueError):
-            TestProp.validate_cls()
+            TestProp7.validate_cls()
 
         # property_tag must not be in the reserved range
-        class TestProp(ExtendedProperty):
+        class TestProp8(ExtendedProperty):
             property_tag = 0x8001
         with self.assertRaises(ValueError):
-            TestProp.validate_cls()
+            TestProp8.validate_cls()
 
         # Must not have property_id or property_tag
-        class TestProp(ExtendedProperty):
+        class TestProp9(ExtendedProperty):
             property_name = 'XXX'
             property_id = 'YYY'
         with self.assertRaises(ValueError):
-            TestProp.validate_cls()
+            TestProp9.validate_cls()
 
         # Must have distinguished_property_set_id or property_set_id
-        class TestProp(ExtendedProperty):
+        class TestProp10(ExtendedProperty):
             property_name = 'XXX'
         with self.assertRaises(ValueError):
-            TestProp.validate_cls()
+            TestProp10.validate_cls()
 
         # Must not have property_name or property_tag
-        class TestProp(ExtendedProperty):
+        class TestProp11(ExtendedProperty):
             property_id = 'XXX'
             property_name = 'YYY'
         with self.assertRaises(ValueError):
-            TestProp.validate_cls()  # This actually hits the check on property_name values
+            TestProp11.validate_cls()  # This actually hits the check on property_name values
 
         # Must have distinguished_property_set_id or property_set_id
-        class TestProp(ExtendedProperty):
+        class TestProp12(ExtendedProperty):
             property_id = 'XXX'
         with self.assertRaises(ValueError):
-            TestProp.validate_cls()
+            TestProp12.validate_cls()
 
         # property_type must be a valid value
-        class TestProp(ExtendedProperty):
+        class TestProp13(ExtendedProperty):
             property_id = 'XXX'
             property_set_id = 'YYY'
             property_type = 'ZZZ'
         with self.assertRaises(ValueError):
-            TestProp.validate_cls()
+            TestProp13.validate_cls()
+
+    def test_multiple_extended_properties(self):
+        class ExternalSharingUrl(ExtendedProperty):
+            property_set_id = "F52A8693-C34D-4980-9E20-9D4C1EABB6A7"
+            property_name = "ExternalSharingUrl"
+            property_type = "String"
+
+        class ExternalSharingFolderId(ExtendedProperty):
+            property_set_id = "F52A8693-C34D-4980-9E20-9D4C1EABB6A7"
+            property_name = "ExternalSharingLocalFolderId"
+            property_type = "Binary"
+
+        try:
+            self.ITEM_CLASS.register("sharing_url", ExternalSharingUrl)
+            self.ITEM_CLASS.register("sharing_folder_id", ExternalSharingFolderId)
+
+            url, folder_id = get_random_url(), self.test_folder.id.encode('utf-8')
+            m = self.get_test_item()
+            m.sharing_url, m.sharing_folder_id = url, folder_id
+            m.save()
+
+            m = self.test_folder.get(sharing_url=url)
+            self.assertEqual(m.sharing_url, url)
+            self.assertEqual(m.sharing_folder_id, folder_id)
+        finally:
+            self.ITEM_CLASS.deregister("sharing_url")
+            self.ITEM_CLASS.deregister("sharing_folder_id")
