@@ -1,15 +1,17 @@
 from collections import namedtuple
+import datetime
 from decimal import Decimal
+try:
+    import zoneinfo
+except ImportError:
+    from backports import zoneinfo
 
-from exchangelib import Version, EWSDateTime, EWSTimeZone, UTC
-from exchangelib.errors import ErrorInvalidServerVersion
 from exchangelib.extended_properties import ExternId
 from exchangelib.fields import BooleanField, IntegerField, DecimalField, TextField, ChoiceField, DateTimeField, \
     Base64Field, TimeZoneField, ExtendedPropertyField, CharListField, Choice, DateField, EnumField, EnumListField, \
-    CharField
+    CharField, InvalidFieldForVersion, InvalidChoiceForVersion
 from exchangelib.indexed_properties import SingleFieldIndexedElement
-from exchangelib.properties import Fields
-from exchangelib.version import EXCHANGE_2007, EXCHANGE_2010, EXCHANGE_2013
+from exchangelib.version import Version, EXCHANGE_2007, EXCHANGE_2010, EXCHANGE_2013
 from exchangelib.util import to_xml, TNS
 
 from .common import TimedTestCase
@@ -50,7 +52,7 @@ class FieldTest(TimedTestCase):
 
         field = DateTimeField('foo', field_uri='bar')
         with self.assertRaises(ValueError) as e:
-            field.clean(EWSDateTime(2017, 1, 1))  # Datetime values must be timezone aware
+            field.clean(datetime.datetime(2017, 1, 1))  # Datetime values must be timezone aware
         self.assertEqual(str(e.exception), "Value '2017-01-01 00:00:00' on field 'foo' must be timezone aware")
 
         field = ChoiceField('foo', field_uri='bar', choices=[Choice('foo'), Choice('bar')])
@@ -130,7 +132,7 @@ class FieldTest(TimedTestCase):
 
     def test_garbage_input(self):
         # Test that we can survive garbage input for common field types
-        tz = EWSTimeZone.timezone('Europe/Copenhagen')
+        tz = zoneinfo.ZoneInfo('Europe/Copenhagen')
         account = namedtuple('Account', ['default_timezone'])(default_timezone=tz)
         payload = b'''\
 <?xml version="1.0" encoding="utf-8"?>
@@ -158,7 +160,7 @@ class FieldTest(TimedTestCase):
 
     def test_versioned_field(self):
         field = TextField('foo', field_uri='bar', supported_from=EXCHANGE_2010)
-        with self.assertRaises(ErrorInvalidServerVersion):
+        with self.assertRaises(InvalidFieldForVersion):
             field.clean('baz', version=Version(EXCHANGE_2007))
         field.clean('baz', version=Version(EXCHANGE_2010))
         field.clean('baz', version=Version(EXCHANGE_2013))
@@ -170,16 +172,17 @@ class FieldTest(TimedTestCase):
         with self.assertRaises(ValueError):
             field.clean('XXX')  # Value must be a valid choice
         field.clean('c2', version=None)
-        with self.assertRaises(ErrorInvalidServerVersion):
+        with self.assertRaises(InvalidChoiceForVersion):
             field.clean('c2', version=Version(EXCHANGE_2007))
         field.clean('c2', version=Version(EXCHANGE_2010))
         field.clean('c2', version=Version(EXCHANGE_2013))
 
     def test_naive_datetime(self):
         # Test that we can survive naive datetimes on a datetime field
-        tz = EWSTimeZone.timezone('Europe/Copenhagen')
+        tz = zoneinfo.ZoneInfo('Europe/Copenhagen')
+        utc = zoneinfo.ZoneInfo('UTC')
         account = namedtuple('Account', ['default_timezone'])(default_timezone=tz)
-        default_value = tz.localize(EWSDateTime(2017, 1, 2, 3, 4))
+        default_value = datetime.datetime(2017, 1, 2, 3, 4, tzinfo=tz)
         field = DateTimeField('foo', field_uri='item:DateTimeSent', default=default_value)
 
         # TZ-aware datetime string
@@ -191,7 +194,9 @@ class FieldTest(TimedTestCase):
     </t:Item>
 </Envelope>'''
         elem = to_xml(payload).find('{%s}Item' % TNS)
-        self.assertEqual(field.from_xml(elem=elem, account=account), UTC.localize(EWSDateTime(2017, 6, 21, 18, 40, 2)))
+        self.assertEqual(
+            field.from_xml(elem=elem, account=account), datetime.datetime(2017, 6, 21, 18, 40, 2, tzinfo=utc)
+        )
 
         # Naive datetime string is localized to tz of the account
         payload = b'''\
@@ -202,7 +207,9 @@ class FieldTest(TimedTestCase):
     </t:Item>
 </Envelope>'''
         elem = to_xml(payload).find('{%s}Item' % TNS)
-        self.assertEqual(field.from_xml(elem=elem, account=account), tz.localize(EWSDateTime(2017, 6, 21, 18, 40, 2)))
+        self.assertEqual(
+            field.from_xml(elem=elem, account=account), datetime.datetime(2017, 6, 21, 18, 40, 2, tzinfo=tz)
+        )
 
         # Garbage string returns None
         payload = b'''\
@@ -228,7 +235,8 @@ class FieldTest(TimedTestCase):
     def test_single_field_indexed_element(self):
         # A SingleFieldIndexedElement must have only one field defined
         class TestField(SingleFieldIndexedElement):
-            FIELDS = Fields(CharField('a'), CharField('b'))
+            a = CharField()
+            b = CharField()
 
         with self.assertRaises(ValueError):
             TestField.value_field()
