@@ -1,9 +1,9 @@
 import datetime
 
 from exchangelib.errors import ErrorInvalidOperation, ErrorItemNotFound
-from exchangelib.ewsdatetime import EWSDateTime, UTC
+from exchangelib.ewsdatetime import UTC
 from exchangelib.folders import Calendar
-from exchangelib.items import CalendarItem
+from exchangelib.items import CalendarItem, BulkCreateResult
 from exchangelib.items.calendar_item import SINGLE, OCCURRENCE, EXCEPTION, RECURRING_MASTER
 from exchangelib.recurrence import Recurrence, DailyPattern, Occurrence, FirstOccurrence, LastOccurrence, \
     DeletedOccurrence
@@ -18,8 +18,18 @@ class CalendarTest(CommonItemTest):
     ITEM_CLASS = CalendarItem
 
     def match_cat(self, i):
+        if isinstance(i, Exception):
+            return False
         return set(i.categories or []) == set(self.categories)
 
+    def test_cancel(self):
+        item = self.get_test_item().save()
+        res = item.cancel()  # Returns (id, changekey) of cancelled item
+        self.assertIsInstance(res, BulkCreateResult)
+        with self.assertRaises(ErrorItemNotFound):
+            # Item is already cancelled
+            item.cancel()
+
     def test_updating_timestamps(self):
         # Test that we can update an item without changing anything, and maintain the hidden timezone fields as local
         # timezones, and that returned timestamps are in UTC.
@@ -109,19 +119,20 @@ class CalendarTest(CommonItemTest):
             account=self.account,
             folder=self.test_folder,
             subject=get_random_string(16),
-            start=self.account.default_timezone.localize(EWSDateTime(2016, 1, 1, 8)),
-            end=self.account.default_timezone.localize(EWSDateTime(2016, 1, 1, 10)),
+            start=datetime.datetime(2016, 1, 1, 8, tzinfo=self.account.default_timezone),
+            end=datetime.datetime(2016, 1, 1, 10, tzinfo=self.account.default_timezone),
             categories=self.categories,
         )
         item2 = self.ITEM_CLASS(
             account=self.account,
             folder=self.test_folder,
             subject=get_random_string(16),
-            start=self.account.default_timezone.localize(EWSDateTime(2016, 2, 1, 8)),
-            end=self.account.default_timezone.localize(EWSDateTime(2016, 2, 1, 10)),
+            start=datetime.datetime(2016, 2, 1, 8, tzinfo=self.account.default_timezone),
+            end=datetime.datetime(2016, 2, 1, 10, tzinfo=self.account.default_timezone),
             categories=self.categories,
         )
         self.test_folder.bulk_create(items=[item1, item2])
+        qs = self.test_folder.view(start=item1.start, end=item2.end)
 
         # Test missing args
         with self.assertRaises(TypeError):
@@ -156,7 +167,9 @@ class CalendarTest(CommonItemTest):
 
         # Test max_items
         self.assertEqual(
-            len([i for i in self.test_folder.view(start=item1.start, end=item2.end, max_items=9999) if self.match_cat(i)]),
+            len([
+                i for i in self.test_folder.view(start=item1.start, end=item2.end, max_items=9999) if self.match_cat(i)
+            ]),
             2
         )
         self.assertEqual(
@@ -164,8 +177,21 @@ class CalendarTest(CommonItemTest):
             1
         )
 
+        # Test client-side ordering
+        self.assertListEqual(
+            [i.subject for i in qs.order_by('subject') if self.match_cat(i)], sorted([item1.subject, item2.subject])
+        )
+        # Test client-side ordering on a field with no default value and no default value_cls value
+        self.assertListEqual(
+            [i.start for i in qs.order_by('-start') if self.match_cat(i)], [item2.start, item1.start]
+        )
+        # Test client-side ordering on multiple fields. Intentionally sort first on a field where values are equal,
+        # to see that we can sort on the 2nd field.
+        self.assertListEqual(
+            [i.start for i in qs.order_by('categories', '-start') if self.match_cat(i)], [item2.start, item1.start]
+        )
+
         # Test chaining
-        qs = self.test_folder.view(start=item1.start, end=item2.end)
         self.assertTrue(qs.count() >= 2)
         with self.assertRaises(ErrorInvalidOperation):
             qs.filter(subject=item1.subject).count()  # EWS does not allow restrictions
@@ -174,13 +200,41 @@ class CalendarTest(CommonItemTest):
             [{'subject': s} for s in sorted([item1.subject, item2.subject])]
         )
 
+    def test_client_side_ordering_on_mixed_all_day_and_normal(self):
+        # Test that client-side ordering on start and end fields works for items that are a mix of normal an all-day
+        # items. This requires us to compare datetime.datetime -> EWSDate values which is not allowed by default
+        # (EWSDate -> datetime.datetime *is* allowed).
+        start = datetime.datetime(2016, 1, 1, 8, tzinfo=self.account.default_timezone)
+        end = datetime.datetime(2016, 1, 1, 10, tzinfo=self.account.default_timezone)
+        all_day_date = (start - datetime.timedelta(days=1)).date()
+        item1 = self.ITEM_CLASS(
+            account=self.account,
+            folder=self.test_folder,
+            subject=get_random_string(16),
+            start=all_day_date,
+            end=all_day_date,
+            is_all_day=True,
+            categories=self.categories,
+        )
+        item2 = self.ITEM_CLASS(
+            account=self.account,
+            folder=self.test_folder,
+            subject=get_random_string(16),
+            start=start,
+            end=end,
+            categories=self.categories,
+        )
+        self.test_folder.bulk_create(items=[item1, item2])
+        list(self.test_folder.view(start=start - datetime.timedelta(days=1), end=end).order_by('start'))
+        list(self.test_folder.view(start=start - datetime.timedelta(days=1), end=end).order_by('-start'))
+
     def test_recurring_item(self):
         # Create a recurring calendar item. Test that occurrence fields are correct on the master item
 
         # Create a master item with 4 daily occurrences from 8:00 to 10:00. 'start' and 'end' are values for the first
         # occurrence.
-        start = self.account.default_timezone.localize(EWSDateTime(2016, 1, 1, 8))
-        end = self.account.default_timezone.localize(EWSDateTime(2016, 1, 1, 10))
+        start = datetime.datetime(2016, 1, 1, 8, tzinfo=self.account.default_timezone)
+        end = datetime.datetime(2016, 1, 1, 10, tzinfo=self.account.default_timezone)
         master_item = self.ITEM_CLASS(
             folder=self.test_folder,
             start=start,
@@ -229,8 +283,8 @@ class CalendarTest(CommonItemTest):
 
     def test_change_occurrence(self):
         # Test that we can make changes to individual occurrences and see the effect on the master item.
-        start = self.account.default_timezone.localize(EWSDateTime(2016, 1, 1, 8))
-        end = self.account.default_timezone.localize(EWSDateTime(2016, 1, 1, 10))
+        start = datetime.datetime(2016, 1, 1, 8, tzinfo=self.account.default_timezone)
+        end = datetime.datetime(2016, 1, 1, 10, tzinfo=self.account.default_timezone)
         master_item = self.ITEM_CLASS(
             folder=self.test_folder,
             start=start,
@@ -271,8 +325,8 @@ class CalendarTest(CommonItemTest):
 
     def test_delete_occurrence(self):
         # Test that we can delete an occurrence and see the cange on the master item
-        start = self.account.default_timezone.localize(EWSDateTime(2016, 1, 1, 8))
-        end = self.account.default_timezone.localize(EWSDateTime(2016, 1, 1, 10))
+        start = datetime.datetime(2016, 1, 1, 8, tzinfo=self.account.default_timezone)
+        end = datetime.datetime(2016, 1, 1, 10, tzinfo=self.account.default_timezone)
         master_item = self.ITEM_CLASS(
             folder=self.test_folder,
             start=start,
@@ -304,8 +358,8 @@ class CalendarTest(CommonItemTest):
 
     def test_change_occurrence_via_index(self):
         # Test updating occurrences via occurrence index without knowing the ID of the occurrence.
-        start = self.account.default_timezone.localize(EWSDateTime(2016, 1, 1, 8))
-        end = self.account.default_timezone.localize(EWSDateTime(2016, 1, 1, 10))
+        start = datetime.datetime(2016, 1, 1, 8, tzinfo=self.account.default_timezone)
+        end = datetime.datetime(2016, 1, 1, 10, tzinfo=self.account.default_timezone)
         master_item = self.ITEM_CLASS(
             folder=self.test_folder,
             start=start,
@@ -341,8 +395,8 @@ class CalendarTest(CommonItemTest):
 
     def test_delete_occurrence_via_index(self):
         # Test deleting occurrences via occurrence index without knowing the ID of the occurrence.
-        start = self.account.default_timezone.localize(EWSDateTime(2016, 1, 1, 8))
-        end = self.account.default_timezone.localize(EWSDateTime(2016, 1, 1, 10))
+        start = datetime.datetime(2016, 1, 1, 8, tzinfo=self.account.default_timezone)
+        end = datetime.datetime(2016, 1, 1, 10, tzinfo=self.account.default_timezone)
         master_item = self.ITEM_CLASS(
             folder=self.test_folder,
             start=start,
@@ -369,14 +423,15 @@ class CalendarTest(CommonItemTest):
 
     def test_get_master_recurrence(self):
         # Test getting the master recurrence via an occurrence
-        start = self.account.default_timezone.localize(EWSDateTime(2016, 1, 1, 8))
-        end = self.account.default_timezone.localize(EWSDateTime(2016, 1, 1, 10))
+        start = datetime.datetime(2016, 1, 1, 8, tzinfo=self.account.default_timezone)
+        end = datetime.datetime(2016, 1, 1, 10, tzinfo=self.account.default_timezone)
+        recurrence = Recurrence(pattern=DailyPattern(interval=1), start=start.date(), number=4)
         master_item = self.ITEM_CLASS(
             folder=self.test_folder,
             start=start,
             end=end,
             subject=get_random_string(16),
-            recurrence=Recurrence(pattern=DailyPattern(interval=1), start=start.date(), number=4),
+            recurrence=recurrence,
             categories=self.categories,
         ).save()
 
@@ -384,9 +439,11 @@ class CalendarTest(CommonItemTest):
         range_start, range_end = start, end + datetime.timedelta(days=3)
         unfolded = [i for i in self.test_folder.view(start=range_start, end=range_end) if self.match_cat(i)]
         third_occurrence = unfolded[2]
+        self.assertEqual(third_occurrence.recurrence, None)
         master_from_occurrence = third_occurrence.recurring_master()
 
         master_from_occurrence.refresh()  # Test that GetItem works
+        self.assertEqual(master_from_occurrence.recurrence, recurrence)
         self.assertEqual(master_from_occurrence.subject, master_item.subject)
 
         master_from_occurrence = third_occurrence.recurring_master()
@@ -396,3 +453,5 @@ class CalendarTest(CommonItemTest):
 
         with self.assertRaises(ErrorItemNotFound):
             master_item.delete()  # Item is gone from the server, so this should fail
+        with self.assertRaises(ErrorItemNotFound):
+            third_occurrence.delete()  # Item is gone from the server, so this should fail
