@@ -1,8 +1,11 @@
 import datetime
+try:
+    import zoneinfo
+except ImportError:
+    from backports import zoneinfo
 
 from exchangelib.attachments import ItemAttachment
-from exchangelib.errors import ErrorItemNotFound
-from exchangelib.ewsdatetime import UTC_NOW
+from exchangelib.errors import ErrorItemNotFound, ErrorInternalServerError
 from exchangelib.extended_properties import ExtendedProperty, ExternId
 from exchangelib.fields import ExtendedPropertyField, CharField
 from exchangelib.folders import Inbox, FolderCollection
@@ -16,7 +19,8 @@ from .test_basics import CommonItemTest
 
 
 class GenericItemTest(CommonItemTest):
-    # Tests that don't need to be run for every single folder type
+    """Tests that don't need to be run for every single folder type"""
+
     TEST_FOLDER = 'inbox'
     FOLDER_CLASS = Inbox
     ITEM_CLASS = Message
@@ -34,94 +38,104 @@ class GenericItemTest(CommonItemTest):
                         setattr(item, f.name, 'a')
 
     def test_invalid_direct_args(self):
+        item = self.get_test_item()
+        item.account = None
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
-            item.account = None
             item.save()  # Must have account on save
+        item = self.get_test_item()
+        item.id = 'XXX'  # Fake a saved item
+        item.account = None
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
-            item.id = 'XXX'  # Fake a saved item
-            item.account = None
             item.save()  # Must have account on update
+        item = self.get_test_item()
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
             item.save(update_fields=['foo', 'bar'])  # update_fields is only valid on update
 
+        item = self.get_test_item()
+        item.account = None
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
-            item.account = None
             item.refresh()  # Must have account on refresh
+        item = self.get_test_item()
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
             item.refresh()  # Refresh an item that has not been saved
+        item = self.get_test_item()
+        item.save()
+        item_id, changekey = item.id, item.changekey
+        item.delete()
+        item.id, item.changekey = item_id, changekey
         with self.assertRaises(ErrorItemNotFound):
-            item = self.get_test_item()
-            item.save()
-            item_id, changekey = item.id, item.changekey
-            item.delete()
-            item.id, item.changekey = item_id, changekey
             item.refresh()  # Refresh an item that doesn't exist
 
+        item = self.get_test_item()
+        item.account = None
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
-            item.account = None
             item.copy(to_folder=self.test_folder)  # Must have an account on copy
+        item = self.get_test_item()
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
             item.copy(to_folder=self.test_folder)  # Must be an existing item
+        item = self.get_test_item()
+        item.save()
+        item_id, changekey = item.id, item.changekey
+        item.delete()
+        item.id, item.changekey = item_id, changekey
         with self.assertRaises(ErrorItemNotFound):
-            item = self.get_test_item()
-            item.save()
-            item_id, changekey = item.id, item.changekey
-            item.delete()
-            item.id, item.changekey = item_id, changekey
             item.copy(to_folder=self.test_folder)  # Item disappeared
 
+        item = self.get_test_item()
+        item.account = None
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
-            item.account = None
             item.move(to_folder=self.test_folder)  # Must have an account on move
+        item = self.get_test_item()
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
             item.move(to_folder=self.test_folder)  # Must be an existing item
+        item = self.get_test_item()
+        item.save()
+        item_id, changekey = item.id, item.changekey
+        item.delete()
+        item.id, item.changekey = item_id, changekey
         with self.assertRaises(ErrorItemNotFound):
-            item = self.get_test_item()
-            item.save()
-            item_id, changekey = item.id, item.changekey
-            item.delete()
-            item.id, item.changekey = item_id, changekey
             item.move(to_folder=self.test_folder)  # Item disappeared
 
+        item = self.get_test_item()
+        item.account = None
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
-            item.account = None
             item.delete()  # Must have an account
+        item = self.get_test_item()
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
             item.delete()  # Must be an existing item
+        item = self.get_test_item()
+        item.save()
+        item_id, changekey = item.id, item.changekey
+        item.delete()
+        item.id, item.changekey = item_id, changekey
         with self.assertRaises(ErrorItemNotFound):
-            item = self.get_test_item()
-            item.save()
-            item_id, changekey = item.id, item.changekey
-            item.delete()
-            item.id, item.changekey = item_id, changekey
             item.delete()  # Item disappeared
+        item = self.get_test_item()
+        with self.assertRaises(ValueError):
+            item._delete(delete_type='FOO', send_meeting_cancellations=None, affected_task_occurrences=None,
+                         suppress_read_receipts=None)
+        with self.assertRaises(ValueError):
+            item.delete(send_meeting_cancellations='XXX')
+        with self.assertRaises(ValueError):
+            item.delete(affected_task_occurrences='XXX')
+        with self.assertRaises(ValueError):
+            item.delete(suppress_read_receipts='XXX')
 
     def test_invalid_kwargs_on_send(self):
         # Only Message class has the send() method
+        item = self.get_test_item()
+        item.account = None
         with self.assertRaises(ValueError):
-            item = self.get_test_item()
-            item.account = None
             item.send()  # Must have account on send
+        item = self.get_test_item()
+        item.save()
+        item_id, changekey = item.id, item.changekey
+        item.delete()
+        item.id, item.changekey = item_id, changekey
         with self.assertRaises(ErrorItemNotFound):
-            item = self.get_test_item()
-            item.save()
-            item_id, changekey = item.id, item.changekey
-            item.delete()
-            item.id, item.changekey = item_id, changekey
             item.send()  # Item disappeared
+        item = self.get_test_item()
         with self.assertRaises(AttributeError):
-            item = self.get_test_item()
             item.send(copy_to_folder=self.account.trash, save_copy=False)  # Inconsistent args
 
     def test_unsupported_fields(self):
@@ -163,11 +177,11 @@ class GenericItemTest(CommonItemTest):
             folder_collection=FolderCollection(account=self.account, folders=[self.test_folder])
         ).filter(categories__contains=self.categories)
         self.assertEqual(
-            [i for i in qs.order_by('subject').values_list('subject', flat=True)],
+            list(qs.order_by('subject').values_list('subject', flat=True)),
             ['Subj 0', 'Subj 1', 'Subj 2', 'Subj 3']
         )
         self.assertEqual(
-            [i for i in qs.order_by('-subject').values_list('subject', flat=True)],
+            list(qs.order_by('-subject').values_list('subject', flat=True)),
             ['Subj 3', 'Subj 2', 'Subj 1', 'Subj 0']
         )
         self.bulk_delete(qs)
@@ -185,11 +199,11 @@ class GenericItemTest(CommonItemTest):
                 folder_collection=FolderCollection(account=self.account, folders=[self.test_folder])
             ).filter(categories__contains=self.categories)
             self.assertEqual(
-                [i for i in qs.order_by('extern_id').values_list('extern_id', flat=True)],
+                list(qs.order_by('extern_id').values_list('extern_id', flat=True)),
                 ['ID 0', 'ID 1', 'ID 2', 'ID 3']
             )
             self.assertEqual(
-                [i for i in qs.order_by('-extern_id').values_list('extern_id', flat=True)],
+                list(qs.order_by('-extern_id').values_list('extern_id', flat=True)),
                 ['ID 3', 'ID 2', 'ID 1', 'ID 0']
             )
         finally:
@@ -211,28 +225,28 @@ class GenericItemTest(CommonItemTest):
                 folder_collection=FolderCollection(account=self.account, folders=[self.test_folder])
             ).filter(categories__contains=self.categories)
             self.assertEqual(
-                [i for i in qs.order_by('subject', 'extern_id').values('subject', 'extern_id')],
+                list(qs.order_by('subject', 'extern_id').values('subject', 'extern_id')),
                 [{'subject': 'Subj 0', 'extern_id': 'ID 0'},
                  {'subject': 'Subj 0', 'extern_id': 'ID 1'},
                  {'subject': 'Subj 1', 'extern_id': 'ID 0'},
                  {'subject': 'Subj 1', 'extern_id': 'ID 1'}]
             )
             self.assertEqual(
-                [i for i in qs.order_by('-subject', 'extern_id').values('subject', 'extern_id')],
+                list(qs.order_by('-subject', 'extern_id').values('subject', 'extern_id')),
                 [{'subject': 'Subj 1', 'extern_id': 'ID 0'},
                  {'subject': 'Subj 1', 'extern_id': 'ID 1'},
                  {'subject': 'Subj 0', 'extern_id': 'ID 0'},
                  {'subject': 'Subj 0', 'extern_id': 'ID 1'}]
             )
             self.assertEqual(
-                [i for i in qs.order_by('subject', '-extern_id').values('subject', 'extern_id')],
+                list(qs.order_by('subject', '-extern_id').values('subject', 'extern_id')),
                 [{'subject': 'Subj 0', 'extern_id': 'ID 1'},
                  {'subject': 'Subj 0', 'extern_id': 'ID 0'},
                  {'subject': 'Subj 1', 'extern_id': 'ID 1'},
                  {'subject': 'Subj 1', 'extern_id': 'ID 0'}]
             )
             self.assertEqual(
-                [i for i in qs.order_by('-subject', '-extern_id').values('subject', 'extern_id')],
+                list(qs.order_by('-subject', '-extern_id').values('subject', 'extern_id')),
                 [{'subject': 'Subj 1', 'extern_id': 'ID 1'},
                  {'subject': 'Subj 1', 'extern_id': 'ID 0'},
                  {'subject': 'Subj 0', 'extern_id': 'ID 1'},
@@ -241,8 +255,55 @@ class GenericItemTest(CommonItemTest):
         finally:
             self.ITEM_CLASS.deregister('extern_id')
 
+    def test_order_by_with_empty_values(self):
+        # Test order_by() when some values are empty
+        test_items = []
+        for i in range(4):
+            item = self.get_test_item()
+            if i % 2 == 0:
+                item.subject = 'Subj %s' % i
+            else:
+                item.subject = None
+            test_items.append(item)
+        self.test_folder.bulk_create(items=test_items)
+        qs = QuerySet(
+            folder_collection=FolderCollection(account=self.account, folders=[self.test_folder])
+        ).filter(categories__contains=self.categories)
+        self.assertEqual(
+            list(qs.order_by('subject').values_list('subject', flat=True)),
+            [None, None, 'Subj 0', 'Subj 2']
+        )
+        self.assertEqual(
+            list(qs.order_by('-subject').values_list('subject', flat=True)),
+            ['Subj 2', 'Subj 0', None, None]
+        )
+
+    def test_order_by_on_list_field(self):
+        # Test order_by() on list fields where some values are empty
+        test_items = []
+        for i in range(4):
+            item = self.get_test_item()
+            item.subject = self.categories[0]  # Make sure we have something unique to filter on
+            if i % 2 == 0:
+                item.categories = ['Cat %s' % i]
+            else:
+                item.categories = []
+            test_items.append(item)
+        self.test_folder.bulk_create(items=test_items)
+        qs = QuerySet(
+            folder_collection=FolderCollection(account=self.account, folders=[self.test_folder])
+        ).filter(subject=self.categories[0])
+        self.assertEqual(
+            list(qs.order_by('categories').values_list('categories', flat=True)),
+            [None, None, ['Cat 0'], ['Cat 2']]
+        )
+        self.assertEqual(
+            list(qs.order_by('-categories').values_list('categories', flat=True)),
+            [['Cat 2'], ['Cat 0'], None, None]
+        )
+
     def test_finditems(self):
-        now = UTC_NOW()
+        now = datetime.datetime.now(tz=zoneinfo.ZoneInfo('UTC'))
 
         # Test argument types
         item = self.get_test_item()
@@ -573,10 +634,14 @@ class GenericItemTest(CommonItemTest):
         item.save()
         # For some reason, the querystring search doesn't work instantly. We may have to wait for up to 60 seconds.
         # I'm too impatient for that, so also allow empty results. This makes the test almost worthless but I blame EWS.
-        self.assertIn(
-            self.test_folder.filter('Subject:%s' % item.subject).count(),
-            (0, 1)
-        )
+        # Also, some servers are misconfigured and don't support querystrings at all. Don't fail on that.
+        try:
+            self.assertIn(
+                self.test_folder.filter('Subject:%s' % item.subject).count(),
+                (0, 1)
+            )
+        except ErrorInternalServerError as e:
+            self.assertIn('AQS parser has been removed from Windows 2016 Server Core', e.args[0])
 
     def test_complex_fields(self):
         # Test that complex fields can be fetched using only(). This is a test for #141.
@@ -612,7 +677,7 @@ class GenericItemTest(CommonItemTest):
 
     def test_text_body(self):
         if self.account.version.build < EXCHANGE_2013:
-            raise self.skipTest('Exchange version too old')
+            self.skipTest('Exchange version too old')
         item = self.get_test_item()
         item.body = 'X' * 500  # Make body longer than the normal 256 char text field limit
         item.save()
@@ -634,7 +699,7 @@ class GenericItemTest(CommonItemTest):
                 if f.name == 'reminder_due_by' and not item.reminder_is_set:
                     # We delete the due date if reminder is not set
                     continue
-                elif f.is_read_only:
+                if f.is_read_only:
                     continue
                 self.assertIsNotNone(getattr(item, f.name), (f, getattr(item, f.name)))
         only_fields = ('subject', 'body', 'categories')
@@ -670,10 +735,10 @@ class GenericItemTest(CommonItemTest):
         for result in export_results:
             self.assertIsInstance(result, str)
 
-        # Try reuploading our results
-        upload_results = self.account.upload([(self.test_folder, data) for data in export_results])
-        self.assertEqual(len(items), len(upload_results), (items, upload_results))
-        for result in upload_results:
+        # Try re-uploading our results (insert)
+        insert_results = self.account.upload([(self.test_folder, data) for data in export_results])
+        self.assertEqual(len(items), len(insert_results), (items, insert_results))
+        for result in insert_results:
             # Must be a completely new ItemId
             self.assertIsInstance(result, tuple)
             self.assertNotIn(result, ids)
@@ -699,10 +764,23 @@ class GenericItemTest(CommonItemTest):
                         a.attachment_id = None
             return dict_item
 
-        uploaded_items = sorted([to_dict(item) for item in self.account.fetch(upload_results)],
-                                key=lambda i: i['subject'])
-        original_items = sorted([to_dict(item) for item in items], key=lambda i: i['subject'])
-        self.assertListEqual(original_items, uploaded_items)
+        inserted_items = list(self.account.fetch(insert_results))
+        uploaded_data = sorted([to_dict(item) for item in inserted_items], key=lambda i: i['subject'])
+        original_data = sorted([to_dict(item) for item in items], key=lambda i: i['subject'])
+        self.assertListEqual(original_data, uploaded_data)
+
+        # Test update instead of insert
+        update_results = self.account.upload([
+            (self.test_folder, ((i.id, i.changekey), i.is_associated, data))
+            for i, data in zip(inserted_items, export_results)
+        ])
+        self.assertEqual(len(export_results), len(update_results), (export_results, update_results))
+        for i, result in zip(inserted_items, update_results):
+            # Must be a completely new ItemId
+            self.assertIsInstance(result, tuple)
+            item_id, changekey = result
+            self.assertEqual(i.id, item_id)
+            self.assertNotEqual(i.changekey, changekey)
 
     def test_export_with_error(self):
         # 15 new items which we will attempt to export and re-upload
@@ -725,6 +803,11 @@ class GenericItemTest(CommonItemTest):
         # Clean up after yourself
         del ids[3]  # Sending the deleted one through will cause an error
 
+    def test_archive(self):
+        item = self.get_test_item(folder=self.test_folder).save()
+        item_id, changekey = item.archive(to_folder=self.account.trash)
+        self.account.root.get(id=item_id, changekey=changekey)
+
     def test_item_attachments(self):
         item = self.get_test_item(folder=self.test_folder)
         item.attachments = []
@@ -737,7 +820,7 @@ class GenericItemTest(CommonItemTest):
 
         self.assertEqual(len(item.attachments), 1)
         item.save()
-        fresh_item = list(self.account.fetch(ids=[item]))[0]
+        fresh_item = self.get_item_by_id(item)
         self.assertEqual(len(fresh_item.attachments), 1)
         fresh_attachments = sorted(fresh_item.attachments, key=lambda a: a.name)
         self.assertEqual(fresh_attachments[0].name, 'attachment1')
@@ -777,7 +860,7 @@ class GenericItemTest(CommonItemTest):
         item.attach(attachment2)
 
         self.assertEqual(len(item.attachments), 2)
-        fresh_item = list(self.account.fetch(ids=[item]))[0]
+        fresh_item = self.get_item_by_id(item)
         self.assertEqual(len(fresh_item.attachments), 2)
         fresh_attachments = sorted(fresh_item.attachments, key=lambda a: a.name)
         self.assertEqual(fresh_attachments[0].name, 'attachment1')
@@ -841,7 +924,7 @@ class GenericItemTest(CommonItemTest):
         item.detach(attachment2)
         self.assertTrue(attachment2.attachment_id is None)
         self.assertTrue(attachment2.parent_item is None)
-        fresh_item = list(self.account.fetch(ids=[item]))[0]
+        fresh_item = self.get_item_by_id(item)
         self.assertEqual(len(fresh_item.attachments), 1)
         fresh_attachments = sorted(fresh_item.attachments, key=lambda a: a.name)
 
