@@ -1,4 +1,5 @@
 import time
+import warnings
 
 from exchangelib.folders import Inbox, FolderCollection
 from exchangelib.items import Message, SHALLOW, ASSOCIATED
@@ -24,7 +25,7 @@ class ItemQuerySetTest(BaseItemTest):
         ).filter(categories__contains=self.categories)
         test_cat = self.categories[0]
         self.assertEqual(
-            set((i.subject, i.categories[0]) for i in qs),
+            {(i.subject, i.categories[0]) for i in qs},
             {('Item 0', test_cat), ('Item 1', test_cat), ('Item 2', test_cat), ('Item 3', test_cat)}
         )
         self.assertEqual(
@@ -36,11 +37,11 @@ class ItemQuerySetTest(BaseItemTest):
             [('Item 2', test_cat)]
         )
         self.assertEqual(
-            set((i.subject, i.categories[0]) for i in qs.exclude(subject__startswith='Item 2')),
+            {(i.subject, i.categories[0]) for i in qs.exclude(subject__startswith='Item 2')},
             {('Item 0', test_cat), ('Item 1', test_cat), ('Item 3', test_cat)}
         )
         self.assertEqual(
-            set((i.subject, i.categories) for i in qs.only('subject')),
+            {(i.subject, i.categories) for i in qs.only('subject')},
             {('Item 0', None), ('Item 1', None), ('Item 2', None), ('Item 3', None)}
         )
         self.assertEqual(
@@ -62,7 +63,7 @@ class ItemQuerySetTest(BaseItemTest):
         with self.assertRaises(ValueError):
             list(qs.values([]))
         self.assertEqual(
-            [i for i in qs.order_by('subject').values('subject')],
+            list(qs.order_by('subject').values('subject')),
             [{'subject': 'Item 0'}, {'subject': 'Item 1'}, {'subject': 'Item 2'}, {'subject': 'Item 3'}]
         )
 
@@ -81,7 +82,7 @@ class ItemQuerySetTest(BaseItemTest):
         )
 
         self.assertEqual(
-            set(i for i in qs.values_list('subject')),
+            set(qs.values_list('subject')),
             {('Item 0',), ('Item 1',), ('Item 2',), ('Item 3',)}
         )
 
@@ -100,21 +101,21 @@ class ItemQuerySetTest(BaseItemTest):
         )
 
         self.assertEqual(
-            set(i.subject for i in qs.only('subject')),
+            {i.subject for i in qs.only('subject')},
             {'Item 0', 'Item 1', 'Item 2', 'Item 3'}
         )
 
         # Test .only() in combinations of 'id' and 'changekey', which are handled specially
         self.assertEqual(
-            list((i.id,) for i in qs.order_by('subject').only('id')),
+            [(i.id,) for i in qs.order_by('subject').only('id')],
             [(i.id,) for i in test_items]
         )
         self.assertEqual(
-            list((i.changekey,) for i in qs.order_by('subject').only('changekey')),
+            [(i.changekey,) for i in qs.order_by('subject').only('changekey')],
             [(i.changekey,) for i in test_items]
         )
         self.assertEqual(
-            list((i.id, i.changekey) for i in qs.order_by('subject').only('id', 'changekey')),
+            [(i.id, i.changekey) for i in qs.order_by('subject').only('id', 'changekey')],
             [(i.id, i.changekey) for i in test_items]
         )
 
@@ -131,7 +132,7 @@ class ItemQuerySetTest(BaseItemTest):
             [i.changekey for i in test_items]
         )
         self.assertEqual(
-            set(i for i in qs.values_list('subject', flat=True)),
+            set(qs.values_list('subject', flat=True)),
             {'Item 0', 'Item 1', 'Item 2', 'Item 3'}
         )
         self.assertEqual(
@@ -139,7 +140,7 @@ class ItemQuerySetTest(BaseItemTest):
             'Item 2'
         )
         self.assertEqual(
-            set((i.subject, i.categories[0]) for i in qs.exclude(subject__startswith='Item 2')),
+            {(i.subject, i.categories[0]) for i in qs.exclude(subject__startswith='Item 2')},
             {('Item 0', test_cat), ('Item 1', test_cat), ('Item 3', test_cat)}
         )
         # Test that we can sort on a field that we don't want
@@ -148,15 +149,18 @@ class ItemQuerySetTest(BaseItemTest):
             [test_cat, test_cat, test_cat, test_cat]
         )
         # Test iterator
-        self.assertEqual(
-            set((i.subject, i.categories[0]) for i in qs.iterator()),
-            {('Item 0', test_cat), ('Item 1', test_cat), ('Item 2', test_cat), ('Item 3', test_cat)}
-        )
-        # Test that iterator() preserves the result format
-        self.assertEqual(
-            set((i[0], i[1][0]) for i in qs.values_list('subject', 'categories').iterator()),
-            {('Item 0', test_cat), ('Item 1', test_cat), ('Item 2', test_cat), ('Item 3', test_cat)}
-        )
+        with warnings.catch_warnings():
+            # iterator() is deprecated but we still want to test it. Silence the DeprecationWarning
+            warnings.simplefilter("ignore")
+            self.assertEqual(
+                {(i.subject, i.categories[0]) for i in qs.iterator()},
+                {('Item 0', test_cat), ('Item 1', test_cat), ('Item 2', test_cat), ('Item 3', test_cat)}
+            )
+            # Test that iterator() preserves the result format
+            self.assertEqual(
+                {(i[0], i[1][0]) for i in qs.values_list('subject', 'categories').iterator()},
+                {('Item 0', test_cat), ('Item 1', test_cat), ('Item 2', test_cat), ('Item 3', test_cat)}
+            )
         self.assertEqual(qs.get(subject='Item 3').subject, 'Item 3')
         with self.assertRaises(DoesNotExist):
             qs.get(subject='Item XXX')
@@ -203,10 +207,6 @@ class ItemQuerySetTest(BaseItemTest):
         qs = QuerySet(
             folder_collection=FolderCollection(account=self.account, folders=[self.test_folder])
         ).filter(categories__contains=self.categories).order_by('subject')
-        for _ in qs:
-            # Build up the cache
-            pass
-        self.assertEqual(len(qs._cache), 4)
         with self.assertRaises(MultipleObjectsReturned):
             qs.get()  # Get with a full cache
         self.assertEqual(qs[2].subject, 'Item 2')  # Index with a full cache
@@ -267,7 +267,7 @@ class ItemQuerySetTest(BaseItemTest):
         self.test_folder.bulk_create(items=items)
         ids = self.test_folder.filter(categories__contains=self.categories).values_list('id', 'changekey')
         ids.page_size = 10
-        self.bulk_delete(ids.iterator())
+        self.bulk_delete(ids)
 
     def test_slicing(self):
         # Test that slicing works correctly
@@ -277,7 +277,7 @@ class ItemQuerySetTest(BaseItemTest):
             item.subject = 'Subj %s' % i
             del item.attachments[:]
             items.append(item)
-        ids = self.test_folder.bulk_create(items=items)
+        self.test_folder.bulk_create(items=items)
         qs = self.test_folder.filter(categories__contains=self.categories).only('subject').order_by('subject')
 
         # Test positive index
@@ -399,6 +399,34 @@ class ItemQuerySetTest(BaseItemTest):
         self.assertEqual(qs.count(), 0)
         self.assertEqual(to_folder_qs.count(), 1)
 
+    def test_mark_as_junk_via_queryset(self):
+        self.get_test_item().save()
+        qs = self.test_folder.filter(categories__contains=self.categories)
+        qs.mark_as_junk(is_junk=False, move_item=False)
+        self.assertEqual(self.test_folder.filter(categories__contains=self.categories).count(), 1)
+        qs.mark_as_junk(is_junk=True, move_item=False)
+        self.assertEqual(self.test_folder.filter(categories__contains=self.categories).count(), 1)
+        qs.mark_as_junk(is_junk=True, move_item=True)
+        self.assertEqual(self.account.junk.filter(categories__contains=self.categories).count(), 1)
+        self.account.junk.filter(categories__contains=self.categories).mark_as_junk(is_junk=False, move_item=True)
+        self.assertEqual(self.account.inbox.filter(categories__contains=self.categories).count(), 1)
+
+    def test_archive_via_queryset(self):
+        self.get_test_item().save()
+        qs = self.test_folder.filter(categories__contains=self.categories)
+        to_folder = self.account.trash
+        qs.archive(to_folder=to_folder)
+        self.assertEqual(qs.count(), 0)
+        self.assertEqual(to_folder.filter(categories__contains=self.categories).count(), 1)
+
     def test_depth(self):
         self.assertGreaterEqual(self.test_folder.all().depth(ASSOCIATED).count(), 0)
         self.assertGreaterEqual(self.test_folder.all().depth(SHALLOW).count(), 0)
+
+    def test_empty_in_sequence(self):
+        # 'Q(foo_in=[])' should always match nothing. We interpret it to mean "is foo contained in the empty set?"
+        # which is always false.
+        item = self.get_test_item().save()
+        qs = self.test_folder.filter(categories__contains=self.categories)
+        self.assertEqual(qs.filter(subject__in=[item.subject]).count(), 1)
+        self.assertEqual(qs.filter(subject__in=[]).count(), 0)
